@@ -1,7 +1,7 @@
 /* capture.c
  * Routines for packet capture windows
  *
- * $Id: capture.c,v 1.83 1999/11/29 01:54:00 guy Exp $
+ * $Id: capture.c,v 1.86 1999/12/09 07:19:02 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -90,6 +90,7 @@
 #include "gtk/menu.h"
 #include "capture.h"
 #include "util.h"
+#include "ui_util.h"
 #include "prefs.h"
 #include "globals.h"
 
@@ -120,7 +121,7 @@ void
 do_capture(char *capfile_name)
 {
   char tmpname[128+1];
-  gboolean is_temp_file;
+  gboolean is_tempfile;
   u_char c;
   int i;
   guint byte_count;
@@ -131,12 +132,12 @@ do_capture(char *capfile_name)
   if (capfile_name != NULL) {
     /* Try to open/create the specified file for use as a capture buffer. */
     cf.save_file_fd = open(capfile_name, O_RDWR|O_TRUNC|O_CREAT, 0600);
-    is_temp_file = FALSE;
+    is_tempfile = FALSE;
   } else {
     /* Choose a random name for the capture buffer */
     cf.save_file_fd = create_tempfile(tmpname, sizeof tmpname, "ether");
     capfile_name = g_strdup(tmpname);
-    is_temp_file = TRUE;
+    is_tempfile = TRUE;
   }
   if (cf.save_file_fd == -1) {
     simple_dialog(ESD_TYPE_WARN, NULL,
@@ -144,15 +145,9 @@ do_capture(char *capfile_name)
 	"could not be opened: %s.", capfile_name, strerror(errno));
     return;
   }
-  close_cap_file(&cf, info_bar, file_ctx);
-  if (cf.save_file != NULL) {
-    /* If the current file is a temporary capture file, remove it. */
-    if (!cf.user_saved)
-      unlink(cf.save_file); /* silently ignore error */
-    g_free(cf.save_file);
-  }
+  close_cap_file(&cf, info_bar);
+  g_assert(cf.save_file == NULL);
   cf.save_file = capfile_name;
-  cf.user_saved = !is_temp_file;
 
   if (sync_mode) {	/*  use fork() for capture */
     int  fork_child;
@@ -192,7 +187,6 @@ do_capture(char *capfile_name)
     } else {
       /* Parent process - read messages from the child process over the
          sync pipe. */
-      cf.filename = cf.save_file;
       close(sync_pipe[1]);
 
       /* Read a byte count from "sync_pipe[0]", terminated with a
@@ -210,6 +204,7 @@ do_capture(char *capfile_name)
 	     XXX - reap the child process and report the status in detail. */
 	  close(sync_pipe[0]);
 	  unlink(cf.save_file);
+	  g_free(cf.save_file);
 	  cf.save_file = NULL;
 	  simple_dialog(ESD_TYPE_WARN, NULL, "Capture child process died");
 	  return;
@@ -222,6 +217,7 @@ do_capture(char *capfile_name)
 	     and report the failure. */
 	  close(sync_pipe[0]);
 	  unlink(cf.save_file);
+	  g_free(cf.save_file);
 	  cf.save_file = NULL;
 	  simple_dialog(ESD_TYPE_WARN, NULL,
 	     "Capture child process sent us a bad message");
@@ -231,7 +227,7 @@ do_capture(char *capfile_name)
       }
       if (byte_count == 0) {
 	/* Success.  Open the capture file, and set up to read it. */
-	err = start_tail_cap_file(cf.save_file, &cf);
+	err = start_tail_cap_file(cf.save_file, is_tempfile, &cf);
 	if (err == 0) {
 	  /* We were able to open and set up to read the capture file;
 	     arrange that our callback be called whenever it's possible
@@ -254,6 +250,7 @@ do_capture(char *capfile_name)
 
 	  /* Don't unlink the save file - leave it around, for debugging
 	     purposes. */
+	  g_free(cf.save_file);
 	  cf.save_file = NULL;
 	}
       } else {
@@ -281,6 +278,7 @@ do_capture(char *capfile_name)
 
 	  /* Get rid of the save file - the capture never started. */
 	  unlink(cf.save_file);
+	  g_free(cf.save_file);
 	  cf.save_file = NULL;
 	}
       }
@@ -294,14 +292,16 @@ do_capture(char *capfile_name)
     }
     if (capture_succeeded) {
       /* Capture succeeded; read in the capture file. */
-      if ((err = open_cap_file(cf.save_file, &cf)) == 0) {
+      if ((err = open_cap_file(cf.save_file, is_tempfile, &cf)) == 0) {
         /* Set the read filter to NULL. */
         cf.rfcode = NULL;
         err = read_cap_file(&cf);
-        set_menu_sensitivity("/File/Save", TRUE);
-        set_menu_sensitivity("/File/Save As...", FALSE);
       }
     }
+    /* We're not doing a capture any more, so we don't have a save
+       file. */
+    g_free(cf.save_file);
+    cf.save_file = NULL;
   }
 }
 
@@ -432,6 +432,12 @@ cap_file_input_cb(gpointer data, gint source, GdkInputCondition condition)
     /* Read what remains of the capture file, and finish the capture.
        XXX - do something if this fails? */
     err = finish_tail_cap_file(cf);
+
+    /* We're not doing a capture any more, so we don't have a save
+       file. */
+    g_free(cf->save_file);
+    cf->save_file = NULL;
+
     return;
   }
 
@@ -475,7 +481,7 @@ int
 capture(void)
 {
   GtkWidget  *cap_w, *main_vb, *count_lb, *tcp_lb, *udp_lb, *icmp_lb,
-             *ospf_lb, *gre_lb, *netbios_lb, *other_lb, *stop_bt;
+             *ospf_lb, *gre_lb, *netbios_lb, *ipx_lb, *other_lb, *stop_bt;
   pcap_t     *pch;
   gchar       err_str[PCAP_ERRBUF_SIZE], label_str[32];
   loop_data   ld;
@@ -494,6 +500,7 @@ capture(void)
   ld.counts.icmp    = 0;
   ld.counts.ospf    = 0;
   ld.counts.gre     = 0;
+  ld.counts.ipx     = 0;
   ld.counts.netbios = 0;
   ld.counts.other   = 0;
   ld.pdh            = NULL;
@@ -625,6 +632,10 @@ capture(void)
   gtk_box_pack_start(GTK_BOX(main_vb), netbios_lb, FALSE, FALSE, 3);
   gtk_widget_show(netbios_lb);
 
+  ipx_lb = gtk_label_new("IPX: 0 (0.0%)");
+  gtk_box_pack_start(GTK_BOX(main_vb), ipx_lb, FALSE, FALSE, 3);
+  gtk_widget_show(ipx_lb);
+
   other_lb = gtk_label_new("Other: 0 (0.0%)");
   gtk_box_pack_start(GTK_BOX(main_vb), other_lb, FALSE, FALSE, 3);
   gtk_widget_show(other_lb);
@@ -680,6 +691,10 @@ capture(void)
 		pct(ld.counts.netbios, ld.counts.total));
       gtk_label_set(GTK_LABEL(netbios_lb), label_str);
 
+      sprintf(label_str, "IPX: %d (%.1f%%)", ld.counts.ipx,
+		pct(ld.counts.ipx, ld.counts.total));
+      gtk_label_set(GTK_LABEL(ipx_lb), label_str);
+
       sprintf(label_str, "Other: %d (%.1f%%)", ld.counts.other,
 		pct(ld.counts.other, ld.counts.total));
       gtk_label_set(GTK_LABEL(other_lb), label_str);
@@ -738,6 +753,8 @@ error:
   /* We couldn't even start the capture, so get rid of the capture
      file. */
   unlink(cf.save_file); /* silently ignore error */
+  g_free(cf.save_file);
+  cf.save_file = NULL;
   if (capture_child) {
     /* This is the child process for a sync mode capture.
        Send the error message to our parent, so they can display a
