@@ -1,7 +1,7 @@
 /* capture.h
  * Definitions for packet capture windows
  *
- * $Id: capture.h,v 1.15 1999/09/23 06:27:19 guy Exp $
+ * $Id: capture.h,v 1.21 1999/10/02 19:57:24 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -28,28 +28,20 @@
 
 #ifdef HAVE_LIBPCAP
 
-#ifndef lib_pcap_h
-#include <pcap.h>
-#endif
+/* Name we give to the child process when doing a "-S" capture. */
+#define	CHILD_NAME	"ethereal-capture"
 
-/* The version of pcap.h that comes with some systems is missing these
- * #defines.
- */
+extern int sync_mode;	/* fork a child to do the capture, and sync between them */
+extern int sync_pipe[2]; /* used to sync father */
+extern int quit_after_cap; /* Makes a "capture only mode". Implies -k */
+extern gboolean capture_child;	/* if this is the child for "-S" */
 
-#ifndef DLT_RAW
-#define DLT_RAW 12
-#endif
+/* Open a specified file, or create a temporary file, and start a capture
+   to the file in question. */
+void   do_capture(char *capfile_name);
 
-#ifndef DLT_SLIP_BSDOS
-#define DLT_SLIP_BSDOS 13
-#endif
-
-#ifndef DLT_PPP_BSDOS
-#define DLT_PPP_BSDOS 14
-#endif
-
-void   do_capture(void);
-void   capture(void);
+/* Do the low-level work of a capture. */
+int    capture(void);
 
 #endif /* HAVE_LIBPCAP */
 #endif /* capture.h */
