@@ -1,7 +1,7 @@
 /* column.h
  * Definitions for column handling routines
  *
- * $Id: column.h,v 1.1 1998/11/17 04:28:41 gerald Exp $
+ * $Id: column.h,v 1.6 1999/09/12 06:11:34 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -31,14 +31,23 @@ typedef struct _fmt_data {
   gchar *fmt;
 } fmt_data;
 
-gint       get_column_format(gint);
-gchar     *get_column_title(gint);
-gchar     *col_format_to_pref_str();
-void       get_column_format_matches(gboolean *, gint);
-gint       get_column_width(gint format, GdkFont *font);
-GtkWidget *column_prefs_show();
-void       column_prefs_ok(GtkWidget *);
-void       column_prefs_save(GtkWidget *);
-void       column_prefs_cancel(GtkWidget *);
+enum col_resize_type {
+  RESIZE_AUTO,          /* Automatically resize */
+  RESIZE_LIVE,          /* Automatically resize even during live capture */
+  RESIZE_MANUAL         /* Don't automatically resize */
+};
+
+gint                 get_column_format(gint);
+enum col_resize_type get_column_resize_type(gint);
+gchar               *get_column_title(gint);
+gchar               *col_format_to_pref_str(void);
+void                 get_column_format_matches(gboolean *, gint);
+gint                 get_column_width(gint format, GdkFont *font);
+gint                 get_column_char_width(gint format);
+GtkWidget           *column_prefs_show(void);
+void                 column_prefs_ok(GtkWidget *);
+void                 column_prefs_save(GtkWidget *);
+void                 column_prefs_cancel(GtkWidget *);
+void                 column_prefs_delete(GtkWidget *);
 
 #endif /* column.h */
