@@ -1,7 +1,7 @@
 /* dfilter.c
  * Routines for display filters
  *
- * $Id: dfilter.c,v 1.19 1999/08/30 16:01:42 gram Exp $
+ * $Id: dfilter.c,v 1.32 1999/11/15 06:32:13 gram Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -94,8 +94,6 @@ static GArray* get_values_from_ptree(dfilter_node *dnode, proto_tree *ptree, con
 static GArray* get_values_from_dfilter(dfilter_node *dnode, GNode *gnode);
 static gboolean check_existence_in_ptree(dfilter_node *dnode, proto_tree *ptree);
 static void clear_byte_array(gpointer data, gpointer user_data);
-static void unlink_gnode_children(gpointer gnode_ptr, gpointer user_data);
-static void destroy_gnode(gpointer gnode_ptr, gpointer user_data);
 
 /* this is not so pretty. I need my own g_array "function" (macro) to
  * retreive the pointer to the data stored in an array cell. I need this
@@ -136,27 +134,32 @@ dfilter_cleanup(void)
  * dfilter_compile will take care of that automatically.
  * 
  * Returns 0 on success, non-zero on failure.
- * If a failure, dfilter_error_msg points to an appropriate error message.
+ *
+ * On success, sets the "dfilter *" pointed to by its second argument
+ * either to a null pointer (if the filter is a null filter, as
+ * generated by an all-blank string) or to a pointer to a newly-allocated
+ * dfilter structure (if the filter isn't null).
+ *
+ * On failure, "dfilter_error_msg" points to an appropriate error message.
  * This error message is a global string, so another invocation of
  * dfilter_compile will clear it. If the caller needs is stored, he
  * needs to g_strdup it himself.
  */
 int
-dfilter_compile(dfilter *df, gchar *dfilter_text)
+dfilter_compile(gchar *dfilter_text, dfilter **dfp)
 {
+	dfilter *df;
 	int retval;
 
 	g_assert(dfilter_text != NULL);
 
-	dfilter_clear_filter(df);
-	df->dftext = g_strdup(dfilter_text);
+	df = dfilter_new();
 
-	/* tell the scanner to use this string as input */
-	dfilter_scanner_text(df->dftext);
+	/* tell the scanner to use the filter string as input */
+	dfilter_scanner_text(dfilter_text);
 
 	/* Assign global variable so dfilter_parse knows which dfilter we're
-	 * talking about. Reset the global error message. We don't have to set
-	 * gnode_slist since it will always be NULL by the time we get here.
+	 * talking about. Reset the global error message.
 	 */
 	global_df = df;
 	dfilter_error_msg = NULL;
@@ -167,55 +170,43 @@ dfilter_compile(dfilter *df, gchar *dfilter_text)
 	/* clean up lex */
 	dfilter_scanner_cleanup();
 
-	/* If a parse error occurred, fill in a generic error message
-	 * if one was not created during parsing. */
+	/* Errors not found by the parser may not cause the parse to
+	 * fail; if "dfilter_error_msg" is set, it means somebody
+	 * else called "dfilter_fail()", e.g. the lexical analyzer,
+	 * so treat that as a parse error. */
+	if (dfilter_error_msg != NULL)
+		retval = 1;
+
 	if (retval != 0) {
 		if (dfilter_error_msg == NULL) {
-			dfilter_error_msg = &dfilter_error_msg_buf[0];
-			snprintf(dfilter_error_msg, sizeof(dfilter_error_msg_buf),
+			snprintf(dfilter_error_msg_buf, sizeof(dfilter_error_msg_buf),
 				"Unable to parse filter string \"%s\".",
 				dfilter_text);
+			dfilter_error_msg = &dfilter_error_msg_buf[0];
 		}
 	}
 
-	/* Clear the list of allocated nodes */
-	if (gnode_slist) {
-		g_slist_free(gnode_slist);
-		gnode_slist = NULL;
-	}
+	/* Set global_df to NULL just to be tidy. */
+	global_df = NULL;
 
+	if (retval == 0) {
+		/* Success.  Check if the filter is empty; if so, discard
+		 * it and set "*dfp" to NULL, otherwise set "*dfp" to
+		 * point to the filter. */
+		if (df->dftree == NULL) {
+			/* The filter is empty. */
+			dfilter_destroy(df);
+			df = NULL;
+		}
+		*dfp = df;
+	} else {
+		/* Failure.  Destroy the filter. */
+		dfilter_destroy(df);
+		df = NULL;
+	}
 	return retval;
 }
 
-/* clear the current filter, w/o clearing memchunk area which is where we'll
- * put new nodes in a future filter */
-void
-dfilter_clear_filter(dfilter *df)
-{
-	if (!df)
-		return;
-
-	if (df->dftext)
-		g_free(df->dftext);
-
-	if (df->dftree != NULL)
-		g_node_destroy(df->dftree);
-
-	/* clear the memory that the tree was using for nodes */
-	if (df->node_memchunk)
-		g_mem_chunk_reset(df->node_memchunk);
-
-	/* clear the memory that the tree was using for byte arrays */
-	if (df->list_of_byte_arrays) {
-		g_slist_foreach(df->list_of_byte_arrays, clear_byte_array, NULL);
-		g_slist_free(df->list_of_byte_arrays);
-	}
-
-	df->dftext = NULL;
-	df->dftree = NULL;
-	df->list_of_byte_arrays = NULL;
-}
-
 /* Allocates new dfilter, initializes values, and returns pointer to dfilter */
 dfilter*
 dfilter_new(void)
@@ -224,7 +215,6 @@ dfilter_new(void)
 
 	df = g_malloc(sizeof(dfilter));
 
-	df->dftext = NULL;
 	df->dftree = NULL;
 	df->node_memchunk = g_mem_chunk_new("df->node_memchunk",
 		sizeof(dfilter_node), 20 * sizeof(dfilter_node), G_ALLOC_ONLY);
@@ -240,7 +230,21 @@ dfilter_destroy(dfilter *df)
 	if (!df)
 		return;
 
-	dfilter_clear_filter(df);
+	if (df->dftree != NULL)
+		g_node_destroy(df->dftree);
+
+	/* clear the memory that the tree was using for nodes */
+	if (df->node_memchunk)
+		g_mem_chunk_reset(df->node_memchunk);
+
+	/* clear the memory that the tree was using for byte arrays */
+	if (df->list_of_byte_arrays) {
+		g_slist_foreach(df->list_of_byte_arrays, clear_byte_array, NULL);
+		g_slist_free(df->list_of_byte_arrays);
+	}
+
+	df->dftree = NULL;
+	df->list_of_byte_arrays = NULL;
 
 	/* Git rid of memchunk */
 	if (df->node_memchunk)
@@ -262,49 +266,27 @@ clear_byte_array(gpointer data, gpointer user_data)
 void
 dfilter_error(char *s)
 {
-	/* The only data we have to worry about freeing is the
-	 * data used by any GNodes that were allocated during
-	 * parsing. The data in those Gnodes will be cleared
-	 * later via df->node_memchunk. Use gnode_slist to
-	 * clear the GNodes, and set global_df to NULL just
-	 * to be tidy.
-	 */
-	global_df = NULL;
+}
 
-	/* I don't want to call g_node_destroy on each GNode ptr,
-	 * since that function frees any children. That could
-	 * mess me up later in the list if I try to free a GNode
-	 * that has already been freed. So, I'll unlink the
-	 * children firs,t then call g_node_destroy on each GNode ptr.
-	 */
-	if (!gnode_slist)
+/* Called when an error other than a parsing error occurs. */
+void
+dfilter_fail(char *format, ...)
+{
+	va_list    ap;
+
+	/* If we've already reported one error, don't overwrite it with this
+	 * one. */
+	if (dfilter_error_msg != NULL)
 		return;
 
-	g_slist_foreach(gnode_slist, unlink_gnode_children, NULL);
-	g_slist_foreach(gnode_slist, destroy_gnode, NULL);
-
-	/* notice we don't clear gnode_slist itself. dfilter_compile()
-	 * will take care of that.
-	 */
+	va_start(ap, format);
+	vsnprintf(dfilter_error_msg_buf, sizeof dfilter_error_msg_buf, format, ap);
+	dfilter_error_msg = dfilter_error_msg_buf;
+	va_end(ap);
 }
 
-static void
-unlink_gnode_children(gpointer gnode_ptr, gpointer user_data)
-{
-	if (gnode_ptr)
-		g_node_unlink((GNode*) gnode_ptr);
-}
-
-static void
-destroy_gnode(gpointer gnode_ptr, gpointer user_data)
-{
-	if (gnode_ptr)
-		g_node_destroy((GNode*) gnode_ptr);
-}
-
-
 /* lookup an abbreviation in our token tree, returing the ID #
- * If the abbreviation doesn't exit, returns 0 */
+ * If the abbreviation doesn't exit, returns -1 */
 int dfilter_lookup_token(char *abbrev)
 {
 	int value;
@@ -313,7 +295,7 @@ int dfilter_lookup_token(char *abbrev)
 	value =  GPOINTER_TO_INT(g_tree_lookup(dfilter_tokens, abbrev));
 
 	if (value < DFILTER_LEX_ABBREV_OFFSET) {
-		return 0;
+		return -1;
 	}
 	return value - DFILTER_LEX_ABBREV_OFFSET;
 }
@@ -329,6 +311,8 @@ gboolean
 dfilter_apply(dfilter *dfcode, proto_tree *ptree, const guint8* pd)
 {
 	gboolean retval;
+	if (dfcode == NULL)
+		return FALSE;
 	retval = dfilter_apply_node(dfcode->dftree, ptree, pd);
 	return retval;
 }
@@ -351,6 +335,7 @@ dfilter_apply_node(GNode *gnode, proto_tree *ptree, const guint8* pd)
 		g_assert_not_reached();
 
 	case logical:
+		g_assert(gnode_a);
 		return check_logical(dnode->value.logical, gnode_a, gnode_b, ptree, pd);
 
 	case relation:
@@ -362,7 +347,9 @@ dfilter_apply_node(GNode *gnode, proto_tree *ptree, const guint8* pd)
 		/* not coded yet */
 	
 	case numeric:
+	case floating:
 	case ipv4:
+	case ipv6:
 	case boolean:
 	case ether:
 	case string:
@@ -392,10 +379,13 @@ check_logical(gint operand, GNode *a, GNode *b, proto_tree *ptree, const guint8
 
 	switch(operand) {
 	case TOK_AND:
+		g_assert(b);
 		return (val_a && dfilter_apply_node(b, ptree, pd));
 	case TOK_OR:
+		g_assert(b);
 		return (val_a || dfilter_apply_node(b, ptree, pd));
 	case TOK_XOR:
+		g_assert(b);
 		val_b = dfilter_apply_node(b, ptree, pd);
 		return ( ( val_a || val_b ) && ! ( val_a && val_b ) );
 	case TOK_NOT:
@@ -491,7 +481,6 @@ get_values_from_dfilter(dfilter_node *dnode, GNode *gnode)
 	array = g_array_new(FALSE, FALSE, dnode->elem_size);
 
 	g_node_traverse(gnode, G_IN_ORDER, G_TRAVERSE_ALL, -1, dnode->fill_array_func, array);
-/*	dnode->fill_array_func(gnode, array);*/
 	return array;
 }
 
@@ -507,6 +496,18 @@ gboolean fill_array_numeric_variable(GNode *gnode, gpointer data)
 	return FALSE; /* FALSE = do not end traversal of GNode tree */
 }
 
+gboolean fill_array_floating_variable(GNode *gnode, gpointer data)
+{
+	proto_tree_search_info	*sinfo = (proto_tree_search_info*)data;
+	field_info		*fi = (field_info*) (gnode->data);
+
+	if (fi->hfinfo->id == sinfo->target) {
+		g_array_append_val(sinfo->result.array, fi->value.floating);
+	}
+
+	return FALSE; /* FALSE = do not end traversal of GNode tree */
+}
+
 gboolean fill_array_ether_variable(GNode *gnode, gpointer data)
 {
 	proto_tree_search_info	*sinfo = (proto_tree_search_info*)data;
@@ -519,19 +520,64 @@ gboolean fill_array_ether_variable(GNode *gnode, gpointer data)
 	return FALSE; /* FALSE = do not end traversal of GNode tree */
 }
 
+gboolean fill_array_ipv4_variable(GNode *gnode, gpointer data)
+{
+	proto_tree_search_info	*sinfo = (proto_tree_search_info*)data;
+	field_info		*fi = (field_info*) (gnode->data);
+
+	if (fi->hfinfo->id == sinfo->target) {
+		g_array_append_val(sinfo->result.array, fi->value.ipv4);
+	}
+
+	return FALSE; /* FALSE = do not end traversal of GNode tree */
+}
+gboolean fill_array_ipv6_variable(GNode *gnode, gpointer data)
+{
+	proto_tree_search_info	*sinfo = (proto_tree_search_info*)data;
+	field_info		*fi = (field_info*) (gnode->data);
+
+	if (fi->hfinfo->id == sinfo->target) {
+		g_array_append_val(sinfo->result.array, fi->value.ipv6);
+	}
+
+	return FALSE; /* FALSE = do not end traversal of GNode tree */
+}
+
 gboolean fill_array_bytes_variable(GNode *gnode, gpointer data)
 {
 	proto_tree_search_info	*sinfo = (proto_tree_search_info*)data;
 	field_info		*fi = (field_info*) (gnode->data);
 	GByteArray		*barray;
+	guint			read_start, pkt_end;
 
 	if (fi->hfinfo->id == sinfo->target) {
+		if (bytes_offset < 0) {
+			/* Handle negative byte offsets */
+			bytes_offset = fi->length + bytes_offset;
+			if (bytes_offset < 0) {
+				goto FAIL;
+			}
+		}
+
+		/* Check to make sure offset exists for this field */
+		if (bytes_offset >= fi->length) {
+			goto FAIL;
+		}
+
+		pkt_end = fi->start + fi->length;
+		read_start = fi->start + bytes_offset;
+
+		/* Check to make sure entire length requested is inside field */
+		if (pkt_end < read_start + bytes_length) {
+			goto FAIL;
+		}
+
 		barray = g_byte_array_new();
-		/*list_of_byte_arrays = g_slist_append(list_of_byte_arrays, barray);*/
-		g_byte_array_append(barray, sinfo->packet_data + fi->start + bytes_offset, bytes_length);
+		g_byte_array_append(barray, sinfo->packet_data + read_start, bytes_length);
 		g_array_append_val(sinfo->result.array, barray);
 	}
 
+ FAIL:
 	return FALSE; /* FALSE = do not end traversal of GNode tree */
 }
 
@@ -544,6 +590,15 @@ gboolean fill_array_numeric_value(GNode *gnode, gpointer data)
 	return FALSE; /* FALSE = do not end traversal of GNode tree */
 }
 
+gboolean fill_array_floating_value(GNode *gnode, gpointer data)
+{
+	GArray		*array = (GArray*)data;
+	dfilter_node	*dnode = (dfilter_node*) (gnode->data);
+
+	g_array_append_val(array, dnode->value.floating);
+	return FALSE; /* FALSE = do not end traversal of GNode tree */
+}
+
 gboolean fill_array_ether_value(GNode *gnode, gpointer data)
 {
 	GArray		*array = (GArray*)data;
@@ -554,6 +609,26 @@ gboolean fill_array_ether_value(GNode *gnode, gpointer data)
 	return FALSE; /* FALSE = do not end traversal of GNode tree */
 }
 
+gboolean fill_array_ipv4_value(GNode *gnode, gpointer data)
+{
+	GArray		*array = (GArray*)data;
+	dfilter_node	*dnode = (dfilter_node*) (gnode->data);
+
+	g_array_append_val(array, dnode->value.ipv4);
+
+	return FALSE; /* FALSE = do not end traversal of GNode tree */
+}
+
+gboolean fill_array_ipv6_value(GNode *gnode, gpointer data)
+{
+	GArray		*array = (GArray*)data;
+	dfilter_node	*dnode = (dfilter_node*) (gnode->data);
+
+	g_array_append_val(array, dnode->value.ipv6);
+
+	return FALSE; /* FALSE = do not end traversal of GNode tree */
+}
+
 gboolean fill_array_bytes_value(GNode *gnode, gpointer data)
 {
 	GArray		*array = (GArray*)data;
@@ -643,6 +718,201 @@ gboolean check_relation_numeric(gint operand, GArray *a, GArray *b)
 	return FALSE;
 }
 
+gboolean check_relation_floating(gint operand, GArray *a, GArray *b)
+{
+	int	i, j, len_a, len_b;
+	double	val_a;
+
+	len_a = a->len;
+	len_b = b->len;
+
+
+	switch(operand) {
+	case TOK_EQ:
+		for(i = 0; i < len_a; i++) {
+			val_a = g_array_index(a, double, i);
+			for (j = 0; j < len_b; j++) {
+				if (val_a == g_array_index(b, double, j))
+					return TRUE;
+			}
+		}
+		return FALSE;
+
+	case TOK_NE:
+		for(i = 0; i < len_a; i++) {
+			val_a = g_array_index(a, double, i);
+			for (j = 0; j < len_b; j++) {
+				if (val_a != g_array_index(b, double, j))
+					return TRUE;
+			}
+		}
+		return FALSE;
+
+	case TOK_GT:
+		for(i = 0; i < len_a; i++) {
+			val_a = g_array_index(a, double, i);
+			for (j = 0; j < len_b; j++) {
+				if (val_a > g_array_index(b, double, j))
+					return TRUE;
+			}
+		}
+		return FALSE;
+
+	case TOK_GE:
+		for(i = 0; i < len_a; i++) {
+			val_a = g_array_index(a, double, i);
+			for (j = 0; j < len_b; j++) {
+				if (val_a >= g_array_index(b, double, j))
+					return TRUE;
+			}
+		}
+		return FALSE;
+
+	case TOK_LT:
+		for(i = 0; i < len_a; i++) {
+			val_a = g_array_index(a, double, i);
+			for (j = 0; j < len_b; j++) {
+				if (val_a < g_array_index(b, double, j))
+					return TRUE;
+			}
+		}
+		return FALSE;
+
+	case TOK_LE:
+		for(i = 0; i < len_a; i++) {
+			val_a = g_array_index(a, double, i);
+			for (j = 0; j < len_b; j++) {
+				if (val_a <= g_array_index(b, double, j))
+					return TRUE;
+			}
+		}
+		return FALSE;
+
+	default:
+		g_assert_not_reached();
+	}
+
+	g_assert_not_reached();
+	return FALSE;
+}
+
+gboolean check_relation_ipv4(gint operand, GArray *a, GArray *b)
+{
+	int		i, j, len_a, len_b;
+	ipv4_addr	*ptr_a, *ptr_b;
+
+	len_a = a->len;
+	len_b = b->len;
+
+
+	switch(operand) {
+	case TOK_EQ:
+		for(i = 0; i < len_a; i++) {
+			ptr_a = (ipv4_addr*) g_array_index_ptr(a, sizeof(ipv4_addr), i);
+			for (j = 0; j < len_b; j++) {
+				ptr_b = (ipv4_addr*) g_array_index_ptr(b, sizeof(ipv4_addr), j);
+				if (ipv4_addr_eq(ptr_a, ptr_b))
+					return TRUE;
+			}
+		}
+		return FALSE;
+
+	case TOK_NE:
+		for(i = 0; i < len_a; i++) {
+			ptr_a = (ipv4_addr*) g_array_index_ptr(a, sizeof(ipv4_addr), i);
+			for (j = 0; j < len_b; j++) {
+				ptr_b = (ipv4_addr*) g_array_index_ptr(b, sizeof(ipv4_addr), j);
+				if (!ipv4_addr_eq(ptr_a, ptr_b))
+					return TRUE;
+			}
+		}
+		return FALSE;
+
+	case TOK_GT:
+		for(i = 0; i < len_a; i++) {
+			ptr_a = (ipv4_addr*) g_array_index_ptr(a, sizeof(ipv4_addr), i);
+			for (j = 0; j < len_b; j++) {
+				ptr_b = (ipv4_addr*) g_array_index_ptr(b, sizeof(ipv4_addr), j);
+				if (ipv4_addr_gt(ptr_a, ptr_b))
+					return TRUE;
+			}
+		}
+		return FALSE;
+
+	case TOK_GE:
+		for(i = 0; i < len_a; i++) {
+			ptr_a = (ipv4_addr*) g_array_index_ptr(a, sizeof(ipv4_addr), i);
+			for (j = 0; j < len_b; j++) {
+				ptr_b = (ipv4_addr*) g_array_index_ptr(b, sizeof(ipv4_addr), j);
+				if (ipv4_addr_ge(ptr_a, ptr_b))
+					return TRUE;
+			}
+		}
+		return FALSE;
+
+	case TOK_LT:
+		for(i = 0; i < len_a; i++) {
+			ptr_a = (ipv4_addr*) g_array_index_ptr(a, sizeof(ipv4_addr), i);
+			for (j = 0; j < len_b; j++) {
+				ptr_b = (ipv4_addr*) g_array_index_ptr(b, sizeof(ipv4_addr), j);
+				if (ipv4_addr_lt(ptr_a, ptr_b))
+					return TRUE;
+			}
+		}
+		return FALSE;
+
+	case TOK_LE:
+		for(i = 0; i < len_a; i++) {
+			ptr_a = (ipv4_addr*) g_array_index_ptr(a, sizeof(ipv4_addr), i);
+			for (j = 0; j < len_b; j++) {
+				ptr_b = (ipv4_addr*) g_array_index_ptr(b, sizeof(ipv4_addr), j);
+				if (ipv4_addr_le(ptr_a, ptr_b))
+					return TRUE;
+			}
+		}
+		return FALSE;
+	}
+
+	g_assert_not_reached();
+	return FALSE;
+}
+
+gboolean check_relation_ipv6(gint operand, GArray *a, GArray *b)
+{
+	int	i, j, len_a, len_b;
+	guint8	*ptr_a, *ptr_b;
+
+	len_a = a->len;
+	len_b = b->len;
+
+
+	switch(operand) {
+	case TOK_EQ:
+		for(i = 0; i < len_a; i++) {
+			ptr_a = g_array_index_ptr(a, 16, i);
+			for (j = 0; j < len_b; j++) {
+				ptr_b = g_array_index_ptr(b, 16, j);
+				if (memcmp(ptr_a, ptr_b, 16) == 0)
+					return TRUE;
+			}
+		}
+		return FALSE;
+
+	case TOK_NE:
+		for(i = 0; i < len_a; i++) {
+			ptr_a = g_array_index_ptr(a, 16, i);
+			for (j = 0; j < len_b; j++) {
+				ptr_b = g_array_index_ptr(b, 16, j);
+				if (memcmp(ptr_a, ptr_b, 16) != 0)
+					return TRUE;
+			}
+		}
+		return FALSE;
+	}
+
+	g_assert_not_reached();
+	return FALSE;
+}
 
 gboolean check_relation_ether(gint operand, GArray *a, GArray *b)
 {
@@ -739,4 +1009,3 @@ gboolean check_relation_bytes(gint operand, GArray *a, GArray *b)
 	g_assert_not_reached();
 	return FALSE;
 }
-
