@@ -1,7 +1,7 @@
 /* file.c
  * File I/O routines
  *
- * $Id: file.c,v 1.125 1999/11/29 01:54:00 guy Exp $
+ * $Id: file.c,v 1.143 1999/12/29 20:09:45 gram Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -76,24 +76,16 @@
 #include "print.h"
 #include "file.h"
 #include "util.h"
+#include "ui_util.h"
 #include "gtk/proto_draw.h"
 #include "dfilter.h"
-#include "timestamp.h"
 #include "conversation.h"
 #include "globals.h"
 
-#ifndef __RESOLV_H__
-#include "resolv.h"
+#ifdef HAVE_DLFCN_H
+#include "plugins.h"
 #endif
 
-#include "packet-atalk.h"
-
-#include "packet-ipv6.h"
-
-#include "packet-sna.h"
-
-#include "packet-vines.h"
-
 extern GtkWidget *packet_list, *prog_bar, *info_bar, *byte_view, *tree_view;
 extern guint      file_ctx;
 
@@ -108,11 +100,15 @@ static void wtap_dispatch_cb(u_char *, const struct wtap_pkthdr *, int,
 static void freeze_clist(capture_file *cf);
 static void thaw_clist(capture_file *cf);
 
+static char *file_rename_error_message(int err);
+static char *file_close_error_message(int err);
+
 /* Update the progress bar this many times when reading a file. */
 #define N_PROGBAR_UPDATES	100
 
 int
-open_cap_file(char *fname, capture_file *cf) {
+open_cap_file(char *fname, gboolean is_tempfile, capture_file *cf)
+{
   wtap       *wth;
   int         err;
   FILE_T      fh;
@@ -134,7 +130,7 @@ open_cap_file(char *fname, capture_file *cf) {
 
   /* The open succeeded.  Close whatever capture file we had open,
      and fill in the information for this file. */
-  close_cap_file(cf, info_bar, file_ctx);
+  close_cap_file(cf, info_bar);
 
   /* Initialize the table of conversations. */
   conversation_init();
@@ -147,11 +143,18 @@ open_cap_file(char *fname, capture_file *cf) {
   cf->filed = fd;
   cf->f_len = cf_stat.st_size;
 
-  /* set the file name because we need it to set the follow stream filter */
+  /* Set the file name because we need it to set the follow stream filter.
+     XXX - is that still true?  We need it for other reasons, though,
+     in any case. */
   cf->filename = g_strdup(fname);
 
+  /* Indicate whether it's a permanent or temporary file. */
+  cf->is_tempfile = is_tempfile;
+
+  /* If it's a temporary capture buffer file, mark it as not saved. */
+  cf->user_saved = !is_tempfile;
+
   cf->cd_t      = wtap_file_type(cf->wth);
-  cf->cd_t_desc = wtap_file_type_string(cf->wth);
   cf->count     = 0;
   cf->drops     = 0;
   cf->esec      = 0;
@@ -173,7 +176,8 @@ fail:
 
 /* Reset everything to a pristine state */
 void
-close_cap_file(capture_file *cf, void *w, guint context) {
+close_cap_file(capture_file *cf, void *w)
+{
   frame_data *fd, *fd_next;
 
   if (cf->fh) {
@@ -184,6 +188,17 @@ close_cap_file(capture_file *cf, void *w, guint context) {
     wtap_close(cf->wth);
     cf->wth = NULL;
   }
+  /* We have no file open... */
+  if (cf->filename != NULL) {
+    /* If it's a temporary file, remove it. */
+    if (cf->is_tempfile)
+      unlink(cf->filename);
+    g_free(cf->filename);
+    cf->filename = NULL;
+  }
+  /* ...which means we have nothing to save. */
+  cf->user_saved = FALSE;
+
   for (fd = cf->plist; fd != NULL; fd = fd_next) {
     fd_next = fd->next;
     g_free(fd);
@@ -196,10 +211,18 @@ close_cap_file(capture_file *cf, void *w, guint context) {
   cf->plist_end = NULL;
   unselect_packet(cf);	/* nothing to select */
 
+  /* Clear the packet list. */
   gtk_clist_freeze(GTK_CLIST(packet_list));
   gtk_clist_clear(GTK_CLIST(packet_list));
   gtk_clist_thaw(GTK_CLIST(packet_list));
-  gtk_statusbar_pop(GTK_STATUSBAR(w), context);
+
+  /* Clear any file-related status bar messages.
+     XXX - should be "clear *ALL* file-related status bar messages;
+     will there ever be more than one on the stack? */
+  gtk_statusbar_pop(GTK_STATUSBAR(w), file_ctx);
+
+  /* Restore the standard title bar message. */
+  set_main_window_name("The Ethereal Network Analyzer");
 
   /* Disable all menu items that make sense only if you have a capture. */
   set_menu_sensitivity("/File/Save", FALSE);
@@ -219,15 +242,19 @@ close_cap_file(capture_file *cf, void *w, guint context) {
   set_menu_sensitivity("/Tools/Summary", FALSE);
 }
 
+/* Set the file name in the status line, in the name for the main window,
+   and in the name for the main window's icon. */
 static void
-set_statusbar_filename(capture_file *cf)
+set_display_filename(capture_file *cf)
 {
   gchar  *name_ptr;
   size_t  msg_len;
   gchar  *done_fmt = " File: %s  Drops: %u";
   gchar  *done_msg;
+  gchar  *win_name_fmt = "%s - Ethereal";
+  gchar  *win_name;
 
-  if (cf->user_saved || !cf->save_file) {
+  if (!cf->is_tempfile) {
     /* Get the last component of the file name, and put that in the
        status bar. */
     if ((name_ptr = (gchar *) strrchr(cf->filename, '/')) == NULL)
@@ -237,7 +264,7 @@ set_statusbar_filename(capture_file *cf)
   } else {
     /* The file we read is a temporary file from a live capture;
        we don't mention its name in the status bar. */
-    name_ptr = "<none>";
+    name_ptr = "<capture>";
   }
 
   msg_len = strlen(name_ptr) + strlen(done_fmt) + 64;
@@ -245,10 +272,17 @@ set_statusbar_filename(capture_file *cf)
   snprintf(done_msg, msg_len, done_fmt, name_ptr, cf->drops);
   gtk_statusbar_push(GTK_STATUSBAR(info_bar), file_ctx, done_msg);
   g_free(done_msg);
+
+  msg_len = strlen(name_ptr) + strlen(win_name_fmt) + 1;
+  win_name = g_malloc(msg_len);
+  snprintf(win_name, msg_len, win_name_fmt, name_ptr);
+  set_main_window_name(win_name);
+  g_free(win_name);
 }
 
 int
-read_cap_file(capture_file *cf) {
+read_cap_file(capture_file *cf)
+{
   gchar  *name_ptr, *load_msg, *load_fmt = " Loading: %s...";
   int     success;
   int     err;
@@ -278,6 +312,11 @@ read_cap_file(capture_file *cf) {
   freeze_clist(cf);
   proto_tree_is_visible = FALSE;
   success = wtap_loop(cf->wth, 0, wtap_dispatch_cb, (u_char *) cf, &err);
+  /* Set the file encapsulation type now; we don't know what it is until
+     we've looked at all the packets, as we don't know until then whether
+     there's more than one type (and thus whether it's
+     WTAP_ENCAP_PER_PACKET). */
+  cf->lnk_t = wtap_file_encap(cf->wth);
   wtap_close(cf->wth);
   cf->wth = NULL;
   cf->filed = open(cf->filename, O_RDONLY);
@@ -292,19 +331,22 @@ read_cap_file(capture_file *cf) {
   gtk_progress_set_value(GTK_PROGRESS(prog_bar), 0);
 
   gtk_statusbar_pop(GTK_STATUSBAR(info_bar), file_ctx);
+  set_display_filename(cf);
 
-  set_statusbar_filename(cf);
-
-  /* Enable menu items that make sense if you have a capture. */
+  /* Enable menu items that make sense if you have a capture file you've
+     finished reading. */
+  set_menu_sensitivity("/File/Save", !cf->user_saved);
+  set_menu_sensitivity("/File/Save As...", TRUE);
   set_menu_sensitivity("/File/Close", TRUE);
   set_menu_sensitivity("/File/Reload", TRUE);
   set_menu_sensitivity("/File/Print...", TRUE);
+
+  /* Enable menu items that make sense if you have some captured packets. */
   set_menu_sensitivity("/Display/Options...", TRUE);
   set_menu_sensitivity("/Display/Match Selected", TRUE);
   set_menu_sensitivity("/Display/Colorize Display...", TRUE);
   set_menu_sensitivity("/Display/Find Frame...", TRUE);
   set_menu_sensitivity("/Display/Go To Frame...", TRUE);
-  set_menu_sensitivity("/Tools/Follow TCP Stream", TRUE);
   set_menu_sensitivity("/Tools/Graph", TRUE);
   set_menu_sensitivity("/Tools/Summary", TRUE);
 
@@ -343,22 +385,30 @@ read_cap_file(capture_file *cf) {
 
 #ifdef HAVE_LIBPCAP
 int
-start_tail_cap_file(char *fname, capture_file *cf) {
+start_tail_cap_file(char *fname, gboolean is_tempfile, capture_file *cf)
+{
   int     err;
   int     i;
 
-  err = open_cap_file(fname, cf);
+  err = open_cap_file(fname, is_tempfile, cf);
   if (err == 0) {
+    /* Disable menu items that make sense only if you have a capture
+       file you've finished reading. */
     set_menu_sensitivity("/File/Open...", FALSE);
+
+    /* Disable menu items that make sense only if you're not currently
+       running a capture. */
+    set_menu_sensitivity("/Capture/Start...", FALSE);
+
+    /* Enable menu items that make sense if you have some captured
+       packets (yes, I know, we don't have any *yet*). */
     set_menu_sensitivity("/Display/Options...", TRUE);
     set_menu_sensitivity("/Display/Match Selected", TRUE);
     set_menu_sensitivity("/Display/Colorize Display...", TRUE);
     set_menu_sensitivity("/Display/Find Frame...", TRUE);
     set_menu_sensitivity("/Display/Go To Frame...", TRUE);
-    set_menu_sensitivity("/Tools/Follow TCP Stream", TRUE);
     set_menu_sensitivity("/Tools/Graph", TRUE);
     set_menu_sensitivity("/Tools/Summary", TRUE);
-    set_menu_sensitivity("/Capture/Start...", FALSE);
 
     for (i = 0; i < cf->cinfo.num_cols; i++) {
       if (get_column_resize_type(cf->cinfo.col_fmt[i]) == RESIZE_LIVE)
@@ -393,7 +443,7 @@ continue_tail_cap_file(capture_file *cf, int to_read)
   wtap_loop(cf->wth, to_read, wtap_dispatch_cb, (u_char *) cf, &err);
 
   gtk_clist_thaw(GTK_CLIST(packet_list));
-  if (auto_scroll_live)
+  if (auto_scroll_live && cf->plist_end != NULL)
     gtk_clist_moveto(GTK_CLIST(packet_list), 
 		       cf->plist_end->row, -1, 1.0, 1.0);
   return err;
@@ -409,22 +459,32 @@ finish_tail_cap_file(capture_file *cf)
   wtap_loop(cf->wth, 0, wtap_dispatch_cb, (u_char *) cf, &err);
 
   thaw_clist(cf);
-  if (auto_scroll_live)
+  if (auto_scroll_live && cf->plist_end != NULL)
     gtk_clist_moveto(GTK_CLIST(packet_list), 
 		       cf->plist_end->row, -1, 1.0, 1.0);
 
+  /* Set the file encapsulation type now; we don't know what it is until
+     we've looked at all the packets, as we don't know until then whether
+     there's more than one type (and thus whether it's
+     WTAP_ENCAP_PER_PACKET). */
+  cf->lnk_t = wtap_file_encap(cf->wth);
+
   /* There's nothing more to read from the capture file - close it. */
   wtap_close(cf->wth);
   cf->wth = NULL;
 
-  set_statusbar_filename(cf);
+  /* Pop the "<live capture in progress>" message off the status bar. */
+  gtk_statusbar_pop(GTK_STATUSBAR(info_bar), file_ctx);
+
+  set_display_filename(cf);
 
   /* Restore the "File/Open" menu item. */
   set_menu_sensitivity("/File/Open...", TRUE);
 
   /* Enable menu items that make sense if you have a capture file
      you've finished reading. */
-  set_menu_sensitivity("/File/Save", TRUE);
+  set_menu_sensitivity("/File/Save", !cf->user_saved);
+  set_menu_sensitivity("/File/Save As...", TRUE);
   set_menu_sensitivity("/File/Close", TRUE);
   set_menu_sensitivity("/File/Reload", TRUE);
   set_menu_sensitivity("/File/Print...", TRUE);
@@ -437,278 +497,38 @@ finish_tail_cap_file(capture_file *cf)
 }
 #endif /* HAVE_LIBPCAP */
 
-/* To do: Add check_col checks to the col_add* routines */
+typedef struct {
+  color_filter_t *colorf;
+  proto_tree	*protocol_tree;
+  guint8	*pd;
+} apply_color_filter_args;
 
+/*
+ * If no color filter has been applied, apply this one.
+ * (The "if no color filter has been applied" is to handle the case where
+ * more than one color filter matches the packet.)
+ */
 static void
-col_set_abs_time(frame_data *fd, int col)
+apply_color_filter(gpointer filter_arg, gpointer argp)
 {
-  struct tm *tmp;
-  time_t then;
+  color_filter_t *colorf = filter_arg;
+  apply_color_filter_args *args = argp;
 
-  then = fd->abs_secs;
-  tmp = localtime(&then);
-  snprintf(fd->cinfo->col_data[col], COL_MAX_LEN, "%02d:%02d:%02d.%04ld",
-    tmp->tm_hour,
-    tmp->tm_min,
-    tmp->tm_sec,
-    (long)fd->abs_usecs/100);
-}
-
-static void
-col_set_rel_time(frame_data *fd, int col)
-{
-  snprintf(fd->cinfo->col_data[col], COL_MAX_LEN, "%d.%06d", fd->rel_secs,
-    fd->rel_usecs);
-}
-
-static void
-col_set_delta_time(frame_data *fd, int col)
-{
-  snprintf(fd->cinfo->col_data[col], COL_MAX_LEN, "%d.%06d", fd->del_secs,
-    fd->del_usecs);
-}
-
-/* Add "command-line-specified" time. */
-static void
-col_set_cls_time(frame_data *fd, int col)
-{
-  switch (timestamp_type) {
-    case ABSOLUTE:
-      col_set_abs_time(fd, col);
-      break;
-
-    case RELATIVE:
-      col_set_rel_time(fd, col);
-      break;
-
-    case DELTA:
-      col_set_delta_time(fd, col);
-      break;
-  }
-}
-
-static void
-col_set_addr(frame_data *fd, int col, address *addr, gboolean is_res)
-{
-  u_int ipv4_addr;
-  struct e_in6_addr ipv6_addr;
-  struct atalk_ddp_addr ddp_addr;
-  struct sna_fid_type_4_addr sna_fid_type_4_addr;
-
-  switch (addr->type) {
-
-  case AT_ETHER:
-    if (is_res)
-      strncpy(fd->cinfo->col_data[col], get_ether_name(addr->data), COL_MAX_LEN);
-    else
-      strncpy(fd->cinfo->col_data[col], ether_to_str(addr->data), COL_MAX_LEN);
-    break;
-
-  case AT_IPv4:
-    memcpy(&ipv4_addr, addr->data, sizeof ipv4_addr);
-    if (is_res)
-      strncpy(fd->cinfo->col_data[col], get_hostname(ipv4_addr), COL_MAX_LEN);
-    else
-      strncpy(fd->cinfo->col_data[col], ip_to_str(addr->data), COL_MAX_LEN);
-    break;
-
-  case AT_IPv6:
-    memcpy(&ipv6_addr.s6_addr, addr->data, sizeof ipv6_addr.s6_addr);
-    if (is_res)
-      strncpy(fd->cinfo->col_data[col], get_hostname6(&ipv6_addr), COL_MAX_LEN);
-    else
-      strncpy(fd->cinfo->col_data[col], ip6_to_str(&ipv6_addr), COL_MAX_LEN);
-    break;
-
-  case AT_IPX:
-    strncpy(fd->cinfo->col_data[col],
-      ipx_addr_to_str(pntohl(&addr->data[0]), &addr->data[4]), COL_MAX_LEN);
-    break;
-
-  case AT_SNA:
-    switch (addr->len) {
-
-    case 1:
-      snprintf(fd->cinfo->col_data[col], COL_MAX_LEN, "%04X", addr->data[0]);
-      break;
-
-    case 2:
-      snprintf(fd->cinfo->col_data[col], COL_MAX_LEN, "%04X",
-        pntohs(&addr->data[0]));
-      break;
-
-    case SNA_FID_TYPE_4_ADDR_LEN:
-      memcpy(&sna_fid_type_4_addr, addr->data, SNA_FID_TYPE_4_ADDR_LEN);
-      strncpy(fd->cinfo->col_data[col],
-        sna_fid_type_4_addr_to_str(&sna_fid_type_4_addr), COL_MAX_LEN);
-      break;
-    }
-    break;
-
-  case AT_ATALK:
-    memcpy(&ddp_addr, addr->data, sizeof ddp_addr);
-    strncpy(fd->cinfo->col_data[col], atalk_addr_to_str(&ddp_addr),
-      COL_MAX_LEN);
-    break;
-
-  case AT_VINES:
-    strncpy(fd->cinfo->col_data[col], vines_addr_to_str(&addr->data[0]),
-      COL_MAX_LEN);
-    break;
-
-  default:
-    break;
-  }
-  fd->cinfo->col_data[col][COL_MAX_LEN - 1] = '\0';
-}
-
-static void
-col_set_port(frame_data *fd, int col, port_type ptype, guint32 port,
-		gboolean is_res)
-{
-  switch (ptype) {
-
-  case PT_TCP:
-    if (is_res)
-      strncpy(fd->cinfo->col_data[col], get_tcp_port(port), COL_MAX_LEN);
-    else
-      snprintf(fd->cinfo->col_data[col], COL_MAX_LEN, "%u", port);
-    break;
-
-  case PT_UDP:
-    if (is_res)
-      strncpy(fd->cinfo->col_data[col], get_udp_port(port), COL_MAX_LEN);
-    else
-      snprintf(fd->cinfo->col_data[col], COL_MAX_LEN, "%u", port);
-    break;
-
-  default:
-    break;
-  }
-  fd->cinfo->col_data[col][COL_MAX_LEN - 1] = '\0';
-}
-
-static void
-fill_in_columns(frame_data *fd)
-{
-  int i;
-
-  for (i = 0; i < fd->cinfo->num_cols; i++) {
-    switch (fd->cinfo->col_fmt[i]) {
-
-    case COL_NUMBER:
-      snprintf(fd->cinfo->col_data[i], COL_MAX_LEN, "%u", fd->num);
-      break;
-
-    case COL_CLS_TIME:
-      col_set_cls_time(fd, i);
-      break;
-
-    case COL_ABS_TIME:
-      col_set_abs_time(fd, i);
-      break;
-
-    case COL_REL_TIME:
-      col_set_rel_time(fd, i);
-      break;
-
-    case COL_DELTA_TIME:
-      col_set_delta_time(fd, i);
-      break;
-
-    case COL_DEF_SRC:
-    case COL_RES_SRC:	/* COL_DEF_SRC is currently just like COL_RES_SRC */
-      col_set_addr(fd, i, &pi.src, TRUE);
-      break;
-
-    case COL_UNRES_SRC:
-      col_set_addr(fd, i, &pi.src, FALSE);
-      break;
-
-    case COL_DEF_DL_SRC:
-    case COL_RES_DL_SRC:
-      col_set_addr(fd, i, &pi.dl_src, TRUE);
-      break;
-
-    case COL_UNRES_DL_SRC:
-      col_set_addr(fd, i, &pi.dl_src, FALSE);
-      break;
-
-    case COL_DEF_NET_SRC:
-    case COL_RES_NET_SRC:
-      col_set_addr(fd, i, &pi.net_src, TRUE);
-      break;
-
-    case COL_UNRES_NET_SRC:
-      col_set_addr(fd, i, &pi.net_src, FALSE);
-      break;
-
-    case COL_DEF_DST:
-    case COL_RES_DST:	/* COL_DEF_DST is currently just like COL_RES_DST */
-      col_set_addr(fd, i, &pi.dst, TRUE);
-      break;
-
-    case COL_UNRES_DST:
-      col_set_addr(fd, i, &pi.dst, FALSE);
-      break;
-
-    case COL_DEF_DL_DST:
-    case COL_RES_DL_DST:
-      col_set_addr(fd, i, &pi.dl_dst, TRUE);
-      break;
-
-    case COL_UNRES_DL_DST:
-      col_set_addr(fd, i, &pi.dl_dst, FALSE);
-      break;
-
-    case COL_DEF_NET_DST:
-    case COL_RES_NET_DST:
-      col_set_addr(fd, i, &pi.net_dst, TRUE);
-      break;
-
-    case COL_UNRES_NET_DST:
-      col_set_addr(fd, i, &pi.net_dst, FALSE);
-      break;
-
-    case COL_DEF_SRC_PORT:
-    case COL_RES_SRC_PORT:	/* COL_DEF_SRC_PORT is currently just like COL_RES_SRC_PORT */
-      col_set_port(fd, i, pi.ptype, pi.srcport, TRUE);
-      break;
-
-    case COL_UNRES_SRC_PORT:
-      col_set_port(fd, i, pi.ptype, pi.srcport, FALSE);
-      break;
-
-    case COL_DEF_DST_PORT:
-    case COL_RES_DST_PORT:	/* COL_DEF_DST_PORT is currently just like COL_RES_DST_PORT */
-      col_set_port(fd, i, pi.ptype, pi.destport, TRUE);
-      break;
-
-    case COL_UNRES_DST_PORT:
-      col_set_port(fd, i, pi.ptype, pi.destport, FALSE);
-      break;
-
-    case COL_PROTOCOL:	/* currently done by dissectors */
-    case COL_INFO:	/* currently done by dissectors */
-      break;
-
-    case COL_PACKET_LENGTH:
-      snprintf(fd->cinfo->col_data[i], COL_MAX_LEN, "%d", fd->pkt_len);
-      break;
-
-    case NUM_COL_FMTS:	/* keep compiler happy - shouldn't get here */
-      break;
-    }
+  if (colorf->c_colorfilter != NULL && args->colorf == NULL) {
+    if (dfilter_apply(colorf->c_colorfilter, args->protocol_tree, args->pd))
+      args->colorf = colorf;
   }
 }
 
 static void
 add_packet_to_packet_list(frame_data *fdata, capture_file *cf, const u_char *buf)
 {
+  apply_color_filter_args args;
   gint          i, row;
-  gint		crow;
-  gint 		color;
-  proto_tree   *protocol_tree;
+  proto_tree   *protocol_tree = NULL;
+
+  /* We don't yet have a color filter to apply. */
+  args.colorf = NULL;
 
   fdata->num = cf->count;
 
@@ -735,34 +555,33 @@ add_packet_to_packet_list(frame_data *fdata, capture_file *cf, const u_char *buf
   }
 
   /* Apply the filters */
-  if (cf->dfcode != NULL || CFILTERS_CONTAINS_FILTER(cf)) {
-	protocol_tree = proto_tree_create_root();
-	dissect_packet(buf, fdata, protocol_tree);
-	if (cf->dfcode != NULL)
-		fdata->passed_dfilter = dfilter_apply(cf->dfcode, protocol_tree, cf->pd);
-	else
-		fdata->passed_dfilter = TRUE;
-	/* Apply color filters. */
-	color = -1;
-        for(crow = 0; cf->colors->num_of_filters && 
-	      crow < cf->colors->num_of_filters; crow++) {
+  if (cf->dfcode != NULL || filter_list != NULL) {
+    protocol_tree = proto_tree_create_root();
+    dissect_packet(buf, fdata, protocol_tree);
+    if (cf->dfcode != NULL)
+      fdata->passed_dfilter = dfilter_apply(cf->dfcode, protocol_tree, cf->pd);
+    else
+      fdata->passed_dfilter = TRUE;
 
-            if(color_filter(cf,crow)->c_colorfilter == NULL) {
-		continue;
-	    }
-            if(dfilter_apply(color_filter(cf,crow)->c_colorfilter, protocol_tree,
-		 cf->pd)){
-                color = crow;
-		break;
-            }
-        }
-
-	proto_tree_free(protocol_tree);
+    /* Apply color filters, if we have any. */
+    if (filter_list != NULL) {
+      args.protocol_tree = protocol_tree;
+      args.pd = cf->pd;
+      g_slist_foreach(filter_list, apply_color_filter, &args);
+    }
+    proto_tree_free(protocol_tree);
   }
   else {
-	dissect_packet(buf, fdata, NULL);
+#ifdef HAVE_DLFCN_H
+	if (plugin_list)
+	    protocol_tree = proto_tree_create_root();
+#endif
+	dissect_packet(buf, fdata, protocol_tree);
 	fdata->passed_dfilter = TRUE;
-	color = -1;
+#ifdef HAVE_DLFCN_H
+	if (protocol_tree)
+	    proto_tree_free(protocol_tree);
+#endif
   }
   if (fdata->passed_dfilter) {
     /* If we don't have the time stamp of the previous displayed packet,
@@ -795,11 +614,11 @@ add_packet_to_packet_list(frame_data *fdata, capture_file *cf, const u_char *buf
     row = gtk_clist_append(GTK_CLIST(packet_list), fdata->cinfo->col_data);
     fdata->row = row;
 
-    if (cf->colors->color_filters && (color != -1)){
+    if (filter_list != NULL && (args.colorf != NULL)) {
         gtk_clist_set_background(GTK_CLIST(packet_list), row,
-                   &(color_filter(cf,color)->bg_color));
+                   &args.colorf->bg_color);
         gtk_clist_set_foreground(GTK_CLIST(packet_list), row,
-                   &(color_filter(cf,color)->fg_color));
+                   &args.colorf->fg_color);
     } else {
         gtk_clist_set_background(GTK_CLIST(packet_list), row, &WHITE);
         gtk_clist_set_foreground(GTK_CLIST(packet_list), row, &BLACK);
@@ -1245,9 +1064,6 @@ change_time_formats(capture_file *cf)
 static void
 clear_tree_and_hex_views(void)
 {
-  GList *selection;
-  GtkWidget *tmp_item;
-
   /* Clear the hex dump. */
   gtk_text_freeze(GTK_TEXT(byte_view));
   gtk_text_set_point(GTK_TEXT(byte_view), 0);
@@ -1255,25 +1071,9 @@ clear_tree_and_hex_views(void)
     gtk_text_get_length(GTK_TEXT(byte_view)));
   gtk_text_thaw(GTK_TEXT(byte_view));
 
-  /* Deselect any selected tree item. gtktree.c should
-   * do this when we clear_items, but it doesn't. I copied
-   * this while() loop from gtktree.c, gtk_real_tree_select_child()
-   */
-  if (GTK_TREE(tree_view)->root_tree) {
-	  selection = GTK_TREE(tree_view)->root_tree->selection;
-	  while (selection) {
-		  tmp_item = selection->data;
-		  gtk_tree_item_deselect(GTK_TREE_ITEM(tmp_item));
-		  gtk_widget_unref(tmp_item);
-		  selection = selection->next;
-	  }
-	  g_list_free(GTK_TREE(tree_view)->root_tree->selection);
-	  GTK_TREE(tree_view)->root_tree->selection = NULL;
-  }
+  /* Remove all nodes in ctree. This is how it's done in testgtk.c in GTK+ */
+  gtk_clist_clear ( GTK_CLIST(tree_view) );
 
-  /* Clear the protocol tree view. The length arg of -1
-   * means to clear all items up to the end. */
-  gtk_tree_clear_items(GTK_TREE(tree_view), 0, -1);
 }
 
 gboolean
@@ -1378,7 +1178,7 @@ find_packet(capture_file *cf, dfilter *sfcode)
     return FALSE;	/* failure */
 }
 
-gboolean
+goto_result_t
 goto_frame(capture_file *cf, guint fnumber)
 {
   frame_data *fd;
@@ -1386,15 +1186,17 @@ goto_frame(capture_file *cf, guint fnumber)
   for (fd = cf->plist; fd != NULL && fd->num < fnumber; fd = fd->next)
     ;
 
-  if (fd != NULL && fd->passed_dfilter) {
-    /* We found that frame, and it's currently being displayed.
-       Make it visible, and select it. */
-    if (!gtk_clist_row_is_visible(GTK_CLIST(packet_list), fd->row))
-      gtk_clist_moveto(GTK_CLIST(packet_list), fd->row, -1, 0.0, 0.0);
-    gtk_clist_select_row(GTK_CLIST(packet_list), fd->row, -1);
-    return TRUE;	/* success */
-  } else
-    return FALSE;	/* failure */
+  if (fd == NULL)
+    return NO_SUCH_FRAME;	/* we didn't find that frame */
+  if (!fd->passed_dfilter)
+    return FRAME_NOT_DISPLAYED;	/* the frame with that number isn't displayed */
+
+  /* We found that frame, and it's currently being displayed.
+     Make it visible, and select it. */
+  if (!gtk_clist_row_is_visible(GTK_CLIST(packet_list), fd->row))
+    gtk_clist_moveto(GTK_CLIST(packet_list), fd->row, -1, 0.0, 0.0);
+  gtk_clist_select_row(GTK_CLIST(packet_list), fd->row, -1);
+  return FOUND_FRAME;
 }
 
 /* Select the packet on a given row. */
@@ -1404,12 +1206,6 @@ select_packet(capture_file *cf, int row)
   frame_data *fd;
   int i;
 
-  /* Clear out whatever's currently in the hex dump. */
-  gtk_text_freeze(GTK_TEXT(byte_view));
-  gtk_text_set_point(GTK_TEXT(byte_view), 0);
-  gtk_text_forward_delete(GTK_TEXT(byte_view),
-    gtk_text_get_length(GTK_TEXT(byte_view)));
-
   /* Search through the list of frames to see which one is in
      this row. */
   for (fd = cf->plist, i = 0; fd != NULL; fd = fd->next, i++) {
@@ -1438,12 +1234,17 @@ select_packet(capture_file *cf, int row)
   proto_tree_draw(cf->protocol_tree, tree_view);
   packet_hex_print(GTK_TEXT(byte_view), cf->pd, cf->current_frame->cap_len,
 			-1, -1, cf->current_frame->encoding);
-  gtk_text_thaw(GTK_TEXT(byte_view));
 
   /* A packet is selected, so "File/Print Packet" has something to print. */
   set_menu_sensitivity("/File/Print Packet", TRUE);
   set_menu_sensitivity("/Display/Collapse All", TRUE);
   set_menu_sensitivity("/Display/Expand All", TRUE);
+  
+  if (pi.ipproto == 6) {
+      set_menu_sensitivity("/Tools/Follow TCP Stream", TRUE);
+  } else {
+      set_menu_sensitivity("/Tools/Follow TCP Stream", FALSE);
+  }
 }
 
 /* Unselect the selected packet, if any. */
@@ -1467,6 +1268,7 @@ unselect_packet(capture_file *cf)
   set_menu_sensitivity("/File/Print Packet", FALSE);
   set_menu_sensitivity("/Display/Collapse All", FALSE);
   set_menu_sensitivity("/Display/Expand All", FALSE);
+  set_menu_sensitivity("/Tools/Follow TCP Stream", FALSE);
 }
 
 static void
@@ -1507,92 +1309,195 @@ thaw_clist(capture_file *cf)
     gtk_clist_set_column_resizeable(GTK_CLIST(packet_list), i, TRUE);
 }
 
-/* Tries to mv a file. If unsuccessful, tries to cp the file.
- * Returns 0 on failure to do either, 1 on success of either
- */
 int
-file_mv(char *from, char *to)
+save_cap_file(char *fname, capture_file *cf, gboolean save_filtered,
+		guint save_format)
 {
+  gchar        *from_filename;
+  gchar        *name_ptr, *save_msg, *save_fmt = " Saving: %s...";
+  size_t        msg_len;
+  int           err;
+  gboolean      do_copy;
+  int           from_fd, to_fd, nread, nwritten;
+  wtap_dumper  *pdh;
+  frame_data   *fd;
+  struct wtap_pkthdr hdr;
+  guint8        pd[65536];
 
-#define COPY_BUFFER_SIZE	8192
+  if ((name_ptr = (gchar *) strrchr(fname, '/')) == NULL)
+    name_ptr = fname;
+  else
+    name_ptr++;
+  msg_len = strlen(name_ptr) + strlen(save_fmt) + 2;
+  save_msg = g_malloc(msg_len);
+  snprintf(save_msg, msg_len, save_fmt, name_ptr);
+  gtk_statusbar_push(GTK_STATUSBAR(info_bar), file_ctx, save_msg);
+  g_free(save_msg);
 
-	int retval;
+  if (!save_filtered && save_format == cf->cd_t) {
+    /* We're not filtering packets, and we're saving it in the format
+       it's already in, so we can just move or copy the raw data. */
 
-#ifndef WIN32
-	/* try a hard link */
-	retval = link(from, to);
-
-	/* or try a copy */
-	if (retval < 0) {
-#endif
-		retval = file_cp(from, to);
-		if (!retval) {
-			return 0;
-		}
-#ifndef WIN32
+    /* In this branch, we set "err" only if we get an error, so we
+       must first clear it. */
+    err = 0;
+    if (cf->is_tempfile) {
+      /* The file being saved is a temporary file from a live
+         capture, so it doesn't need to stay around under that name;
+	 first, try renaming the capture buffer file to the new name. */
+      if (rename(cf->filename, fname) == 0) {
+      	/* That succeeded - there's no need to copy the source file. */
+      	from_filename = NULL;
+	do_copy = FALSE;
+      } else {
+      	if (errno == EXDEV) {
+	  /* They're on different file systems, so we have to copy the
+	     file. */
+	  do_copy = TRUE;
+          from_filename = cf->filename;
+	} else {
+	  /* The rename failed, but not because they're on different
+	     file systems - put up an error message.  (Or should we
+	     just punt and try to copy?  The only reason why I'd
+	     expect the rename to fail and the copy to succeed would
+	     be if we didn't have permission to remove the file from
+	     the temporary directory, and that might be fixable - but
+	     is it worth requiring the user to go off and fix it?) */
+	  err = errno;
+	  simple_dialog(ESD_TYPE_WARN, NULL,
+				file_rename_error_message(err), fname);
+	  goto done;
 	}
-#endif
+      }
+    } else {
+      /* It's a permanent file, so we should copy it, and not remove the
+         original. */
+      do_copy = TRUE;
+      from_filename = cf->filename;
+    }
 
-	unlink(from);
-	return 1;
-}
+    /* Copy the file, if we haven't moved it. */
+    if (do_copy) {
+      /* Copy the raw bytes of the file. */
+      from_fd = open(from_filename, O_RDONLY);
+      if (from_fd < 0) {
+      	err = errno;
+	simple_dialog(ESD_TYPE_WARN, NULL,
+			file_open_error_message(err, TRUE), from_filename);
+	goto done;
+      }
 
-/* Copies a file.
- * Returns 0 on failure to do either, 1 on success of either
- */
-int
-file_cp(char *from, char *to)
-{
+      to_fd = creat(fname, 0644);
+      if (to_fd < 0) {
+      	err = errno;
+	simple_dialog(ESD_TYPE_WARN, NULL,
+			file_open_error_message(err, TRUE), fname);
+	close(from_fd);
+	goto done;
+      }
 
-#define COPY_BUFFER_SIZE	8192
-
-	int from_fd, to_fd, nread, nwritten;
-	char *buffer;
-
-	buffer = g_malloc(COPY_BUFFER_SIZE);
-
-	from_fd = open(from, O_RDONLY);
-	if (from_fd < 0) {
-		simple_dialog(ESD_TYPE_WARN, NULL,
-			file_open_error_message(errno, TRUE), from);
-		return 0;
-	}
-
-	to_fd = creat(to, 0644);
-	if (to_fd < 0) {
-		simple_dialog(ESD_TYPE_WARN, NULL,
-			file_open_error_message(errno, TRUE), to);
-		close(from_fd);
-		return 0;
-	}
-
-	while( (nread = read(from_fd, buffer, COPY_BUFFER_SIZE)) > 0) {
-		nwritten = write(to_fd, buffer, nread);
-		if (nwritten < nread) {
-			if (nwritten < 0) {
-				simple_dialog(ESD_TYPE_WARN, NULL,
-					file_write_error_message(errno), to);
-			} else {
-				simple_dialog(ESD_TYPE_WARN, NULL,
-"The file \"%s\" could not be saved: tried writing %d, wrote %d.\n",
-					to, nread, nwritten);
-			}
-			close(from_fd);
-			close(to_fd);
-			return 0;
-		}
-	}
-	if (nread < 0) {
-		simple_dialog(ESD_TYPE_WARN, NULL,
-			file_read_error_message(errno), from);
-		close(from_fd);
-		close(to_fd);
-		return 0;
+      while ((nread = read(from_fd, pd, sizeof pd)) > 0) {
+	nwritten = write(to_fd, pd, nread);
+	if (nwritten < nread) {
+	  if (nwritten < 0)
+	    err = errno;
+	  else
+	    err = WTAP_ERR_SHORT_WRITE;
+	  simple_dialog(ESD_TYPE_WARN, NULL,
+				file_write_error_message(err), fname);
+	  close(from_fd);
+	  close(to_fd);
+	  goto done;
 	}
+      }
+      if (nread < 0) {
+      	err = errno;
+	simple_dialog(ESD_TYPE_WARN, NULL,
+			file_read_error_message(err), from_filename);
 	close(from_fd);
 	close(to_fd);
+	goto done;
+      }
+      close(from_fd);
+      if (close(to_fd) < 0) {
+      	err = errno;
+	simple_dialog(ESD_TYPE_WARN, NULL,
+		file_close_error_message(err), fname);
+	goto done;
+      }
+    }
+  } else {
+    /* Either we're filtering packets, or we're saving in a different
+       format; we can't do that by copying or moving the capture file,
+       we have to do it by writing the packets out in Wiretap. */
+    pdh = wtap_dump_open(fname, save_format, cf->lnk_t, cf->snap, &err);
+    if (pdh == NULL) {
+      simple_dialog(ESD_TYPE_WARN, NULL,
+			file_open_error_message(err, TRUE), fname);
+      goto done;
+    }
 
-	return 1;
+    /* XXX - have a way to save only the packets currently selected by
+       the display filter.
+
+       If we do that, should we make that file the current file?  If so,
+       it means we can no longer get at the other packets.  What does
+       NetMon do? */
+    for (fd = cf->plist; fd != NULL; fd = fd->next) {
+      /* XXX - do a progress bar */
+      if (!save_filtered || fd->passed_dfilter) {
+      	/* Either we're saving all frames, or we're saving filtered frames
+	   and this one passed the display filter - save it. */
+        hdr.ts.tv_sec = fd->abs_secs;
+        hdr.ts.tv_usec = fd->abs_usecs;
+        hdr.caplen = fd->cap_len;
+        hdr.len = fd->pkt_len;
+        hdr.pkt_encap = fd->lnk_t;
+        hdr.pseudo_header = fd->pseudo_header;
+	wtap_seek_read(cf->cd_t, cf->fh, fd->file_off, pd, fd->cap_len);
+
+        if (!wtap_dump(pdh, &hdr, pd, &err)) {
+	    simple_dialog(ESD_TYPE_WARN, NULL,
+				file_write_error_message(err), fname);
+	    wtap_dump_close(pdh, &err);
+	    goto done;
+	}
+      }
+    }
+
+    if (!wtap_dump_close(pdh, &err)) {
+      simple_dialog(ESD_TYPE_WARN, NULL,
+		file_close_error_message(err), fname);
+      goto done;
+    }
+  }
+
+done:
+
+  /* Pop the "Saving:" message off the status bar. */
+  gtk_statusbar_pop(GTK_STATUSBAR(info_bar), file_ctx);
+  if (err == 0) {
+    if (!save_filtered) {
+      /* We saved the entire capture, not just some packets from it.
+         Open and read the file we saved it to.
+
+	 XXX - this is somewhat of a waste; we already have the
+	 packets, all this gets us is updated file type information
+	 (which we could just stuff into "cf"), and having the new
+	 file be the one we have opened and from which we're reading
+	 the data, and it means we have to spend time opening and
+	 reading the file, which could be a significant amount of
+	 time if the file is large. */
+      cf->user_saved = TRUE;
+
+      if ((err = open_cap_file(fname, FALSE, cf)) == 0) {
+	/* XXX - report errors if this fails? */
+	err = read_cap_file(cf);
+	set_menu_sensitivity("/File/Save", FALSE);
+      }
+    }
+  }
+  return err;
 }
 
 char *
@@ -1609,9 +1514,21 @@ file_open_error_message(int err, int for_writing)
 
   case WTAP_ERR_FILE_UNKNOWN_FORMAT:
   case WTAP_ERR_UNSUPPORTED:
+    /* Seen only when opening a capture file for reading. */
     errmsg = "The file \"%s\" is not a capture file in a format Ethereal understands.";
     break;
 
+  case WTAP_ERR_UNSUPPORTED_FILE_TYPE:
+    /* Seen only when opening a capture file for writing. */
+    errmsg = "Ethereal does not support writing capture files in that format.";
+    break;
+
+  case WTAP_ERR_UNSUPPORTED_ENCAP:
+  case WTAP_ERR_ENCAP_PER_PACKET_UNSUPPORTED:
+    /* Seen only when opening a capture file for writing. */
+    errmsg = "Ethereal cannot save this capture in that format.";
+    break;
+
   case WTAP_ERR_BAD_RECORD:
     errmsg = "The file \"%s\" appears to be damaged or corrupt.";
     break;
@@ -1628,6 +1545,10 @@ file_open_error_message(int err, int for_writing)
              " in the middle of a packet.";
     break;
 
+  case WTAP_ERR_SHORT_WRITE:
+    errmsg = "A full header couldn't be written to the file \"%s\".";
+    break;
+
   case ENOENT:
     if (for_writing)
       errmsg = "The path to the file \"%s\" does not exist.";
@@ -1651,6 +1572,31 @@ file_open_error_message(int err, int for_writing)
   return errmsg;
 }
 
+static char *
+file_rename_error_message(int err)
+{
+  char *errmsg;
+  static char errmsg_errno[1024+1];
+
+  switch (err) {
+
+  case ENOENT:
+    errmsg = "The path to the file \"%s\" does not exist.";
+    break;
+
+  case EACCES:
+    errmsg = "You do not have permission to move the capture file to \"%s\".";
+    break;
+
+  default:
+    sprintf(errmsg_errno, "The file \"%%s\" could not be moved: %s.",
+				wtap_strerror(err));
+    errmsg = errmsg_errno;
+    break;
+  }
+  return errmsg;
+}
+
 char *
 file_read_error_message(int err)
 {
@@ -1687,3 +1633,42 @@ file_write_error_message(int err)
   }
   return errmsg;
 }
+
+/* Check for write errors - if the file is being written to an NFS server,
+   a write error may not show up until the file is closed, as NFS clients
+   might not send writes to the server until the "write()" call finishes,
+   so that the write may fail on the server but the "write()" may succeed. */
+static char *
+file_close_error_message(int err)
+{
+  char *errmsg;
+  static char errmsg_errno[1024+1];
+
+  switch (err) {
+
+  case WTAP_ERR_CANT_CLOSE:
+    errmsg = "The file \"%s\" couldn't be closed for some unknown reason.";
+    break;
+
+  case WTAP_ERR_SHORT_WRITE:
+    errmsg = "Not all the data could be written to the file \"%s\".";
+    break;
+
+  case ENOSPC:
+    errmsg = "The file \"%s\" could not be saved because there is no space left on the file system.";
+    break;
+
+#ifdef EDQUOT
+  case EDQUOT:
+    errmsg = "The file \"%s\" could not be saved because you are too close to, or over, your disk quota.";
+    break;
+#endif
+
+  default:
+    sprintf(errmsg_errno, "An error occurred while closing the file \"%%s\": %s.",
+				wtap_strerror(err));
+    errmsg = errmsg_errno;
+    break;
+  }
+  return errmsg;
+}
