@@ -1,6 +1,6 @@
 /* follow.c
  *
- * $Id: follow.c,v 1.18 1999/11/28 03:35:09 gerald Exp $
+ * $Id: follow.c,v 1.20 1999/12/10 04:25:59 gram Exp $
  *
  * Copyright 1998 Mike Hall <mlh@io.com>
  *
@@ -29,7 +29,6 @@
 # include "config.h"
 #endif
 
-#include <gtk/gtk.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -42,7 +41,6 @@
 #endif
 
 #include <glib.h>
-#include "gtk/main.h"
 #include "packet.h"
 #include "follow.h"
 
@@ -274,9 +272,6 @@ reset_tcp_reassembly() {
 
 static void 
 write_packet_data( tcp_stream_chunk *sc, const char *data ) {
-  if (sc->dlen == 0)
-    return;
   fwrite( sc, 1, sizeof(tcp_stream_chunk), data_out_file );
   fwrite( data, 1, sc->dlen, data_out_file );
 }
-  
