@@ -1,7 +1,7 @@
 /* gtkglobals.h
  * GTK-related Global defines, etc.
  *
- * $Id: gtkglobals.h,v 1.1 1999/10/20 22:36:05 gram Exp $
+ * $Id: gtkglobals.h,v 1.4 1999/12/29 20:10:09 gram Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -30,10 +30,14 @@
 #include <gtk/gtk.h>
 #endif
 
-extern GtkWidget   *file_sel, *packet_list, *tree_view, *byte_view, *prog_bar,
-            *info_bar;
+extern GtkWidget   *top_level, *file_sel, *packet_list, *tree_view,
+            *byte_view, *prog_bar, *info_bar;
 extern GdkFont     *m_r_font, *m_b_font;
 
 extern GtkStyle *item_style;
 
+void set_scrollbar_placement(int); /* 1=right, 0=left */
+void set_plist_sel_browse(gboolean);
+void set_ptree_sel_browse(gboolean);
+
 #endif
