@@ -1,6 +1,6 @@
 /* main.c
  *
- * $Id: main.c,v 1.52 1999/11/29 04:30:21 guy Exp $
+ * $Id: main.c,v 1.77 1999/12/29 20:10:10 gram Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -31,7 +31,6 @@
  * - Multiple window support
  * - Add cut/copy/paste
  * - Create header parsing routines
- * - Make byte view scrollbars automatic?
  * - Make byte view selections more fancy?
  *
  */
@@ -100,15 +99,18 @@
 #include "resolv.h"
 #include "follow.h"
 #include "util.h"
+#include "ui_util.h"
 #include "proto_draw.h"
 #include "dfilter.h"
 #include "keys.h"
+#include "gtkglobals.h"
 
 FILE        *data_out_file = NULL;
 packet_info  pi;
 capture_file cf;
-GtkWidget   *file_sel, *packet_list, *tree_view, *byte_view, *prog_bar,
-            *info_bar;
+GtkWidget   *top_level, *file_sel, *packet_list, *tree_view, *byte_view,
+            *prog_bar, *info_bar, *tv_scrollw, *pkt_scrollw,
+	    *bv_vscroll_left, *bv_vscroll_right;
 GdkFont     *m_r_font, *m_b_font;
 guint        main_ctx, file_ctx;
 gchar        comp_info_str[256];
@@ -134,45 +136,14 @@ static char* hfinfo_numeric_format(header_field_info *hfinfo);
 void
 about_ethereal( GtkWidget *w, gpointer data ) {
   simple_dialog(ESD_TYPE_INFO, NULL,
-		"GNU Ethereal - network protocol analyzer\n"
-		"Version %s (C) 1999 Gerald Combs <gerald@zing.org>\n"
+		"Ethereal - Network Protocol Analyzer\n"
+		"Version %s (C) 1998-2000 Gerald Combs <gerald@zing.org>\n"
                 "Compiled with %s\n\n"
-		"Contributors:\n"
 
-		"Gilbert Ramirez          <gramirez@tivoli.com>\n"
-		"Hannes R. Boehm          <hannes@boehm.org>\n"
-		"Mike Hall                <mlh@io.com>\n"
-		"Bobo Rajec               <bobo@bsp-consulting.sk>\n"
-		"Laurent Deniel           <deniel@worldnet.fr>\n"
-		"Don Lafontaine           <lafont02@cn.ca>\n"
-		"Guy Harris               <guy@alum.mit.edu>\n"
-		"Simon Wilkinson          <sxw@dcs.ed.ac.uk>\n"
-		"Joerg Mayer              <jmayer@telemation.de>\n"
-		"Martin Maciaszek         <fastjack@i-s-o.net>\n"
-		"Didier Jorand            <Didier.Jorand@alcatel.fr>\n"
-		"Jun-ichiro itojun Hagino <itojun@iijlab.net>\n"
-		"Richard Sharpe           <sharpe@ns.aus.com>\n"
-		"John McDermott           <jjm@jkintl.com>\n"
-		"Jeff Jahr                <jjahr@shastanets.com>\n"
-		"Brad Robel-Forrest       <bradr@watchguard.com>\n"
-		"Ashok Narayanan          <ashokn@cisco.com>\n"
-		"Aaron Hillegass          <aaron@classmax.com>\n"
-		"Jason Lango              <jal@netapp.com>\n"
-		"Johan Feyaerts           <Johan.Feyaerts@siemens.atea.be>\n"
-		"Olivier Abad             <abad@daba.dhis.org>\n"
-		"Thierry Andry            <Thierry.Andry@advalvas.be>\n"
-		"Jeff Foster              <jfoste@woodward.com>\n"
-		"Peter Torvals            <petertv@xoommail.com>\n"
-		"Christophe Tronche       <ch.tronche@computer.org>\n"
-		"Nathan Neulinger         <nneul@umr.edu>\n"
-		"Tomislav Vujec           <tvujec@carnet.hr>\n"
-		"Kojak                    <kojak@bigwig.net>\n"
-		"Uwe Girlich              <Uwe.Girlich@philosys.de>\n"
-		"Warren Young             <tangent@mail.com>\n"
-		"Heikki Vatiainen         <hessu@cs.tut.fi>\n"
-		"Greg Hankins             <gregh@twoguys.org>\n"
+		"Check the man page for complete documentation and\n"
+		"for the list of contributors.\n"
 
-		"\nSee http://ethereal.zing.org for more information",
+		"\nSee http://ethereal.zing.org/ for more information.",
                 VERSION, comp_info_str);
 }
 
@@ -436,19 +407,17 @@ static void follow_print_stream(GtkWidget *w, gpointer parent_w)
 static void
 follow_load_text(GtkWidget *text, char *filename, gboolean show_ascii)
 {
-	int bytes_already, bcount;
-        tcp_stream_chunk sc;
-        guint32 client_addr = 0;
-        guint16 client_port = 0;
-        GdkColor client = { 0, 16383, 0, 0 };
-        GdkColor server = { 0, 0, 0, 16383 };
+  int bytes_already, bcount;
+  tcp_stream_chunk sc;
+  guint32 client_addr = 0;
+  guint16 client_port = 0;
 
-	/* Delete any info already in text box */
-	bytes_already = gtk_text_get_length(GTK_TEXT(text));
-	if (bytes_already > 0) {
-		gtk_text_set_point(GTK_TEXT(text), 0);
-		gtk_text_forward_delete(GTK_TEXT(text), bytes_already);
-	}
+  /* Delete any info already in text box */
+  bytes_already = gtk_text_get_length(GTK_TEXT(text));
+  if (bytes_already > 0) {
+    gtk_text_set_point(GTK_TEXT(text), 0);
+    gtk_text_forward_delete(GTK_TEXT(text), bytes_already);
+  }
 
     /* stop the updates while we fill the text box */
     gtk_text_freeze( GTK_TEXT(text) );
@@ -478,9 +447,11 @@ follow_load_text(GtkWidget *text, char *filename, gboolean show_ascii)
 		  EBCDIC_to_ASCII(buffer, nchars);
 	  }
           if (client_addr == sc.src_addr && client_port == sc.src_port)
-	    gtk_text_insert( GTK_TEXT(text), m_r_font, &client, NULL, buffer, nchars );
+	    gtk_text_insert( GTK_TEXT(text), m_r_font, &prefs.st_client_fg,
+              &prefs.st_client_bg, buffer, nchars );
           else
-	    gtk_text_insert( GTK_TEXT(text), m_r_font, &server, NULL, buffer, nchars );
+	    gtk_text_insert( GTK_TEXT(text), m_r_font, &prefs.st_server_fg,
+              &prefs.st_server_bg, buffer, nchars );
 	}
       }
       if( ferror( data_out_file ) ) {
@@ -736,7 +707,7 @@ filter_reset_cb(GtkWidget *w, gpointer data)
 }
 
 /* What to do when a list item is selected/unselected */
-void
+static void
 packet_list_select_cb(GtkWidget *w, gint row, gint col, gpointer evt) {
 
 #ifdef HAVE_LIBPCAP
@@ -751,37 +722,36 @@ packet_list_select_cb(GtkWidget *w, gint row, gint col, gpointer evt) {
   select_packet(&cf, row);
 }
 
-void
+static void
 packet_list_unselect_cb(GtkWidget *w, gint row, gint col, gpointer evt) {
   unselect_packet(&cf);
 }
 
-void
-tree_view_cb(GtkWidget *w, gpointer data) {
+static void
+tree_view_select_row_cb(GtkCTree *ctree, GList *node, gint column, gpointer user_data)
+{
+	field_info	*finfo;
+	int		tree_selected_start = -1;
+	int		tree_selected_len = -1;
 
-  field_info	*finfo;
-  int		tree_selected_start = -1;
-  int		tree_selected_len = -1;
+	g_assert(node);
+	finfo = gtk_ctree_node_get_row_data( ctree, GTK_CTREE_NODE(node) );
+	g_assert(finfo);
 
-  if (GTK_TREE(w)->selection) {
-    finfo = 
-	gtk_object_get_data(GTK_OBJECT(GTK_TREE(w)->selection->data),
-				   E_TREEINFO_FIELD_INFO_KEY);
-    g_assert(finfo);
-    finfo_selected = finfo;
-    tree_selected_start = finfo->start;
-    tree_selected_len   = finfo->length;
-  }
+	finfo_selected = finfo;
+	tree_selected_start = finfo->start;
+	tree_selected_len   = finfo->length;
 
-  gtk_text_freeze(GTK_TEXT(byte_view));
-  gtk_text_set_point(GTK_TEXT(byte_view), 0);
-  gtk_text_forward_delete(GTK_TEXT(byte_view),
-    gtk_text_get_length(GTK_TEXT(byte_view)));
-  packet_hex_print(GTK_TEXT(byte_view), cf.pd, cf.current_frame->cap_len, 
-		   tree_selected_start, tree_selected_len,
-		   cf.current_frame->encoding);
-  
-  gtk_text_thaw(GTK_TEXT(byte_view));
+	packet_hex_print(GTK_TEXT(byte_view), cf.pd, cf.current_frame->cap_len, 
+		tree_selected_start, tree_selected_len, cf.current_frame->encoding);
+}
+
+static void
+tree_view_unselect_row_cb(GtkCTree *ctree, GList *node, gint column, gpointer user_data)
+{
+	finfo_selected = NULL;
+	packet_hex_print(GTK_TEXT(byte_view), cf.pd, cf.current_frame->cap_len, 
+		-1, -1, cf.current_frame->encoding);
 }
 
 void collapse_all_cb(GtkWidget *widget, gpointer data) {
@@ -794,11 +764,58 @@ void expand_all_cb(GtkWidget *widget, gpointer data) {
     expand_all_tree(cf.protocol_tree, tree_view);
 }
 
+void
+set_scrollbar_placement(int pos) /* 0=left, 1=right */
+{
+	if (pos) {
+		gtk_scrolled_window_set_placement( GTK_SCROLLED_WINDOW(pkt_scrollw),
+				GTK_CORNER_TOP_LEFT );
+		gtk_scrolled_window_set_placement( GTK_SCROLLED_WINDOW(tv_scrollw),
+				GTK_CORNER_TOP_LEFT );
+		gtk_widget_hide(bv_vscroll_left);
+		gtk_widget_show(bv_vscroll_right);
+	}
+	else {
+		gtk_scrolled_window_set_placement( GTK_SCROLLED_WINDOW(pkt_scrollw),
+				GTK_CORNER_TOP_RIGHT );
+		gtk_scrolled_window_set_placement( GTK_SCROLLED_WINDOW(tv_scrollw),
+				GTK_CORNER_TOP_RIGHT );
+		gtk_widget_hide(bv_vscroll_right);
+		gtk_widget_show(bv_vscroll_left);
+	}
+}
+
+void
+set_plist_sel_browse(gboolean val)
+{
+	if (finfo_selected)
+		unselect_packet(&cf);
+
+	if (val) {
+		gtk_clist_set_selection_mode(GTK_CLIST(packet_list), GTK_SELECTION_SINGLE);
+	}
+	else {
+		gtk_clist_set_selection_mode(GTK_CLIST(packet_list), GTK_SELECTION_EXTENDED);
+	}
+}
+
+void
+set_ptree_sel_browse(gboolean val)
+{
+	if (val) {
+		gtk_clist_set_selection_mode(GTK_CLIST(tree_view), GTK_SELECTION_SINGLE);
+	}
+	else {
+		gtk_clist_set_selection_mode(GTK_CLIST(tree_view), GTK_SELECTION_EXTENDED);
+	}
+}
+
 void
 file_quit_cmd_cb (GtkWidget *widget, gpointer data) {
-  if (cf.save_file && !cf.user_saved) {
-	unlink(cf.save_file);
-  }
+  /* If we have a capture file open, and it's a temporary file,
+     unlink it. */
+  if (cf.filename != NULL && cf.is_tempfile)
+	unlink(cf.filename);
   gtk_exit(0);
 }
 
@@ -822,11 +839,19 @@ print_usage(void) {
 
   fprintf(stderr, "This is GNU %s %s, compiled with %s\n", PACKAGE,
 	  VERSION, comp_info_str);
+#ifdef HAVE_LIBPCAP
   fprintf(stderr, "%s [-vh] [-kQS] [-b <bold font>] [-B <byte view height>] [-c count]\n",
 	  PACKAGE);
-  fprintf(stderr, "         [-f <filter expression>] [-i interface] [-m <medium font>] [-n]\n");
-  fprintf(stderr, "         [-P <packet list height>] [-r infile] [-s snaplen]\n");
-  fprintf(stderr, "         [-t <time stamp format>] [-T <tree view height>] [-w savefile] \n");
+  fprintf(stderr, "         [-f <capture filter>] [-i interface] [-m <medium font>] [-n]\n");
+  fprintf(stderr, "         [-P <packet list height>] [-r infile] [-R <read filter>]\n");
+  fprintf(stderr, "         [-s snaplen] [-t <time stamp format>] [-T <tree view height>]\n");
+  fprintf(stderr, "         [-w savefile]\n");
+#else
+  fprintf(stderr, "%s [-vh] [-b <bold font>] [-B <byte view height>] [-m <medium font>]\n",
+	  PACKAGE);
+  fprintf(stderr, "         [-n] [-P <packet list height>] [-r infile] [-R <read filter>]\n");
+  fprintf(stderr, "         [-t <time stamp format>] [-T <tree view height>]\n");
+#endif
 }
 
 /* And now our feature presentation... [ fade to music ] */
@@ -841,25 +866,27 @@ main(int argc, char *argv[])
 #ifndef WIN32
   int                  opt;
   extern char         *optarg;
+  gboolean             arg_error = FALSE;
 #endif
 #ifdef HAVE_LIBPCAP
-  extern char         pcap_version[];
+  extern char          pcap_version[];
 #endif
   char                *pf_path;
-  int                 pf_open_errno = 0;
-  int                 err;
+  int                  pf_open_errno = 0;
+  int                  err;
 #ifdef HAVE_LIBPCAP
-  gboolean            start_capture = FALSE;
-  gchar              *save_file = NULL;
+  gboolean             start_capture = FALSE;
+  gchar               *save_file = NULL;
+#else
+  gboolean             capture_option_specified = FALSE;
 #endif
-  GtkWidget           *window, *main_vbox, *menubar, *u_pane, *l_pane,
-                      *bv_table, *bv_hscroll, *bv_vscroll, *stat_hbox, 
-                      *tv_scrollw, *filter_bt, *filter_cm, *filter_te,
+  GtkWidget           *main_vbox, *menubar, *u_pane, *l_pane,
+                      *bv_table, *stat_hbox,
+                      *filter_bt, *filter_cm, *filter_te,
                       *filter_reset;
   GList               *filter_list = NULL;
   GtkStyle            *pl_style;
   GtkAccelGroup       *accel;
-  GtkWidget	      *packet_sw;
   gint                 pl_size = 280, tv_size = 95, bv_size = 75;
   gchar               *rc_file, *cf_name = NULL, *rfilter = NULL;
   dfilter             *rfcode = NULL;
@@ -920,6 +947,9 @@ main(int argc, char *argv[])
   cf.plist_end		= NULL;
   cf.wth		= NULL;
   cf.fh			= NULL;
+  cf.filename		= NULL;
+  cf.user_saved		= FALSE;
+  cf.is_tempfile	= FALSE;
   cf.rfcode		= NULL;
   cf.dfilter		= NULL;
   cf.dfcode		= NULL;
@@ -929,7 +959,6 @@ main(int argc, char *argv[])
   cf.iface		= NULL;
   cf.save_file		= NULL;
   cf.save_file_fd	= -1;
-  cf.user_saved		= 0;
   cf.snap		= WTAP_MAX_PACKET_SIZE;
   cf.count		= 0;
   cf.cinfo.num_cols	= prefs->num_cols;
@@ -964,6 +993,7 @@ main(int argc, char *argv[])
    "without libz", "",
 #endif /* HAVE_LIBZ */
 
+/* Oh, this is pretty */
 #if defined(HAVE_UCD_SNMP_SNMP_H)
 #ifdef HAVE_UCD_SNMP_VERSION_H
    "with UCD SNMP ", VersionInfo
@@ -992,19 +1022,40 @@ main(int argc, char *argv[])
         bv_size = atoi(optarg);
         break;
       case 'c':        /* Capture xxx packets */
-        cf.count = atoi(optarg);
-        break;
 #ifdef HAVE_LIBPCAP
-      case 'f':
-	cf.cfilter = g_strdup(optarg);
-	break;
+        cf.count = atoi(optarg);
+#else
+        capture_option_specified = TRUE;
+        arg_error = TRUE;
 #endif
+        break;
+      case 'f':
+#ifdef HAVE_LIBPCAP
+	cf.cfilter = g_strdup(optarg);
+#else
+        capture_option_specified = TRUE;
+        arg_error = TRUE;
+#endif
+	break;
       case 'h':        /* Print help and exit */
 	print_usage();
 	exit(0);
         break;
       case 'i':        /* Use interface xxx */
+#ifdef HAVE_LIBPCAP
         cf.iface = g_strdup(optarg);
+#else
+        capture_option_specified = TRUE;
+        arg_error = TRUE;
+#endif
+        break;
+      case 'k':        /* Start capture immediately */
+#ifdef HAVE_LIBPCAP
+        start_capture = TRUE;
+#else
+        capture_option_specified = TRUE;
+        arg_error = TRUE;
+#endif
         break;
       case 'm':        /* Medium font */
 	medium_font = g_strdup(optarg);
@@ -1012,34 +1063,43 @@ main(int argc, char *argv[])
       case 'n':        /* No name resolution */
 	g_resolving_actif = 0;
 	break;
-#ifdef HAVE_LIBPCAP
-      case 'k':        /* Start capture immediately */
-        start_capture = TRUE;
-        break;
-#endif
       case 'P':        /* Packet list pane height */
         pl_size = atoi(optarg);
         break;
-#ifdef HAVE_LIBPCAP
       case 'Q':        /* Quit after capture (just capture to file) */
+#ifdef HAVE_LIBPCAP
         quit_after_cap = 1;
         start_capture = TRUE;  /*** -Q implies -k !! ***/
-        break;
+#else
+        capture_option_specified = TRUE;
+        arg_error = TRUE;
 #endif
+        break;
       case 'r':        /* Read capture file xxx */
+	/* We may set "last_open_dir" to "cf_name", and if we change
+	   "last_open_dir" later, we free the old value, so we have to
+	   set "cf_name" to something that's been allocated. */
         cf_name = g_strdup(optarg);
         break;
       case 'R':        /* Read file filter */
         rfilter = optarg;
         break;
-#ifdef HAVE_LIBPCAP
       case 's':        /* Set the snapshot (capture) length */
+#ifdef HAVE_LIBPCAP
         cf.snap = atoi(optarg);
+#else
+        capture_option_specified = TRUE;
+        arg_error = TRUE;
+#endif
         break;
       case 'S':        /* "Sync" mode: used for following file ala tail -f */
+#ifdef HAVE_LIBPCAP
         sync_mode = TRUE;
-        break;
+#else
+        capture_option_specified = TRUE;
+        arg_error = TRUE;
 #endif
+        break;
       case 't':        /* Time stamp type */
         if (strcmp(optarg, "r") == 0)
           timestamp_type = RELATIVE;
@@ -1062,21 +1122,36 @@ main(int argc, char *argv[])
         printf("%s %s, with %s\n", PACKAGE, VERSION, comp_info_str);
         exit(0);
         break;
-#ifdef HAVE_LIBPCAP
       case 'w':        /* Write to capture file xxx */
+#ifdef HAVE_LIBPCAP
         save_file = g_strdup(optarg);
+#else
+        capture_option_specified = TRUE;
+        arg_error = TRUE;
+#endif
 	break;
       case 'W':        /* Write to capture file FD xxx */
+#ifdef HAVE_LIBPCAP
         cf.save_file_fd = atoi(optarg);
-	break;
+#else
+        capture_option_specified = TRUE;
+        arg_error = TRUE;
 #endif
+	break;
+      default:
       case '?':        /* Bad flag - print usage message */
-        print_usage();
+        arg_error = TRUE;
         break;
     }
   }
 #endif
 
+#ifndef HAVE_LIBPCAP
+  if (capture_option_specified)
+    fprintf(stderr, "This version of Ethereal was not built with support for capturing packets.\n");
+#endif
+  if (arg_error)
+    print_usage();
 #ifdef HAVE_LIBPCAP
   if (start_capture) {
     if (cf.iface == NULL) {
@@ -1127,25 +1202,25 @@ main(int argc, char *argv[])
   }
 
   /* Main window */  
-  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-  gtk_widget_set_name(window, "main window");
-  gtk_signal_connect(GTK_OBJECT(window), "delete_event",
+  top_level = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+  gtk_widget_set_name(top_level, "main window");
+  gtk_signal_connect(GTK_OBJECT(top_level), "delete_event",
     GTK_SIGNAL_FUNC(file_quit_cmd_cb), "WM destroy");
-  gtk_signal_connect(GTK_OBJECT(window), "destroy", 
+  gtk_signal_connect(GTK_OBJECT(top_level), "destroy", 
     GTK_SIGNAL_FUNC(file_quit_cmd_cb), "WM destroy");
-  gtk_window_set_title(GTK_WINDOW(window), "The Ethereal Network Analyzer");
-  gtk_widget_set_usize(GTK_WIDGET(window), DEF_WIDTH, -1);
-  gtk_window_set_policy(GTK_WINDOW(window), TRUE, TRUE, FALSE);
+  gtk_window_set_title(GTK_WINDOW(top_level), "The Ethereal Network Analyzer");
+  gtk_widget_set_usize(GTK_WIDGET(top_level), DEF_WIDTH, -1);
+  gtk_window_set_policy(GTK_WINDOW(top_level), TRUE, TRUE, FALSE);
 
   /* Container for menu bar, paned windows and progress/info box */
   main_vbox = gtk_vbox_new(FALSE, 1);
   gtk_container_border_width(GTK_CONTAINER(main_vbox), 1);
-  gtk_container_add(GTK_CONTAINER(window), main_vbox);
+  gtk_container_add(GTK_CONTAINER(top_level), main_vbox);
   gtk_widget_show(main_vbox);
 
   /* Menu bar */
   get_main_menu(&menubar, &accel);
-  gtk_window_add_accel_group(GTK_WINDOW(window), accel);
+  gtk_window_add_accel_group(GTK_WINDOW(top_level), accel);
   gtk_box_pack_start(GTK_BOX(main_vbox), menubar, FALSE, TRUE, 0);
   gtk_widget_show(menubar);
 
@@ -1154,17 +1229,22 @@ main(int argc, char *argv[])
   gtk_paned_gutter_size(GTK_PANED(u_pane), (GTK_PANED(u_pane))->handle_size);
   l_pane = gtk_vpaned_new();
   gtk_paned_gutter_size(GTK_PANED(l_pane), (GTK_PANED(l_pane))->handle_size);
-  gtk_container_add(GTK_CONTAINER(main_vbox), u_pane);
+  gtk_container_add(GTK_CONTAINER(main_vbox), l_pane);
   gtk_widget_show(u_pane);
-  gtk_paned_add2 (GTK_PANED(u_pane), l_pane);
+  gtk_paned_add1 (GTK_PANED(l_pane), u_pane);
   gtk_widget_show(l_pane);
 
   /* Packet list */
+  pkt_scrollw = gtk_scrolled_window_new(NULL, NULL);
+  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(pkt_scrollw),
+    GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+  gtk_widget_show(pkt_scrollw);
+  gtk_paned_add1(GTK_PANED(u_pane), pkt_scrollw);
+
   packet_list = gtk_clist_new_with_titles(cf.cinfo.num_cols, cf.cinfo.col_title);
+  gtk_container_add(GTK_CONTAINER(pkt_scrollw), packet_list);
   gtk_clist_column_titles_passive(GTK_CLIST(packet_list));
-  packet_sw = gtk_scrolled_window_new(NULL, NULL);
-  gtk_widget_show(packet_sw);
-  gtk_container_add(GTK_CONTAINER(packet_sw), packet_list);
+  set_plist_sel_browse(prefs->gui_plist_sel_browse);
   pl_style = gtk_style_new();
   gdk_font_unref(pl_style->font);
   pl_style->font = m_r_font;
@@ -1189,57 +1269,69 @@ main(int argc, char *argv[])
 						pl_style->font);
   }
   gtk_widget_set_usize(packet_list, -1, pl_size);
-  gtk_paned_add1(GTK_PANED(u_pane), packet_sw);
   gtk_widget_show(packet_list);
   
   /* Tree view */
   tv_scrollw = gtk_scrolled_window_new(NULL, NULL);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(tv_scrollw),
     GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-  gtk_paned_add1(GTK_PANED(l_pane), tv_scrollw);
+  gtk_paned_add2(GTK_PANED(u_pane), tv_scrollw);
   gtk_widget_set_usize(tv_scrollw, -1, tv_size);
   gtk_widget_show(tv_scrollw);
   
-  tree_view = gtk_tree_new();
-  gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(tv_scrollw),
-		  tree_view);
-  gtk_tree_set_selection_mode(GTK_TREE(tree_view), GTK_SELECTION_SINGLE);
-
-  /* XXX - what's the difference between the next two lines? */
-  gtk_tree_set_view_lines(GTK_TREE(tree_view), FALSE);
-  gtk_tree_set_view_mode(GTK_TREE(tree_view), GTK_TREE_VIEW_ITEM);
-
-  gtk_signal_connect(GTK_OBJECT(tree_view), "selection_changed",
-    GTK_SIGNAL_FUNC(tree_view_cb), NULL);
+  tree_view = gtk_ctree_new(1, 0);
+  gtk_container_add(GTK_CONTAINER(tv_scrollw), tree_view);
+  gtk_ctree_set_line_style(GTK_CTREE(tree_view), GTK_CTREE_LINES_NONE);
+  gtk_ctree_set_expander_style(GTK_CTREE(tree_view), GTK_CTREE_EXPANDER_SQUARE);
+  set_ptree_sel_browse(prefs->gui_ptree_sel_browse);
+  gtk_signal_connect(GTK_OBJECT(tree_view), "tree-select-row",
+    GTK_SIGNAL_FUNC(tree_view_select_row_cb), NULL);
+  gtk_signal_connect(GTK_OBJECT(tree_view), "tree-unselect-row",
+    GTK_SIGNAL_FUNC(tree_view_unselect_row_cb), NULL);
   gtk_widget_show(tree_view);
 
   item_style = gtk_style_new();
   gdk_font_unref(item_style->font);
   item_style->font = m_r_font;
 
-  /* Byte view */
-  bv_table = gtk_table_new (2, 2, FALSE);
-  gtk_paned_add2(GTK_PANED(l_pane), bv_table);
+  /* Byte view. The table is only one row high, but 3 columns
+   * wide. The middle column contains the GtkText with the hex dump.
+   * The left and right columns contain vertical scrollbars. They both
+   * do the same thing, but only one will be shown at a time, in accordance
+   * with where the user wants the other vertical scrollbars places
+   * (on the left or the right).
+   */
+  bv_table = gtk_table_new (1, 3, FALSE);
+  gtk_paned_pack2(GTK_PANED(l_pane), bv_table, FALSE, FALSE);
   gtk_widget_set_usize(bv_table, -1, bv_size);
   gtk_widget_show(bv_table);
 
   byte_view = gtk_text_new(NULL, NULL);
   gtk_text_set_editable(GTK_TEXT(byte_view), FALSE);
   gtk_text_set_word_wrap(GTK_TEXT(byte_view), FALSE);
-  gtk_table_attach (GTK_TABLE (bv_table), byte_view, 0, 1, 0, 1,
+  gtk_table_attach (GTK_TABLE (bv_table), byte_view, 1, 2, 0, 1,
     GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND | GTK_SHRINK, 0, 0);
   gtk_widget_show(byte_view);
 
-  bv_hscroll = gtk_hscrollbar_new(GTK_TEXT(byte_view)->hadj);
-  gtk_table_attach(GTK_TABLE(bv_table), bv_hscroll, 0, 1, 1, 2,
-    GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
-  gtk_widget_show (bv_hscroll);
+  /* The gtk_text widget doesn't scroll horizontally (see gtktext.c)
+   * in the GTK+ distribution, so I removed the horizontal scroll bar
+   * that used to be here. I tried the gtk_text widget with a 
+   * gtk_scrolled_window w/ viewport, but the vertical scrollowing
+   * did not work well, and sometimes a few pixels were cut off on
+   * the bottom. */
 
-  bv_vscroll = gtk_vscrollbar_new(GTK_TEXT(byte_view)->vadj);
-  gtk_table_attach(GTK_TABLE(bv_table), bv_vscroll, 1, 2, 0, 1,
+  bv_vscroll_left = gtk_vscrollbar_new(GTK_TEXT(byte_view)->vadj);
+  gtk_table_attach(GTK_TABLE(bv_table), bv_vscroll_left, 0, 1, 0, 1,
     GTK_FILL, GTK_EXPAND | GTK_FILL | GTK_SHRINK, 0, 0);
-  gtk_widget_show(bv_vscroll);
-  
+
+  bv_vscroll_right = gtk_vscrollbar_new(GTK_TEXT(byte_view)->vadj);
+  gtk_table_attach(GTK_TABLE(bv_table), bv_vscroll_right, 2, 3, 0, 1,
+    GTK_FILL, GTK_EXPAND | GTK_FILL | GTK_SHRINK, 0, 0);
+ 
+  /* Now that the 3 panes are created, set the vertical scrollbar
+   * on the left or right according to the user's preference */
+  set_scrollbar_placement(prefs->gui_scrollbar_on_right);
+
   /* Progress/filter/info box */
   stat_hbox = gtk_hbox_new(FALSE, 1);
   gtk_container_border_width(GTK_CONTAINER(stat_hbox), 0);
@@ -1252,7 +1344,7 @@ main(int argc, char *argv[])
 
   filter_bt = gtk_button_new_with_label("Filter:");
   gtk_signal_connect(GTK_OBJECT(filter_bt), "clicked",
-    GTK_SIGNAL_FUNC(prefs_cb), (gpointer) E_PR_PG_FILTER);
+    GTK_SIGNAL_FUNC(filter_dialog_cb), NULL);
   gtk_box_pack_start(GTK_BOX(stat_hbox), filter_bt, FALSE, TRUE, 0);
   gtk_widget_show(filter_bt);
   
@@ -1281,6 +1373,7 @@ main(int argc, char *argv[])
    * that text entry pointer */
   set_menu_object_data("/File/Open...", E_DFILTER_TE_KEY, filter_te);
   set_menu_object_data("/File/Reload", E_DFILTER_TE_KEY, filter_te);
+  set_menu_object_data("/Edit/Filters...", E_FILT_TE_PTR_KEY, filter_te);
   set_menu_object_data("/Display/Match Selected", E_DFILTER_TE_KEY, filter_te);
   set_menu_object_data("/Tools/Follow TCP Stream", E_DFILTER_TE_KEY, filter_te);
 
@@ -1306,9 +1399,9 @@ main(int argc, char *argv[])
     /* No.  Pop up the main window, and read in a capture file if
        we were told to. */
 
-    gtk_widget_show(window);
+    gtk_widget_show(top_level);
 
-    colors_init(&cf);
+    cf.colors = colfilter_new();
 
     /* If we were given the name of a capture file, read it in now;
        we defer it until now, so that, if we can't open it, and pop
@@ -1324,7 +1417,7 @@ main(int argc, char *argv[])
         }
       }
       if (!rfilter_parse_failed) {
-        if ((err = open_cap_file(cf_name, &cf)) == 0) {
+        if ((err = open_cap_file(cf_name, FALSE, &cf)) == 0) {
           /* "open_cap_file()" succeeded, so it closed the previous
 	     capture file, and thus destroyed any previous read filter
 	     attached to "cf". */
@@ -1335,7 +1428,6 @@ main(int argc, char *argv[])
             last_open_dir = cf_name;
             *s = '\0';
           }
-          set_menu_sensitivity("/File/Save As...", TRUE);
         } else {
           dfilter_destroy(rfcode);
           cf.rfcode = NULL;
