@@ -1,7 +1,7 @@
 /* gtkpacket.c
  * Routines for GTK+ packet display
  *
- * $Id: proto_draw.c,v 1.6 1999/11/22 06:24:55 gram Exp $
+ * $Id: proto_draw.c,v 1.12 1999/12/29 20:10:12 gram Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -50,7 +50,6 @@
 #define BYTE_VIEW_WIDTH    16
 #define BYTE_VIEW_SEP      8
 
-extern GtkWidget    *byte_view;
 extern GdkFont      *m_r_font, *m_b_font;
 
 static void
@@ -62,13 +61,25 @@ packet_hex_print(GtkText *bv, guint8 *pd, gint len, gint bstart, gint blen,
   gint     i = 0, j, k, cur;
   gchar    line[128], hexchars[] = "0123456789abcdef", c = '\0';
   GdkFont *cur_font, *new_font;
+  gint	   bend = -1;
+
+  /* Freeze the text for faster display */
+  gtk_text_freeze(bv);
+
+  /* Clear out the text */
+  gtk_text_set_point(bv, 0);
+  gtk_text_forward_delete(bv, gtk_text_get_length(bv));
+
+  if (bstart >= 0 && blen >= 0) {
+	  bend = bstart + blen;
+  }
 
   while (i < len) {
     /* Print the line number */
     sprintf(line, "%04x  ", i);
     gtk_text_insert(bv, m_r_font, NULL, NULL, line, -1);
     /* Do we start in bold? */
-    cur_font = (i >= bstart && i < (bstart + blen)) ? m_b_font : m_r_font;
+    cur_font = (i >= bstart && i < bend) ? m_b_font : m_r_font;
     j   = i;
     k   = i + BYTE_VIEW_WIDTH;
     cur = 0;
@@ -85,7 +96,7 @@ packet_hex_print(GtkText *bv, guint8 *pd, gint len, gint bstart, gint blen,
       /* insert a space every BYTE_VIEW_SEP bytes */
       if( ( i % BYTE_VIEW_SEP ) == 0 ) line[cur++] = ' ';
       /* Did we cross a bold/plain boundary? */
-      new_font = (i >= bstart && i < (bstart + blen)) ? m_b_font : m_r_font;
+      new_font = (i >= bstart && i < bend) ? m_b_font : m_r_font;
       if (cur_font != new_font) {
         gtk_text_insert(bv, cur_font, NULL, NULL, line, cur);
         cur_font = new_font;
@@ -98,7 +109,7 @@ packet_hex_print(GtkText *bv, guint8 *pd, gint len, gint bstart, gint blen,
     cur = 0;
     i = j;
     /* Print the ASCII bit */
-    cur_font = (i >= bstart && i < (bstart + blen)) ? m_b_font : m_r_font;
+    cur_font = (i >= bstart && i < bend) ? m_b_font : m_r_font;
     while (i < k) {
       if (i < len) {
 	      if (encoding == CHAR_ASCII) {
@@ -118,7 +129,7 @@ packet_hex_print(GtkText *bv, guint8 *pd, gint len, gint bstart, gint blen,
       /* insert a space every BYTE_VIEW_SEP bytes */
       if( ( i % BYTE_VIEW_SEP ) == 0 ) line[cur++] = ' ';
       /* Did we cross a bold/plain boundary? */
-      new_font = (i >= bstart && i < (bstart + blen)) ? m_b_font : m_r_font;
+      new_font = (i >= bstart && i < bend) ? m_b_font : m_r_font;
       if (cur_font != new_font) {
         gtk_text_insert(bv, cur_font, NULL, NULL, line, cur);
         cur_font = new_font;
@@ -129,6 +140,20 @@ packet_hex_print(GtkText *bv, guint8 *pd, gint len, gint bstart, gint blen,
     line[cur]   = '\0';
     gtk_text_insert(bv, cur_font, NULL, NULL, line, -1);
   }
+
+  /* scroll text into position */
+  gtk_text_thaw(bv); /* must thaw before adjusting scroll bars */
+  if ( bstart > 0 ) {
+    int lineheight, linenum;
+    float scrollval;
+    linenum = bstart / BYTE_VIEW_WIDTH;
+
+    /* need to change to some way of getting that offset instead of +4 */
+    lineheight = gdk_string_height(m_b_font, "0") + 4;
+    scrollval = MIN(linenum * lineheight,bv->vadj->upper - bv->vadj->page_size);
+
+    gtk_adjustment_set_value(bv->vadj, scrollval);
+  }
 }
 
 void expand_all_tree(proto_tree *protocol_tree, GtkWidget *tree_view) {
@@ -136,8 +161,9 @@ void expand_all_tree(proto_tree *protocol_tree, GtkWidget *tree_view) {
   for(i=0; i < num_tree_types; i++) {
     tree_is_expanded[i] = TRUE;
   }
-  gtk_tree_clear_items(GTK_TREE(tree_view), 0, -1);
+  gtk_clist_clear ( GTK_CLIST(tree_view) );
   proto_tree_draw(protocol_tree, tree_view);
+  gtk_ctree_expand_recursive(GTK_CTREE(tree_view), NULL);
 }
 
 void collapse_all_tree(proto_tree *protocol_tree, GtkWidget *tree_view) {
@@ -145,43 +171,73 @@ void collapse_all_tree(proto_tree *protocol_tree, GtkWidget *tree_view) {
   for(i=0; i < num_tree_types; i++) {
     tree_is_expanded[i] = FALSE;
   }
-  gtk_tree_clear_items(GTK_TREE(tree_view), 0, -1);
+  gtk_clist_clear ( GTK_CLIST(tree_view) );
   proto_tree_draw(protocol_tree, tree_view);
 }
 
 static void
-expand_tree(GtkWidget *w, gpointer data) {
-  gboolean *val = (gint *) data;
-  *val = TRUE;
-}
-
-static void
-collapse_tree(GtkWidget *w, gpointer data) {
-  gboolean *val = (gint *) data;
-  *val = FALSE;
-}
-
-static void
-set_item_style(GtkWidget *widget, gpointer dummy)
+expand_tree(GtkCTree *ctree, GList *node, gpointer user_data)
 {
-  gtk_widget_set_style(widget, item_style);
+	field_info	*finfo;
+	gboolean	*val;
+
+	finfo = gtk_ctree_node_get_row_data( ctree, GTK_CTREE_NODE(node) );
+	g_assert(finfo);
+
+	val = &tree_is_expanded[finfo->tree_type];
+	*val = TRUE;
 }
 
+static void
+collapse_tree(GtkCTree *ctree, GList *node, gpointer user_data)
+{
+	field_info	*finfo;
+	gboolean	*val;
+
+	finfo = gtk_ctree_node_get_row_data( ctree, GTK_CTREE_NODE(node) );
+	g_assert(finfo);
+
+	val = &tree_is_expanded[finfo->tree_type];
+	*val = FALSE;
+}
+
+struct proto_tree_draw_info {
+	GtkCTree	*ctree;
+	GtkCTreeNode	*ctree_node;
+};
+
 void
 proto_tree_draw(proto_tree *protocol_tree, GtkWidget *tree_view)
 {
+	struct proto_tree_draw_info	info;
+
+	info.ctree = GTK_CTREE(tree_view);
+	info.ctree_node = NULL;
+
+	gtk_clist_freeze ( GTK_CLIST(tree_view) );
+
 	g_node_children_foreach((GNode*) protocol_tree, G_TRAVERSE_ALL,
-		proto_tree_draw_node, tree_view);
+		proto_tree_draw_node, &info);
+
+	gtk_signal_connect( GTK_OBJECT(info.ctree), "tree-expand",
+		(GtkSignalFunc) expand_tree, NULL );
+	gtk_signal_connect( GTK_OBJECT(info.ctree), "tree-collapse",
+		(GtkSignalFunc) collapse_tree, NULL );
+
+	gtk_clist_thaw ( GTK_CLIST(tree_view) );
 }
 
 static void
 proto_tree_draw_node(GNode *node, gpointer data)
 {
-	GtkWidget	*tree_view = (GtkWidget*) data;
+	struct proto_tree_draw_info	info;
+	struct proto_tree_draw_info	*parent_info = (struct proto_tree_draw_info*) data;
+
 	field_info	*fi = (field_info*) (node->data);
-	GtkWidget	*ti, *subtree;
 	gchar		label_str[ITEM_LABEL_LENGTH];
 	gchar		*label_ptr;
+	GtkCTreeNode	*parent;
+	gboolean	is_leaf, is_expanded;
 
 	if (!fi->visible)
 		return;
@@ -194,25 +250,32 @@ proto_tree_draw_node(GNode *node, gpointer data)
 		label_ptr = label_str;
 		proto_item_fill_label(fi, label_str);
 	}
-		
-	ti = gtk_tree_item_new_with_label(label_ptr);
-	gtk_container_foreach(GTK_CONTAINER(ti), set_item_style, NULL);
-	
-	gtk_object_set_data(GTK_OBJECT(ti), E_TREEINFO_FIELD_INFO_KEY, (gpointer) fi);
-	gtk_tree_append(GTK_TREE(tree_view), ti);
-	gtk_widget_show(ti);
 
 	if (g_node_n_children(node) > 0) {
-		subtree = gtk_tree_new();
-		gtk_tree_item_set_subtree(GTK_TREE_ITEM(ti), GTK_WIDGET(subtree));
-		if (tree_is_expanded[fi->tree_type])
-			gtk_tree_item_expand(GTK_TREE_ITEM(ti));
-		gtk_signal_connect(GTK_OBJECT(ti), "expand", (GtkSignalFunc) expand_tree,
-			(gpointer) &tree_is_expanded[fi->tree_type]);
-		gtk_signal_connect(GTK_OBJECT(ti), "collapse", (GtkSignalFunc) collapse_tree,
-			(gpointer) &tree_is_expanded[fi->tree_type]);
+		is_leaf = FALSE;
+		if (tree_is_expanded[fi->tree_type]) {
+			is_expanded = TRUE;
+		}
+		else {
+			is_expanded = FALSE;
+		}
+	}
+	else {
+		is_leaf = TRUE;
+		is_expanded = FALSE;
+	}
+	
+	info.ctree = parent_info->ctree;
+	parent = gtk_ctree_insert_node ( info.ctree, parent_info->ctree_node, NULL,
+			&label_ptr, 5, NULL, NULL, NULL, NULL,
+			is_leaf, is_expanded );
 
+	gtk_ctree_node_set_row_data( GTK_CTREE(info.ctree), parent, fi );
+	gtk_ctree_node_set_row_style( GTK_CTREE(info.ctree), parent, item_style);
+
+	if (!is_leaf) {
+		info.ctree_node = parent;
 		g_node_children_foreach(node, G_TRAVERSE_ALL,
-			proto_tree_draw_node, subtree);
+			proto_tree_draw_node, &info);
 	}
 }
