@@ -1,7 +1,7 @@
 /* gtkpacket.h
  * Definitions for GTK+ packet display structures and routines
  *
- * $Id: proto_draw.h,v 1.2 1999/09/11 12:38:18 deniel Exp $
+ * $Id: proto_draw.h,v 1.4 1999/11/22 06:24:56 gram Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -27,10 +27,9 @@
 #ifndef __GTKPACKET_H__
 #define __GTKPACKET_H__
 
-void       packet_hex_print(GtkText *, guint8 *, gint, gint, gint);
+void packet_hex_print(GtkText *, guint8 *, gint, gint, gint, char_enc);
 
-#define E_TREEINFO_START_KEY "tree_info_start"
-#define E_TREEINFO_LEN_KEY   "tree_info_len"
+#define E_TREEINFO_FIELD_INFO_KEY "tree_info_finfo"
 
 void proto_tree_draw(proto_tree *protocol_tree, GtkWidget *tree_view);
 void expand_all_tree(proto_tree *protocol_tree, GtkWidget *tree_view);
