@@ -1,7 +1,7 @@
 /* packet-ascend.c
  * Routines for decoding Lucent/Ascend packet traces
  *
- * $Id: packet-ascend.c,v 1.4 1999/09/13 03:48:58 gerald Exp $
+ * $Id: packet-ascend.c,v 1.8 1999/11/16 11:42:25 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -34,9 +34,14 @@
 #include "packet.h"
 
 static int proto_ascend  = -1;
+static int hf_link_type  = -1;
 static int hf_session_id = -1;
+static int hf_called_number = -1;
 static int hf_chunk      = -1;
 static int hf_task       = -1;
+static int hf_user_name  = -1;
+
+static gint ett_raw = -1;
 
 static const value_string encaps_vals[] = {
   {ASCEND_PFX_WDS_X, "PPP Transmit"},
@@ -64,32 +69,23 @@ dissect_ascend( const u_char *pd, frame_data *fd, proto_tree *tree ) {
      layer (ie none) */
   if(tree) {
     ti = proto_tree_add_text(tree, 0, 0, "Lucent/Ascend packet trace" );
-    fh_tree = proto_item_add_subtree(ti, ETT_RAW);
-
-    /* XXX - should these be added with "proto_tree_add_item_format()"
-       (see "dissect_packet()" for an example of how to add items
-       that aren't in the packet data in that fashion) so that we
-       can filter on them? */
-    proto_tree_add_text(fh_tree, 0, 0, "Link type: %s",
-      val_to_str(fd->pseudo_header.ascend.type, encaps_vals, "Unknown (%d)"));
+    fh_tree = proto_item_add_subtree(ti, ett_raw);
+    proto_tree_add_item(fh_tree, hf_link_type, 0, 0, 
+			fd->pseudo_header.ascend.type);
     if (fd->pseudo_header.ascend.type == ASCEND_PFX_WDD) {
-      proto_tree_add_text(fh_tree, 0, 0, "Called number: %s",
-        fd->pseudo_header.ascend.call_num);
-      proto_tree_add_item_format(fh_tree, hf_chunk, 0, 0,
-        fd->pseudo_header.ascend.chunk, "Chunk: 0x%08x",
-        fd->pseudo_header.ascend.chunk);
+      proto_tree_add_item(fh_tree, hf_called_number, 0, 0, 
+			  fd->pseudo_header.ascend.call_num);
+      proto_tree_add_item(fh_tree, hf_chunk, 0, 0,
+			  fd->pseudo_header.ascend.chunk);
       proto_tree_add_item_hidden(fh_tree, hf_session_id, 0, 0, 0);
     } else {  /* It's wandsession data */
-      proto_tree_add_text(fh_tree, 0, 0, "Username: %s",
-        fd->pseudo_header.ascend.user);
-      proto_tree_add_item_format(fh_tree, hf_session_id, 0, 0,
-        fd->pseudo_header.ascend.sess, "Session: %d",
-        fd->pseudo_header.ascend.sess);
+      proto_tree_add_item(fh_tree, hf_user_name, 0, 0, 
+			  fd->pseudo_header.ascend.user);
+      proto_tree_add_item(fh_tree, hf_session_id, 0, 0,
+			  fd->pseudo_header.ascend.sess);
       proto_tree_add_item_hidden(fh_tree, hf_chunk, 0, 0, 0);
     }
-    proto_tree_add_item_format(fh_tree, hf_task, 0, 0,
-      fd->pseudo_header.ascend.task, "Task: 0x%08X",
-      fd->pseudo_header.ascend.task);
+    proto_tree_add_item(fh_tree, hf_task, 0, 0, fd->pseudo_header.ascend.task);
   }
 
   switch (fd->pseudo_header.ascend.type) {
@@ -109,17 +105,35 @@ void
 proto_register_ascend(void)
 {
   static hf_register_info hf[] = {
+    { &hf_link_type,
+    { "Link type",	"ascend.type",	FT_UINT32, BASE_DEC,	VALS(encaps_vals),	0x0,
+      "" }},
+
     { &hf_session_id,
-    { "Session ID",	"ascend.sess",	FT_UINT32,	NULL }},
+    { "Session ID",	"ascend.sess",	FT_UINT32, BASE_DEC,	NULL, 0x0,
+      "" }},
+
+    { &hf_called_number,
+    { "Called number",	"ascend.number", FT_STRING, BASE_NONE,	NULL, 0x0,
+      "" }},
 
     { &hf_chunk,
-    { "WDD Chunk",	"ascend.chunk",	FT_UINT32,	NULL }},
+    { "WDD Chunk",	"ascend.chunk",	FT_UINT32, BASE_HEX,	NULL, 0x0,
+      "" }},
 
     { &hf_task,
-    { "Task",		"ascend.task",	FT_UINT32,	NULL }}
+    { "Task",		"ascend.task",	FT_UINT32, BASE_HEX,	NULL, 0x0,
+      "" }},
+
+    { &hf_user_name,
+    { "User name",     	"ascend.user",	FT_STRING, BASE_NONE,	NULL, 0x0,
+      "" }},
+  };
+  static gint *ett[] = {
+    &ett_raw,
   };
 
   proto_ascend = proto_register_protocol("Lucent/Ascend debug output", "ascend");
   proto_register_field_array(proto_ascend, hf, array_length(hf));
+  proto_register_subtree_array(ett, array_length(ett));
 }
-
