@@ -3,7 +3,7 @@
  *
  * Laurent Deniel <deniel@worldnet.fr>
  *
- * $Id: packet-giop.c,v 1.6 1999/09/17 05:56:53 guy Exp $
+ * $Id: packet-giop.c,v 1.10 1999/11/16 11:42:30 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -41,6 +41,10 @@
 #include "packet.h"
 
 static int proto_giop = -1;
+static int hf_giop_message_type = -1;
+static int hf_giop_message_size = -1;
+
+static gint ett_giop = -1;
 
 /*
  * GIOP / IIOP types definition - OMG CORBA 2.x / GIOP 1.[01]
@@ -165,7 +169,7 @@ typedef struct LocateReplyHeader {
 } LocateReplyHeader;
 
 
-u_char *print_object_key(int length, u_char *from) 
+static u_char *print_object_key(int length, u_char *from) 
 {
 #define MAX_OBJECT_KEY_LENGTH 64
   static u_char buffer[MAX_OBJECT_KEY_LENGTH];
@@ -262,7 +266,7 @@ void dissect_giop(const u_char *pd, int offset, frame_data *fd, proto_tree *tree
   if (tree) {
     ti = proto_tree_add_item(tree, proto_giop, offset, 
 			  GIOP_HEADER_SIZE + message_size, NULL);
-    clnp_tree = proto_item_add_subtree(ti, ETT_GIOP);
+    clnp_tree = proto_item_add_subtree(ti, ett_giop);
     proto_tree_add_text(clnp_tree, offset,      4,
 		     "Magic number: %s", GIOP_MAGIC);
     proto_tree_add_text(clnp_tree, offset +  4, 2, 
@@ -286,19 +290,24 @@ void dissect_giop(const u_char *pd, int offset, frame_data *fd, proto_tree *tree
 	break;
     } /* minor_version */
 
-    proto_tree_add_text(clnp_tree, offset +  7, 1, 
-		     "Message type: %s",
-		     (header.message_type == Request) ? "Request" :
-		     (header.message_type == Reply) ? "Reply" :
-		     (header.message_type == CancelRequest) ? "CancelRequest" :
-		     (header.message_type == LocateRequest) ? "LocateRequest" :
-		     (header.message_type == LocateReply) ? "LocateReply" :
-		     (header.message_type == CloseConnection) ? "CloseConnection" :
-		     (header.message_type == MessageError) ? "MessageError" :
-		     (header.message_type == Fragment) ? "Fragment" : "?");
+    proto_tree_add_item_format(clnp_tree, 
+			       hf_giop_message_type,
+			       offset +  7, 1, 
+			       header.message_type,
+			       "Message type: %s",
+			       (header.message_type == Request) ? "Request" :
+			       (header.message_type == Reply) ? "Reply" :
+			       (header.message_type == CancelRequest) ? "CancelRequest" :
+			       (header.message_type == LocateRequest) ? "LocateRequest" :
+			       (header.message_type == LocateReply) ? "LocateReply" :
+			       (header.message_type == CloseConnection) ? "CloseConnection" :
+			       (header.message_type == MessageError) ? "MessageError" :
+			       (header.message_type == Fragment) ? "Fragment" : "?");
 
-    proto_tree_add_text(clnp_tree, offset +  8, 4, 
-		     "Message size: %d", message_size);
+    proto_tree_add_item(clnp_tree, 
+			hf_giop_message_size,
+			offset +  8, 4, 
+			message_size);
 
   } /* tree */
 
@@ -697,14 +706,22 @@ void dissect_giop(const u_char *pd, int offset, frame_data *fd, proto_tree *tree
 
 } /* dissect_giop */
 
-void
+void 
 proto_register_giop(void)
 {
-/*        static hf_register_info hf[] = {
-                { &variable,
-                { "Name",           "giop.abbreviation", TYPE, VALS_POINTER }},
-        };*/
+  static hf_register_info hf[] = {
+    { &hf_giop_message_type,
+      { "Message type",		"giop.type",	FT_UINT8,	BASE_DEC, NULL, 0x0,
+      	"" }},
+    { &hf_giop_message_size,
+      { "Message size",		"giop.len",	FT_UINT32,	BASE_DEC, NULL, 0x0,
+      	"" }},
+  };
+  static gint *ett[] = {
+    &ett_giop,
+  };
 
-        proto_giop = proto_register_protocol("General Inter-ORB Protocol", "giop");
- /*       proto_register_field_array(proto_giop, hf, array_length(hf));*/
+  proto_giop = proto_register_protocol("General Inter-ORB Protocol", "giop");
+  proto_register_field_array(proto_giop, hf, array_length(hf));
+  proto_register_subtree_array(ett, array_length(ett));
 }
