@@ -1,7 +1,7 @@
 /* packet-ip.h
  * Definitions for IP packet disassembly structures and routines
  *
- * $Id: packet-ip.h,v 1.5 1999/08/28 08:31:27 guy Exp $
+ * $Id: packet-ip.h,v 1.9 1999/11/21 14:43:53 gram Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -27,12 +27,6 @@
 #ifndef __PACKET_IP_H__
 #define __PACKET_IP_H__
 
-#define IP_PROTO_ICMP  1
-#define IP_PROTO_IGMP  2
-#define IP_PROTO_TCP   6
-#define IP_PROTO_UDP  17
-#define IP_PROTO_OSPF 89
-
 #define IP_PROTO_IP		0		/* dummy for IP */
 #define IP_PROTO_HOPOPTS	0		/* IP6 hop-by-hop options */
 #define IP_PROTO_ICMP		1		/* control message protocol */
@@ -57,9 +51,12 @@
 #define IP_PROTO_NONE		59		/* IP6 no next header */
 #define IP_PROTO_DSTOPTS	60		/* IP6 no next header */
 #define IP_PROTO_EON		80		/* ISO cnlp */
+#define IP_PROTO_EIGRP   88
 #define IP_PROTO_OSPF		89
 #define IP_PROTO_ENCAP		98		/* encapsulation header */
 #define IP_PROTO_PIM		103		/* Protocol Independent Mcast */
+#define IP_PROTO_IPCOMP		108		/* IP payload compression */
+#define IP_PROTO_VRRP		112		/* Virtual Router Redundancy Protocol */
 
 typedef enum {
   NO_LENGTH,		/* option has no data, hence no length */
@@ -70,8 +67,8 @@ typedef enum {
 /* Member of table of IP or TCP options. */
 typedef struct ip_tcp_opt {
   int   optcode;	/* code for option */
-  char *name;		/* name of option */
-  int   subtree_index;	/* ETT_ value for option */
+  char  *name;		/* name of option */
+  int   *subtree_index;	/* pointer to subtree index for option */
   opt_len_type len_type; /* type of option length field */
   int	optlen;		/* value length should be (minimum if VARIABLE) */
   void	(*dissect)(const struct ip_tcp_opt *, const u_char *, int, guint, proto_tree *);
