@@ -2,7 +2,7 @@
  * Routines for IEEE 802.2 LLC layer
  * Gilbert Ramirez <gramirez@tivoli.com>
  *
- * $Id: packet-llc.c,v 1.28 1999/11/16 11:42:38 guy Exp $
+ * $Id: packet-llc.c,v 1.36 1999/12/29 05:20:00 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@unicom.net>
@@ -34,14 +34,18 @@
 
 #include <glib.h>
 #include "packet.h"
+#include "oui.h"
 #include "xdlc.h"
 	
 static int proto_llc = -1;
 static int hf_llc_dsap = -1;
 static int hf_llc_ssap = -1;
+static int hf_llc_dsap_ig = -1;
+static int hf_llc_ssap_cr = -1;
 static int hf_llc_ctrl = -1;
 static int hf_llc_type = -1;
 static int hf_llc_oui = -1;
+static int hf_llc_pid = -1;
 
 static gint ett_llc = -1;
 static gint ett_llc_ctrl = -1;
@@ -58,12 +62,42 @@ struct sap_info {
 	dissect_func_t *dissect_func;
 };
 
+/*
+ * Group/Individual bit, in the DSAP.
+ */
+#define	DSAP_GI_BIT	0x01
+
+/*
+ * Command/Response bit, in the SSAP.
+ *
+ * The low-order bit of the SSAP apparently determines whether this
+ * is a request or a response.  (RFC 1390, "Transmission of IP and
+ * ARP over FDDI Networks", says
+ *
+ *	Command frames are identified by having the low order
+ *	bit of the SSAP address reset to zero.  Response frames
+ *	have the low order bit of the SSAP address set to one.
+ *
+ * and a page I've seen seems to imply that's part of 802.2.)
+ */
+#define	SSAP_CR_BIT	0x01
+
+/*
+ * Mask to extrace the SAP number from the DSAP or the SSAP.
+ */
+#define	SAP_MASK	0xFE
+
+/*
+ * These are for SSAP and DSAP, wth last bit always zero.
+ * XXX - some DSAPs come in separate "individual" and "group" versions,
+ * with the last bit 0 and 1, respectively (e.g., LLC Sub-layer Management,
+ * IBM SNA Path Control, IBM Net Management), and, whilst 0xFE is
+ * the ISO Network Layer Protocol, 0xFF is the Global LSAP.
+ */
 static const value_string sap_vals[] = {
 	{ 0x00, "NULL LSAP" },
-	{ 0x02, "LLC Sub-Layer Management Individual" },
-	{ 0x03, "LLC Sub-Layer Management Group" },
-	{ 0x04, "SNA Path Control Individual" },
-	{ 0x05, "SNA Path Control Group" },
+	{ 0x02, "LLC Sub-Layer Management" },
+	{ 0x04, "SNA Path Control" },
 	{ 0x06, "TCP/IP" },
 	{ 0x08, "SNA" },
 	{ 0x0C, "SNA" },
@@ -75,8 +109,7 @@ static const value_string sap_vals[] = {
 	{ 0xBC, "Banyan Vines" },
 	{ 0xE0, "NetWare" },
 	{ 0xF0, "NetBIOS" },
-	{ 0xF4, "IBM Net Management Individual" },
-	{ 0xF5, "IBM Net Management Group" },
+	{ 0xF4, "IBM Net Management" },
 	{ 0xF8, "Remote Program Load" },
 	{ 0xFC, "Remote Program Load" },
 	{ 0xFE, "ISO Network Layer" },
@@ -99,7 +132,7 @@ static struct sap_info	saps[] = {
 	{ 0xAA, NULL,		NULL },
 	{ 0xBA, NULL,		NULL },
 	{ 0xBC, NULL,		NULL },
-	{ 0xE0, NULL,		dissect_ipx },
+	{ 0xE0, capture_ipx,	dissect_ipx },
 	{ 0xF0, capture_netbios, dissect_netbios },
 	{ 0xF4, NULL,		NULL },
 	{ 0xF5, NULL,		NULL },
@@ -118,15 +151,17 @@ static const value_string llc_ctrl_vals[] = {
 	{ 0, NULL }
 };
 
-static const value_string llc_oui_vals[] = {
-	{ 0x000000, "Encapsulated Ethernet" },
+const value_string oui_vals[] = {
+	{ OUI_ENCAP_ETHER, "Encapsulated Ethernet" },
 /*
 http://www.cisco.com/univercd/cc/td/doc/product/software/ios113ed/113ed_cr/ibm_r/brprt1/brsrb.htm
 */
-	{ 0x0000f8, "Cisco 90-Compatible" },
-	{ 0x0000c0, "Cisco" },
-	{ 0x0080c2, "Bridged Frame-Relay" }, /* RFC 2427 */
-	{ 0,        NULL }
+	{ 0x00000c,        "Cisco" },
+	{ 0x0000f8,        "Cisco 90-Compatible" },
+	{ 0x0080c2,        "Bridged Frame-Relay" }, /* RFC 2427 */
+	{ OUI_ATM_FORUM,   "ATM Forum" },
+	{ OUI_APPLE_ATALK, "Apple (AppleTalk)" },
+	{ 0,               NULL }
 };
 
 static capture_func_t *
@@ -161,47 +196,65 @@ sap_dissect_func(u_char sap) {
 	return dissect_data;
 }
 
-
 void
 capture_llc(const u_char *pd, int offset, guint32 cap_len, packet_counts *ld) {
 
 	int		is_snap;
 	guint16		control;
 	int		llc_header_len;
+	guint32		oui;
 	guint16		etype;
 	capture_func_t	*capture;
 
+	if (!BYTES_ARE_IN_FRAME(offset, 2)) {
+		ld->other++;
+		return;
+	}
 	is_snap = (pd[offset] == 0xAA) && (pd[offset+1] == 0xAA);
 	llc_header_len = 2;	/* DSAP + SSAP */
 
 	/*
-	 * The low-order bit of the SSAP apparently determines whether this
-	 * is a request or a response.  (RFC 1390, "Transmission of IP and
-	 * ARP over FDDI Networks", says
-	 *
-	 *	Command frames are identified by having the low order
-	 *	bit of the SSAP address reset to zero.  Response frames
-	 *	have the low order bit of the SSAP address set to one.
-	 *
-	 * and a page I've seen seems to imply that's part of 802.2.)
-	 *
-	 * XXX - that page also implies that LLC Type 2 always uses
-	 * extended operation, so we don't need to determine whether
-	 * it's basic or extended operation; is that the case?
+	 * XXX - the page referred to in the comment above about the
+	 * Command/Response bit also implies that LLC Type 2 always
+	 * uses extended operation, so we don't need to determine
+	 * whether it's basic or extended operation; is that the case?
 	 */
-	control = get_xdlc_control(pd, offset+2, pd[offset+1] & 0x01, TRUE);
+	control = get_xdlc_control(pd, offset+2, pd[offset+1] & SSAP_CR_BIT,
+	    TRUE);
 	llc_header_len += XDLC_CONTROL_LEN(control, TRUE);
 	if (is_snap)
-		llc_header_len += 5;	/* 3 bytes of OUI, 2 bytes of ethertype */
+		llc_header_len += 5;	/* 3 bytes of OUI, 2 bytes of protocol ID */
+	if (!BYTES_ARE_IN_FRAME(offset, llc_header_len)) {
+		ld->other++;
+		return;
+	}
 
 	if (is_snap) {
+		oui = pd[offset+3] << 16 | pd[offset+4] << 8 | pd[offset+5];
 		if (XDLC_HAS_PAYLOAD(control)) {
 			/*
 			 * This frame has a payload to be analyzed.
 			 */
-			etype  = (pd[offset+6] << 8) | pd[offset+7];
-			offset += llc_header_len;
-			capture_ethertype(etype, offset, pd, cap_len, ld);
+			etype = pntohs(&pd[offset+6]);
+			switch (oui) {
+
+			case OUI_ENCAP_ETHER:
+			case OUI_APPLE_ATALK:
+				/* No, I have no idea why Apple used
+				   one of their own OUIs, rather than
+				   OUI_ENCAP_ETHER, and an Ethernet
+				   packet type as protocol ID, for
+				   AppleTalk data packets - but used
+				   OUI_ENCAP_ETHER and an Ethernet
+				   packet type for AARP packets. */
+				capture_ethertype(etype, offset+8, pd,
+				    cap_len, ld);
+				break;
+
+			default:
+				ld->other++;
+				break;
+			}
 		}
 	}		
 	else {
@@ -232,45 +285,51 @@ dissect_llc(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
 	int		is_snap;
 	guint16		control;
 	int		llc_header_len;
+	guint32		oui;
 	guint16		etype;
 	dissect_func_t	*dissect;
 
+	if (!BYTES_ARE_IN_FRAME(offset, 2)) {
+		dissect_data(pd, offset, fd, tree);
+		return;
+	}
 	is_snap = (pd[offset] == 0xAA) && (pd[offset+1] == 0xAA);
 	llc_header_len = 2;	/* DSAP + SSAP */
 
 	if (check_col(fd, COL_PROTOCOL)) {
 		col_add_str(fd, COL_PROTOCOL, "LLC");
 	}
-  
+
 	if (tree) {
 		ti = proto_tree_add_item(tree, proto_llc, offset, 0, NULL);
 		llc_tree = proto_item_add_subtree(ti, ett_llc);
-		proto_tree_add_item(llc_tree, hf_llc_dsap, offset, 1, pd[offset]);
-		proto_tree_add_item(llc_tree, hf_llc_ssap, offset+1, 1, pd[offset+1]);
+		proto_tree_add_item(llc_tree, hf_llc_dsap, offset, 
+			1, pd[offset] & SAP_MASK);
+		proto_tree_add_item(llc_tree, hf_llc_dsap_ig, offset, 
+			1, pd[offset] & DSAP_GI_BIT);
+		proto_tree_add_item(llc_tree, hf_llc_ssap, offset+1, 
+			1, pd[offset+1] & SAP_MASK);
+		proto_tree_add_item(llc_tree, hf_llc_ssap_cr, offset+1, 
+			1, pd[offset+1] & SSAP_CR_BIT);
 	} else
 		llc_tree = NULL;
 
 	/*
-	 * The low-order bit of the SSAP apparently determines whether this
-	 * is a request or a response.  (RFC 1390, "Transmission of IP and
-	 * ARP over FDDI Networks", says
-	 *
-	 *	Command frames are identified by having the low order
-	 *	bit of the SSAP address reset to zero.  Response frames
-	 *	have the low order bit of the SSAP address set to one.
-	 *
-	 * and a page I've seen seems to imply that's part of 802.2.)
-	 *
-	 * XXX - that page also implies that LLC Type 2 always uses
-	 * extended operation, so we don't need to determine whether
-	 * it's basic or extended operation; is that the case?
+	 * XXX - the page referred to in the comment above about the
+	 * Command/Response bit also implies that LLC Type 2 always
+	 * uses extended operation, so we don't need to determine
+	 * whether it's basic or extended operation; is that the case?
 	 */
 	control = dissect_xdlc_control(pd, offset+2, fd, llc_tree,
 				hf_llc_ctrl, ett_llc_ctrl,
-				pd[offset+1] & 0x01, TRUE);
+				pd[offset+1] & SSAP_CR_BIT, TRUE);
 	llc_header_len += XDLC_CONTROL_LEN(control, TRUE);
 	if (is_snap)
-		llc_header_len += 5;	/* 3 bytes of OUI, 2 bytes of ethertype */
+		llc_header_len += 5;	/* 3 bytes of OUI, 2 bytes of protocol ID */
+	if (!BYTES_ARE_IN_FRAME(offset, llc_header_len)) {
+		dissect_data(pd, offset, fd, tree);
+		return;
+	}
 	if (tree)
 		proto_item_set_len(ti, llc_header_len);
 
@@ -280,28 +339,55 @@ dissect_llc(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
 	 * than overwriting it?
 	 */
 	if (is_snap) {
+		oui = pd[offset+3] << 16 | pd[offset+4] << 8 | pd[offset+5];
+		etype = pntohs(&pd[offset+6]);
 		if (check_col(fd, COL_INFO)) {
-			col_add_str(fd, COL_INFO, "802.2 LLC (SNAP)");
+			col_add_fstr(fd, COL_INFO, "SNAP, OUI 0x%06X (%s), PID 0x%04X",
+			    oui, val_to_str(oui, oui_vals, "Unknown"),
+			    etype);
 		}
 		if (tree) {
 			proto_tree_add_item(llc_tree, hf_llc_oui, offset+3, 3,
-				pd[offset+3] << 16 | pd[offset+4] << 8 | pd[offset+5]);
+				oui);
 		}
 		if (XDLC_HAS_PAYLOAD(control)) {
 			/*
 			 * This frame has a payload to be analyzed.
 			 */
-			etype = pntohs(&pd[offset+6]);
-			offset += llc_header_len;
-			/* w/o even checking, assume OUI is ethertype */
-			ethertype(etype, offset, pd, fd, tree, llc_tree,
-			    hf_llc_type);
+			switch (oui) {
+
+			case OUI_ENCAP_ETHER:
+			case OUI_APPLE_ATALK:
+				/* No, I have no idea why Apple used
+				   one of their own OUIs, rather than
+				   OUI_ENCAP_ETHER, and an Ethernet
+				   packet type as protocol ID, for
+				   AppleTalk data packets - but used
+				   OUI_ENCAP_ETHER and an Ethernet
+				   packet type for AARP packets. */
+				ethertype(etype, offset+8, pd,
+				    fd, tree, llc_tree, hf_llc_type);
+				break;
+
+			default:
+				proto_tree_add_item(llc_tree, hf_llc_pid,
+				    offset+6, 2, etype);
+				dissect_data(pd, offset+8, fd, tree);
+				break;
+			}
 		}
 	}		
 	else {
 		if (check_col(fd, COL_INFO)) {
-			col_add_fstr(fd, COL_INFO, "802.2 LLC (%s)",
-				val_to_str(pd[offset], sap_vals, "%02x"));
+			col_add_fstr(fd, COL_INFO, 
+			    "DSAP %s %s, SSAP %s %s",
+			    val_to_str(pd[offset] & SAP_MASK, sap_vals, "%02x"),
+			    pd[offset] & DSAP_GI_BIT ?
+			      "Group" : "Individual",
+			    val_to_str(pd[offset+1] & SAP_MASK, sap_vals, "%02x"),
+			    pd[offset+1] & SSAP_CR_BIT ?
+			      "Command" : "Response"
+			);
 		}
 
 		if (XDLC_HAS_PAYLOAD(control)) {
@@ -326,27 +412,42 @@ dissect_llc(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
 void
 proto_register_llc(void)
 {
+	static struct true_false_string ig_bit = { "Group", "Individual" };
+	static struct true_false_string cr_bit = { "Response", "Command" };
+
 	static hf_register_info hf[] = {
 		{ &hf_llc_dsap,
-		{ "DSAP",		"llc.dsap", FT_UINT8, BASE_HEX, VALS(sap_vals), 0x0,
-			"" }},
+		{ "DSAP",	"llc.dsap", FT_UINT8, BASE_HEX, 
+			VALS(sap_vals), 0x0, "" }},
+
+		{ &hf_llc_dsap_ig,
+		{ "IG Bit",	"llc.dsap.ig", FT_BOOLEAN, BASE_HEX, 
+			&ig_bit, 0x0, "Individual/Group" }},
 
 		{ &hf_llc_ssap,
-		{ "SSAP",		"llc.ssap", FT_UINT8, BASE_HEX, VALS(sap_vals), 0x0,
-			"" }},
+		{ "SSAP", "llc.ssap", FT_UINT8, BASE_HEX, 
+			VALS(sap_vals), 0x0, "" }},
+
+		{ &hf_llc_ssap_cr,
+		{ "CR Bit", "llc.ssap.cr", FT_BOOLEAN, BASE_HEX, 
+			&cr_bit, 0x0, "Command/Response" }},
 
 		{ &hf_llc_ctrl,
-		{ "Control",		"llc.control", FT_UINT8, BASE_HEX, VALS(llc_ctrl_vals), 0x0,
-			"" }},
+		{ "Control", "llc.control", FT_UINT8, BASE_HEX, 
+			VALS(llc_ctrl_vals), 0x0, "" }},
 
 		/* registered here but handled in ethertype.c */
 		{ &hf_llc_type,
-		{ "Type",		"llc.type", FT_UINT16, BASE_HEX, VALS(etype_vals), 0x0,
-			"" }},
+		{ "Type", "llc.type", FT_UINT16, BASE_HEX, 
+			VALS(etype_vals), 0x0, "" }},
 
 		{ &hf_llc_oui,
-		{ "Organization Code",	"llc.oui", FT_UINT24, BASE_HEX, VALS(llc_oui_vals), 0x0,
-			""}}
+		{ "Organization Code",	"llc.oui", FT_UINT24, BASE_HEX, 
+			VALS(oui_vals), 0x0, ""}},
+
+		{ &hf_llc_pid,
+		{ "Protocol ID", "llc.pid", FT_UINT16, BASE_HEX, 
+			NULL, 0x0, ""}}
 	};
 	static gint *ett[] = {
 		&ett_llc,
