@@ -1,8 +1,9 @@
 /* packet-ncp.c
  * Routines for NetWare Core Protocol
  * Gilbert Ramirez <gram@verdict.uthscsa.edu>
+ * Modified to allow NCP over TCP/IP decodes by James Coe <jammer@cin.net>
  *
- * $Id: packet-ncp.c,v 1.23 1999/11/18 01:45:02 guy Exp $
+ * $Id: packet-ncp.c,v 1.27 1999/12/15 04:20:46 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@unicom.net>
@@ -44,6 +45,8 @@
 #include "packet-ncp.h"
 
 static int proto_ncp = -1;
+static int hf_ncp_ip_ver = -1;
+static int hf_ncp_ip_sig = -1;
 static int hf_ncp_type = -1;
 static int hf_ncp_seq = -1;
 static int hf_ncp_connection = -1;
@@ -75,6 +78,26 @@ guint ncp_hash  (gconstpointer v);
 
 int ncp_packet_init_count = 200;
 
+/* These are the header structures to handle NCP over IP */
+#define	NCPIP_RQST	0x446d6454	// "DmdT"
+#define NCPIP_RPLY	0x744e6350	// "tNcP"
+
+struct ncp_ip_header {
+	guint32	signature;
+	guint32 length;
+};
+
+/* This header only appears on NCP over IP request packets */
+struct ncp_ip_rqhdr {
+	guint32 version;
+	guint32 rplybufsize;
+};
+
+static const value_string ncp_ip_signature[] = {
+	{ NCPIP_RQST, "Demand Transport (Request)" },
+	{ NCPIP_RPLY, "Transport is NCP (Reply)" },
+};
+
 /* The information in this module comes from:
 	NetWare LAN Analysis, Second Edition
 	Laura A. Chappell and Dan E. Hakes
@@ -273,6 +296,58 @@ static svc_record ncp_48_00_R[] = {
 
 static ncp2222_record ncp2222[] = {
 
+{ 0x00, 0x00, NOSUB, "Create service connection",
+	NULL, NULL, NCP_CONNECTION_SERVICES
+},
+
+{ 0x14, 0x00, NOSUB, "Get server's clock",
+	NULL, NULL, NCP_FILE_SERVICES
+},
+
+{ 0x16, 0x01, SUBFUNC, "Get path of directory handle",
+	NULL, NULL, NCP_FILE_SERVICES
+},
+
+{ 0x16, 0x13, SUBFUNC, "Create temporary directory handle",
+	NULL, NULL, NCP_BINDERY_SERVICES
+},
+
+{ 0x16, 0x0A, SUBFUNC, "Create directory",
+	NULL, NULL, NCP_BINDERY_SERVICES
+},
+
+{ 0x16, 0x0D, SUBFUNC, "Add trustee to directory",
+	NULL, NULL, NCP_BINDERY_SERVICES
+},
+
+{ 0x17, 0x11, SUBFUNC, "Get fileserver information",
+	NULL, NULL, NCP_BINDERY_SERVICES
+},
+
+{ 0x17, 0x37, SUBFUNC, "Scan bindery object",
+	NULL, NULL, NCP_BINDERY_SERVICES
+},
+
+{ 0x17, 0x36, SUBFUNC, "Get bindery object name",
+	NULL, NULL, NCP_BINDERY_SERVICES
+},
+
+{ 0x17, 0x32, SUBFUNC, "Create bindery object",
+	NULL, NULL, NCP_BINDERY_SERVICES
+},
+
+{ 0x17, 0x39, SUBFUNC, "Create property",
+	NULL, NULL, NCP_BINDERY_SERVICES
+},
+
+{ 0x17, 0x41, SUBFUNC, "Add bindery object to set",
+	NULL, NULL, NCP_BINDERY_SERVICES
+},
+
+{ 0x17, 0x43, SUBFUNC, "Is bindery object in set",
+	NULL, NULL, NCP_BINDERY_SERVICES
+},
+
 { 0x17, 0x35, SUBFUNC, "Get Bindery Object ID",
 	ncp_17_35_C, NULL, NCP_BINDERY_SERVICES
 },
@@ -281,6 +356,10 @@ static ncp2222_record ncp2222[] = {
 	ncp_17_7C_C, ncp_17_7C_R, NCP_QUEUE_SERVICES
 },
 
+{ 0x17, 0x3D, SUBFUNC, "Read property value",
+	NULL, NULL, NCP_FILE_SERVICES
+},
+
 { 0x18, 0x00, NOSUB, "End of Job",
 	NULL, NULL, NCP_CONNECTION_SERVICES
 },
@@ -293,6 +372,18 @@ static ncp2222_record ncp2222[] = {
 	ncp_21_00_C, ncp_21_00_R, NCP_CONNECTION_SERVICES
 },
 
+{ 0x24, 0x00, SUBFUNC, "Destroy service connection",
+	NULL, NULL, NCP_CONNECTION_SERVICES
+},
+
+{ 0x3E, 0x53, SUBFUNC, "Get alternate directory search paths",
+	NULL, NULL, NCP_FILE_SERVICES
+},
+
+{ 0x3F, 0x89, SUBFUNC, "File search continue",
+	NULL, NULL, NCP_FILE_SERVICES
+},
+
 { 0x42, 0x00, NOSUB, "Close File",
 	ncp_42_00_C, ncp_42_00_R, NCP_FILE_SERVICES
 },
@@ -301,6 +392,22 @@ static ncp2222_record ncp2222[] = {
 	ncp_48_00_C, ncp_48_00_R, NCP_FILE_SERVICES
 },
 
+{ 0x4C, 0x11, SUBFUNC, "Open file",
+	NULL, NULL, NCP_FILE_SERVICES
+},
+
+{ 0x61, 0x00, NOSUB, "Get big packet NCP max packet size",
+	NULL, NULL, NCP_FILE_SERVICES
+},
+
+{ 0x68, 0x01, SUBFUNC, "Ping for NDS NCP",
+	NULL, NULL, NCP_FILE_SERVICES
+},
+
+{ 0x68, 0x02, SUBFUNC, "Send NDS fragmented message",
+	NULL, NULL, NCP_FILE_SERVICES
+},
+
 { 0x00, 0x00, NOSUB, NULL,
 	NULL, NULL, NCP_UNKNOWN_SERVICE
 }
@@ -417,7 +524,8 @@ ncp2222_find(guint8 func, guint8 subfunc)
 
 	ncp_record = ncp2222;
 
-	while(ncp_record->func != 0) {
+	while(ncp_record->func != 0 || ncp_record->subfunc != 0 ||
+		ncp_record->funcname != NULL ) {
 		if (ncp_record->func == func &&
 			ncp_record->subfunc == (subfunc & ncp_record->submask)) {
 			retval = ncp_record;
@@ -435,8 +543,21 @@ dissect_ncp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
 	proto_tree	*ncp_tree = NULL;
 	proto_item	*ti;
 	int		ncp_hdr_length = 0;
+	struct ncp_ip_header		ncpiph;
+	struct ncp_ip_rqhdr		ncpiphrq;
 	struct ncp_common_header	header;
 
+	if ( pi.ptype == PT_TCP || pi.ptype == PT_UDP ) {
+		ncpiph.signature = pntohl(&pd[offset]);
+		ncpiph.length = pntohl(&pd[offset + 4]);
+		offset += 8;
+		if ( ncpiph.signature == NCPIP_RQST ) {
+			ncpiphrq.version = pntohl(&pd[offset]);
+			ncpiphrq.rplybufsize = pntohl(&pd[offset + 4]);
+			offset += 8;
+		};
+	};
+
 	memcpy(&header, &pd[offset], sizeof(header));
 	header.type = ntohs(header.type);
 
@@ -461,6 +582,14 @@ dissect_ncp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
 		ti = proto_tree_add_item(tree, proto_ncp, offset, END_OF_FRAME, NULL);
 		ncp_tree = proto_item_add_subtree(ti, ett_ncp);
 
+		if ( pi.ptype == PT_TCP || pi.ptype == PT_UDP ) {
+			proto_tree_add_item(ncp_tree, hf_ncp_ip_sig, offset - 16, 4, ncpiph.signature);
+			proto_tree_add_text(ncp_tree, offset - 12, 4, "Length: %d", ncpiph.length);
+			if ( ncpiph.signature == NCPIP_RQST ) {
+				proto_tree_add_item(ncp_tree, hf_ncp_ip_ver, offset - 8, 4, ncpiphrq.version);
+				proto_tree_add_text(ncp_tree, offset - 4, 4, "Reply buffer size: %d", ncpiphrq.rplybufsize);
+			};
+		};
 		proto_tree_add_item_format(ncp_tree, hf_ncp_type, 
 					   offset,      2,
 					   header.type,
@@ -502,7 +631,11 @@ dissect_ncp_request(const u_char *pd, int offset, frame_data *fd,
 
 	/*memcpy(&request, &pd[offset], sizeof(request));*/
 	request.function = pd[offset+6];
-	request.subfunc = pd[offset+9];
+	if ( BYTES_ARE_IN_FRAME(offset, 9) ) {
+		request.subfunc = pd[offset+9];
+	} else {
+		request.subfunc = 0;
+	}
 
 	ncp_request = ncp2222_find(request.function, request.subfunc);
 
@@ -865,6 +998,14 @@ proto_register_ncp(void)
 {
 
   static hf_register_info hf[] = {
+    { &hf_ncp_ip_sig,
+      { "NCP over IP signature",		"ncp.ip.signature",
+        FT_UINT32, BASE_HEX, VALS(ncp_ip_signature), 0x0,
+        "NCP over IP transport signature"}},
+    { &hf_ncp_ip_ver,
+      { "Version",		"ncp.ip.version",
+        FT_UINT32, BASE_DEC, NULL, 0x0,
+        "NCP over IP verion"}},
     { &hf_ncp_type,
       { "Type",			"ncp.type",
 	FT_UINT16, BASE_HEX, NULL, 0x0,
