@@ -5,7 +5,7 @@
  * 
  * derived from the packet-nbns.c
  *
- * $Id: packet-netbios.h,v 1.3 1999/09/03 01:43:09 guy Exp $
+ * $Id: packet-netbios.h,v 1.4 1999/11/30 07:45:42 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -31,9 +31,8 @@
 #define NETBIOS_NAME_LEN	16
 
 extern int process_netbios_name(const u_char *name_ptr, char *name_ret);
-extern guint get_netbios_name(const u_char *data_ptr, int offset,
+extern int get_netbios_name(const u_char *data_ptr, int offset,
     char *name_ret);
 extern char *netbios_name_type_descr(int name_type);
-extern void netbios_add_name( char* label, const u_char *pd, int offset,
-    int nb_offset, proto_tree *tree);
-
+extern gboolean netbios_add_name( char* label, const u_char *pd, int offset,
+    proto_tree *tree);
