@@ -1,5 +1,5 @@
 /* packet-nfs.h (c) 1999 Uwe Girlich */
-/* $Id: packet-nfs.h,v 1.2 1999/11/15 14:17:18 nneul Exp $ */
+/* $Id: packet-nfs.h,v 1.5 1999/12/16 08:34:10 girlich Exp $ */
 
 #ifndef __PACKET_NFS_H__
 #define __PACKET_NFS_H__
@@ -10,6 +10,21 @@
 
 #define FHSIZE 32
 
+/* verifier */
+#define NFS3_COOKIEVERFSIZE 8
+#define NFS3_CREATEVERFSIZE 8
+#define NFS3_WRITEVERFSIZE 8
+
+/* for write */
+#define UNSTABLE 0
+#define DATA_SYNC 1
+#define FILE_SYNC 2
+
+/* for create */
+#define UNCHECKED 0
+#define GUARDED 1
+#define EXCLUSIVE 2
+
 /* the RPC mount protocol needs both function to decode a MNT reply */
 int dissect_fhandle(const u_char *pd, int offset, frame_data *fd, proto_tree *tree, char* name);
 int dissect_nfs_fh3(const u_char *pd, int offset, frame_data *fd, proto_tree *tree, char* name);
