@@ -1,7 +1,7 @@
 /* packet-null.c
  * Routines for null packet disassembly
  *
- * $Id: packet-null.c,v 1.15 1999/08/24 17:26:13 gram Exp $
+ * $Id: packet-null.c,v 1.17 1999/11/16 11:42:43 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -48,6 +48,8 @@ static int proto_null = -1;
 static int hf_null_etype = -1;
 static int hf_null_family = -1;
 
+static gint ett_null = -1;
+
 /* Null/loopback structs and definitions */
 
 /* Macro to byte-swap 32-bit quantities. */
@@ -260,7 +262,7 @@ dissect_null( const u_char *pd, frame_data *fd, proto_tree *tree )
     if (null_header > IEEE_802_3_MAX_LEN) {
       if (tree) {
         ti = proto_tree_add_item(tree, proto_null, 0, 4, NULL);
-        fh_tree = proto_item_add_subtree(ti, ETT_NULL);
+        fh_tree = proto_item_add_subtree(ti, ett_null);
       } else
       	fh_tree = NULL;
       ethertype(null_header, 4, pd, fd, tree, fh_tree, hf_null_etype);
@@ -269,7 +271,7 @@ dissect_null( const u_char *pd, frame_data *fd, proto_tree *tree )
          layer (ie none) */
       if (tree) {
         ti = proto_tree_add_item(tree, proto_null, 0, 4, NULL);
-        fh_tree = proto_item_add_subtree(ti, ETT_NULL);
+        fh_tree = proto_item_add_subtree(ti, ett_null);
         proto_tree_add_item(fh_tree, hf_null_family, 0, 4, null_header);
       }
 
@@ -311,12 +313,18 @@ proto_register_null(void)
 
 		/* registered here but handled in ethertype.c */
 		{ &hf_null_etype,
-		{ "Type",		"null.type", FT_VALS_UINT16, VALS(etype_vals) }},
+		{ "Type",		"null.type", FT_UINT16, BASE_HEX, VALS(etype_vals), 0x0,
+			"" }},
 
 		{ &hf_null_family,
-		{ "Family",		"null.family",	FT_VALS_UINT32, VALS(family_vals) }}
+		{ "Family",		"null.family",	FT_UINT32, BASE_HEX, VALS(family_vals), 0x0,
+			"" }}
+	};
+	static gint *ett[] = {
+		&ett_null,
 	};
 
 	proto_null = proto_register_protocol ("Null/Loopback", "null" );
 	proto_register_field_array(proto_null, hf, array_length(hf));
+	proto_register_subtree_array(ett, array_length(ett));
 }
