@@ -2,7 +2,7 @@
  * Routines for OSPF packet disassembly
  * (c) Copyright Hannes R. Boehm <hannes@boehm.org>
  *
- * $Id: packet-ospf.c,v 1.13 1999/08/26 07:34:41 guy Exp $
+ * $Id: packet-ospf.c,v 1.17 1999/11/17 19:07:10 guy Exp $
  *
  * At this time, this module is able to analyze OSPF
  * packets as specified in RFC2328. MOSPF (RFC1584) and other
@@ -52,14 +52,24 @@
 
 static int proto_ospf = -1;
 
+static gint ett_ospf = -1;
+static gint ett_ospf_hdr = -1;
+static gint ett_ospf_hello = -1;
+static gint ett_ospf_desc = -1;
+static gint ett_ospf_lsr = -1;
+static gint ett_ospf_lsa = -1;
+static gint ett_ospf_lsa_upd = -1;
+
 void 
 dissect_ospf(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
     e_ospfhdr ospfh;
+    e_ospf_crypto *crypto;
+    int i, saved_len, ospflen;
 
     proto_tree *ospf_tree = NULL;
 	proto_item *ti; 
     proto_tree *ospf_header_tree;
-    char auth_data[9]="";
+    char auth_data[(2 * 16) + 1]="";
     char *packet_type;
     static value_string pt_vals[] = { {OSPF_HELLO,   "Hello Packet"   },
                                       {OSPF_DB_DESC, "DB Descr."      },
@@ -82,10 +92,10 @@ dissect_ospf(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
 
     if (tree) {
 	ti = proto_tree_add_item(tree, proto_ospf, offset, ntohs(ospfh.length), NULL);
-	ospf_tree = proto_item_add_subtree(ti, ETT_OSPF);
+	ospf_tree = proto_item_add_subtree(ti, ett_ospf);
 
 	ti = proto_tree_add_text(ospf_tree, offset, OSPF_HEADER_LENGTH, "OSPF Header"); 
-	ospf_header_tree = proto_item_add_subtree(ti, ETT_OSPF_HDR);
+	ospf_header_tree = proto_item_add_subtree(ti, ett_ospf_hdr);
 
         proto_tree_add_text(ospf_header_tree, offset, 1, "OSPF Version: %d", ospfh.version);  
 	proto_tree_add_text(ospf_header_tree, offset + 1 , 1, "OSPF Packet Type: %d (%s)", 
@@ -116,8 +126,19 @@ dissect_ospf(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
 	         proto_tree_add_text(ospf_header_tree, offset + 16 , 8, "Auth Data: %s", auth_data);
 		 break;
 	    case OSPF_AUTH_CRYPT:
+                 crypto = (e_ospf_crypto *)ospfh.auth_data;
 	         proto_tree_add_text(ospf_header_tree, offset + 14 , 2, "Auth Type: crypt");
-	         proto_tree_add_text(ospf_header_tree, offset + 16 , 8, "Auth Data (crypt)");
+                 proto_tree_add_text(ospf_header_tree, offset + 18 , 1, "Auth Key ID: %d",
+                                     crypto->key_id);
+                 proto_tree_add_text(ospf_header_tree, offset + 19 , 1, "Auth Data Length: %d",
+                                     crypto->length);
+                 proto_tree_add_text(ospf_header_tree, offset + 20 , 4, "Auth Crypto Sequence Number: 0x%x",
+                                     ntohl(crypto->sequence_num));
+  
+                 ospflen = ntohs(ospfh.length);
+                 for (i = 0; i < crypto->length && i < (sizeof(auth_data)/2); i++)
+                     sprintf(&auth_data[i*2],"%02x",pd[offset + ospflen + i]);
+                 proto_tree_add_text(ospf_header_tree, offset + ospflen , 16, "Auth Data: %s", auth_data);
 		 break;
             default:
 	         proto_tree_add_text(ospf_header_tree, offset + 14 , 2, "Auth Type (unknown)");
@@ -126,8 +147,18 @@ dissect_ospf(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
 
     }
 
+    /* Temporarily adjust the captured length to match the size of the OSPF
+     * packet (since the dissect routines use it to work out where the end of
+     * the ospf packet is).
+     */
+    saved_len = pi.captured_len;
+    if (BYTES_ARE_IN_FRAME(offset, ntohs(ospfh.length))) {
+      pi.captured_len = offset + ntohs(ospfh.length);
+    }
+
     /*  Skip over header */
     offset += OSPF_HEADER_LENGTH;
+
     switch(ospfh.packet_type){
 	case OSPF_HELLO:
 	    dissect_ospf_hello(pd, offset, fd, ospf_tree); 
@@ -145,8 +176,10 @@ dissect_ospf(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
 	    dissect_ospf_ls_ack(pd, offset, fd, ospf_tree);
 	    break;
 	default:
+            pi.captured_len = saved_len;
             dissect_data(pd, offset, fd, tree); 
     }
+    pi.captured_len = saved_len;
 }
 
 void
@@ -163,7 +196,7 @@ dissect_ospf_hello(const u_char *pd, int offset, frame_data *fd, proto_tree *tre
 
     if (tree) {
 	ti = proto_tree_add_text(tree, offset, END_OF_FRAME, "OSPF Hello Packet"); 
-	ospf_hello_tree = proto_item_add_subtree(ti, ETT_OSPF_HELLO);
+	ospf_hello_tree = proto_item_add_subtree(ti, ett_ospf_hello);
 
 
 	proto_tree_add_text(ospf_hello_tree, offset , 4, "Network Mask: %s",  ip_to_str((guint8 *) &ospfhello.network_mask));
@@ -201,7 +234,6 @@ dissect_ospf_hello(const u_char *pd, int offset, frame_data *fd, proto_tree *tre
 
 	offset+=20;
 	while(((int)(pi.captured_len - offset)) >= 4){
-	    printf("%d", pi.captured_len - offset);
 	    ospfneighbor=(guint32 *) &pd[offset];
 	    proto_tree_add_text(ospf_hello_tree, offset, 4, "Active Neighbor: %s",  ip_to_str((guint8 *) ospfneighbor));
 	    offset+=4;
@@ -224,7 +256,7 @@ dissect_ospf_db_desc(const u_char *pd, int offset, frame_data *fd, proto_tree *t
 
     if (tree) {
 	ti = proto_tree_add_text(tree, offset, END_OF_FRAME, "OSPF DB Description"); 
-	ospf_db_desc_tree = proto_item_add_subtree(ti, ETT_OSPF_DESC);
+	ospf_db_desc_tree = proto_item_add_subtree(ti, ett_ospf_desc);
 
 	proto_tree_add_text(ospf_db_desc_tree, offset, 2, "Interface MTU: %d", ntohs(ospf_dbd.interface_mtu) );
 
@@ -294,7 +326,7 @@ dissect_ospf_ls_req(const u_char *pd, int offset, frame_data *fd, proto_tree *tr
 	while( ((int) (pi.captured_len - offset)) >= OSPF_LS_REQ_LENGTH ){
              memcpy(&ospf_lsr, &pd[offset], sizeof(e_ospf_ls_req));
 	     ti = proto_tree_add_text(tree, offset, OSPF_LS_REQ_LENGTH, "Link State Request"); 
-	     ospf_lsr_tree = proto_item_add_subtree(ti, ETT_OSPF_LSR);
+	     ospf_lsr_tree = proto_item_add_subtree(ti, ett_ospf_lsr);
 
 	     switch( ntohl( ospf_lsr.ls_type ) ){
 		 case OSPF_LSTYPE_ROUTER:
@@ -343,7 +375,7 @@ dissect_ospf_ls_upd(const u_char *pd, int offset, frame_data *fd, proto_tree *tr
 
     if (tree) {
 	ti = proto_tree_add_text(tree, offset, END_OF_FRAME, "LS Update Packet"); 
-	ospf_lsa_upd_tree = proto_item_add_subtree(ti, ETT_OSPF_LSA_UPD);
+	ospf_lsa_upd_tree = proto_item_add_subtree(ti, ett_ospf_lsa_upd);
 
 	proto_tree_add_text(ospf_lsa_upd_tree, offset, 4, "Nr oF LSAs: %ld", (long)ntohl(upd_hdr.lsa_nr) );
     }
@@ -372,6 +404,7 @@ int
 dissect_ospf_lsa(const u_char *pd, int offset, frame_data *fd, proto_tree *tree, int disassemble_body) {
     e_ospf_lsa_hdr 	 lsa_hdr;
     char		*lsa_type;
+    int			 lsa_end;
 
     /* data strutures for the router LSA */
     e_ospf_router_lsa 		router_lsa;
@@ -408,7 +441,7 @@ dissect_ospf_lsa(const u_char *pd, int offset, frame_data *fd, proto_tree *tree,
 	    lsa_type="Network LSA";
             break;
         case OSPF_LSTYPE_SUMMERY:
-	    lsa_type="Summery LSA";
+	    lsa_type="Summary LSA";
             break;
         case OSPF_LSTYPE_ASBR:
 	    lsa_type="ASBR LSA";
@@ -427,7 +460,7 @@ dissect_ospf_lsa(const u_char *pd, int offset, frame_data *fd, proto_tree *tree,
         } else {
              ti = proto_tree_add_text(tree, offset, OSPF_LSA_HEADER_LENGTH, "LSA Header"); 
         }
-        ospf_lsa_tree = proto_item_add_subtree(ti, ETT_OSPF_LSA);
+        ospf_lsa_tree = proto_item_add_subtree(ti, ett_ospf_lsa);
 
 	
         proto_tree_add_text(ospf_lsa_tree, offset, 2, "LS Age: %d seconds", ntohs(lsa_hdr.ls_age));
@@ -449,7 +482,8 @@ dissect_ospf_lsa(const u_char *pd, int offset, frame_data *fd, proto_tree *tree,
            return OSPF_LSA_HEADER_LENGTH;
         }
 
-	/* the LSA body starts afte 20 bytes of LSA Header */
+	lsa_end = offset + ntohs(lsa_hdr.length);
+	/* the LSA body starts after 20 bytes of LSA Header */
 	offset+=20;
 
         switch(lsa_hdr.ls_type){
@@ -523,7 +557,7 @@ dissect_ospf_lsa(const u_char *pd, int offset, frame_data *fd, proto_tree *tree,
                                                  ip_to_str((guint8 *) &(network_lsa.network_mask)));
 		offset += 4;
 
-		while( ((int) (pi.captured_len - offset)) >= 4){
+		while( (lsa_end - offset) >= 4){
 		    attached_router = (guint32 *) &pd[offset];
 		    proto_tree_add_text(ospf_lsa_tree, offset, 4, "Attached Router: %s", 
                                                  ip_to_str((guint8 *) attached_router));
@@ -581,7 +615,17 @@ proto_register_ospf(void)
                 { &variable,
                 { "Name",           "ospf.abbreviation", TYPE, VALS_POINTER }},
         };*/
+	static gint *ett[] = {
+		&ett_ospf,
+		&ett_ospf_hdr,
+		&ett_ospf_hello,
+		&ett_ospf_desc,
+		&ett_ospf_lsr,
+		&ett_ospf_lsa,
+		&ett_ospf_lsa_upd,
+	};
 
         proto_ospf = proto_register_protocol("Open Shortest Path First", "ospf");
  /*       proto_register_field_array(proto_ospf, hf, array_length(hf));*/
+	proto_register_subtree_array(ett, array_length(ett));
 }
