@@ -2,7 +2,7 @@
  * Routines for pop packet dissection
  * Copyright 1999, Richard Sharpe <rsharpe@ns.aus.com>
  *
- * $Id: packet-pop.c,v 1.7 1999/08/24 17:26:13 gram Exp $
+ * $Id: packet-pop.c,v 1.11 1999/11/20 23:03:09 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@unicom.net>
@@ -44,7 +44,13 @@
 #include "packet.h"
 
 static int proto_pop = -1;
+static int hf_pop_response = -1;
+static int hf_pop_request = -1;
 
+static gint ett_pop = -1;
+
+static gboolean is_continuation(const u_char *data);
+	
 void
 dissect_pop(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 {
@@ -67,35 +73,54 @@ dissect_pop(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 
 	  strncpy(rr, pd + offset, MIN(i1, sizeof(rr) - 1));
 	  i2 = ((u_char *)strchr(pd + offset + i1 + 1, '\r') - (pd + offset)) - i1 - 1;
+	  if (i2 > max_data - i1 - 1 || i2 <= 0) {
+	    i2 = ((u_char *)strchr(pd + offset + i1 + 1, '\n') - (pd + offset)) - i1 - 1;
+	    if (i2 > max_data - i1 - 1 || i2 <= 0)
+	      i2 = max_data - i1 - 1;
+	  }
 	  strncpy(rd, pd + offset + i1 + 1, MIN(i2, sizeof(rd) - 1));
 	}
 
 	if (check_col(fd, COL_PROTOCOL))
-		col_add_str(fd, COL_PROTOCOL, "POP");
+	  col_add_str(fd, COL_PROTOCOL, "POP");
 
 	if (check_col(fd, COL_INFO)) {
-
-	  col_add_fstr(fd, COL_INFO, "%s: %s %s", (pi.match_port == pi.destport)? "Request" : "Response", rr, rd);
-
+	  /*
+	   * Do it like HTTP does.
+	   * Put the first line from the buffer into the summary,
+	   * if it's an POP request or reply.
+	   * Otherwise, just call it a continuation.
+	   */
+	  if (pi.match_port == pi.srcport && is_continuation(pd+offset))
+	    col_add_str(fd, COL_INFO, "Continuation");
+	  else
+	    col_add_fstr(fd, COL_INFO, "%s: %s %s", (pi.match_port == pi.destport)? "Request" : "Response", rr, rd);	  
 	}
 
 	if (tree) {
 
 	  ti = proto_tree_add_item(tree, proto_pop, offset, END_OF_FRAME, NULL);
-	  pop_tree = proto_item_add_subtree(ti, ETT_POP);
+	  pop_tree = proto_item_add_subtree(ti, ett_pop);
 
 	  if (pi.match_port == pi.destport) { /* Request */
-
+	    proto_tree_add_item_hidden(pop_tree, hf_pop_request, offset, i1, TRUE);
 	    proto_tree_add_text(pop_tree, offset, i1, "Request: %s", rr);
 
-	    proto_tree_add_text(pop_tree, offset + i1 + 1, END_OF_FRAME, "Request Arg: %s", rd);
+	    if (strlen(rd) != 0)
+	      proto_tree_add_text(pop_tree, offset + i1 + 1, END_OF_FRAME, "Request Arg: %s", rd);
 
 	  }
 	  else {
+	    proto_tree_add_item_hidden(pop_tree, hf_pop_response, offset, i1, TRUE);
 
-	    proto_tree_add_text(pop_tree, offset, i1, "Response: %s", rr);
+	    if (is_continuation(pd+offset))
+	      dissect_data(pd, offset, fd, pop_tree);
+	    else {
+	      proto_tree_add_text(pop_tree, offset, i1, "Response: %s", rr);
 
-	    proto_tree_add_text(pop_tree, offset + i1 + 1, END_OF_FRAME, "Response Arg: %s", rd);
+	      if (strlen(rd) != 0)
+	      proto_tree_add_text(pop_tree, offset + i1 + 1, END_OF_FRAME, "Response Arg: %s", rd);
+	    }
 	  }
 
 	}
@@ -104,11 +129,34 @@ dissect_pop(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 void
 proto_register_pop(void)
 {
-/*        static hf_register_info hf[] = {
-                { &variable,
-                { "Name",           "pop.abbreviation", TYPE, VALS_POINTER }},
-        };*/
 
-        proto_pop = proto_register_protocol("Post Office Protocol", "pop");
- /*       proto_register_field_array(proto_pop, hf, array_length(hf));*/
+  static hf_register_info hf[] = {
+    { &hf_pop_response,
+      { "Response",           "pop.response",
+	FT_BOOLEAN, BASE_NONE, NULL, 0x0,
+      	"TRUE if POP response" }},
+
+    { &hf_pop_request,
+      { "Request",            "pop.request",
+	FT_BOOLEAN, BASE_NONE, NULL, 0x0,
+      	"TRUE if POP request" }}
+  };
+  static gint *ett[] = {
+    &ett_pop,
+  };
+
+  proto_pop = proto_register_protocol("Post Office Protocol", "pop");
+  proto_register_field_array(proto_pop, hf, array_length(hf));
+  proto_register_subtree_array(ett, array_length(ett));
+}
+
+static gboolean is_continuation(const u_char *data)
+{
+  if (strncmp(data, "+OK", strlen("+OK")) == 0)
+    return FALSE;
+
+  if (strncmp(data, "-ERR", strlen("-ERR")) == 0)
+    return FALSE;
+
+  return TRUE;
 }
