@@ -1,7 +1,7 @@
 /* packet-raw.c
  * Routines for raw packet disassembly
  *
- * $Id: packet-raw.c,v 1.9 1999/03/23 03:14:43 gram Exp $
+ * $Id: packet-raw.c,v 1.11 1999/11/16 11:42:50 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -35,6 +35,8 @@
 #include <glib.h>
 #include "packet.h"
 
+static gint ett_raw = -1;
+
 void
 capture_raw( const u_char *pd, guint32 cap_len, packet_counts *ld ) {
 
@@ -71,10 +73,9 @@ dissect_raw( const u_char *pd, frame_data *fd, proto_tree *tree ) {
   /* populate a tree in the second pane with the status of the link
      layer (ie none) */
   if(tree) {
-    ti = proto_tree_add_item(tree, 0, 0, "Raw packet data" );
-    fh_tree = proto_tree_new();
-    proto_item_add_subtree(ti, fh_tree, ETT_RAW);
-    proto_tree_add_item(fh_tree, 0, 0, "No link information available");
+    ti = proto_tree_add_text(tree, 0, 0, "Raw packet data" );
+    fh_tree = proto_item_add_subtree(ti, ett_raw);
+    proto_tree_add_text(fh_tree, 0, 0, "No link information available");
   }
 
   /* So far, the only time we get raw connection types are with Linux and
@@ -91,3 +92,12 @@ dissect_raw( const u_char *pd, frame_data *fd, proto_tree *tree ) {
     dissect_ip(pd, 0, fd, tree);
 }
 
+void
+proto_register_raw(void)
+{
+  static gint *ett[] = {
+    &ett_raw,
+  };
+
+  proto_register_subtree_array(ett, array_length(ett));
+}
