@@ -2,7 +2,7 @@
  * Routines for rpc dissection
  * Copyright 1999, Uwe Girlich <Uwe.Girlich@philosys.de>
  * 
- * $Id: packet-rpc.c,v 1.19 1999/11/19 23:23:40 guy Exp $
+ * $Id: packet-rpc.c,v 1.24 1999/12/14 11:43:59 girlich Exp $
  * 
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@unicom.net>
@@ -116,6 +116,9 @@ static int hf_rpc_state_accept = -1;
 static int hf_rpc_state_reply = -1;
 static int hf_rpc_state_reject = -1;
 static int hf_rpc_state_auth = -1;
+static int hf_rpc_dup = -1;
+static int hf_rpc_call_dup = -1;
+static int hf_rpc_reply_dup = -1;
 
 static gint ett_rpc = -1;
 static gint ett_rpc_string = -1;
@@ -341,6 +344,22 @@ rpc_roundup(unsigned int a)
 }
 
 
+int
+dissect_rpc_bool(const u_char *pd, int offset, frame_data *fd, proto_tree *tree,
+int hfindex)
+{
+	guint32 value;
+
+	if (!BYTES_ARE_IN_FRAME(offset,4)) return offset;
+	value = EXTRACT_UINT(pd, offset+0);
+	if (tree)
+		proto_tree_add_item(tree, hfindex, offset, 4, value);
+	offset += 4;
+
+	return offset;
+}
+
+
 int
 dissect_rpc_uint32(const u_char *pd, int offset, frame_data *fd, proto_tree *tree,
 char* name, char* type)
@@ -374,7 +393,7 @@ char* name, char* type)
 	if (tree) {
 		if (value_high)
 			proto_tree_add_text(tree, offset, 8,
-				"%s: %x%08x", name, value_high, value_low);
+				"%s: 0x%x%08x", name, value_high, value_low);
 		else
 			proto_tree_add_text(tree, offset, 8,
 				"%s: %u", name, value_low);
@@ -386,18 +405,25 @@ char* name, char* type)
 
 
 int
-dissect_rpc_string(const u_char *pd, int offset, frame_data *fd, proto_tree *tree, int hfindex)
+dissect_rpc_opaque_data(const u_char *pd, int offset, frame_data *fd, proto_tree *tree, int hfindex, int string_data)
 {
 	proto_item *string_item = NULL;
 	proto_tree *string_tree = NULL;
 	int old_offset = offset;
 
-	int truncated_length = 0;
+	int length_truncated = 0;
+
+	int string_truncated = 0;
 	guint32 string_length = 0;
+	guint32 string_length_full;
 	guint32 string_length_packet;
 	guint32 string_length_copy = 0;
-	guint32 string_length_full;
-	guint32 string_fill = 0;
+
+	int fill_truncated = 0;
+	guint32 fill_length  = 0;
+	guint32 fill_length_packet  = 0;
+	guint32 fill_length_copy  = 0;
+
 	char *string_buffer = NULL;
 	char *string_buffer_print = NULL;
 
@@ -405,31 +431,69 @@ dissect_rpc_string(const u_char *pd, int offset, frame_data *fd, proto_tree *tre
 		string_length = EXTRACT_UINT(pd,offset+0);
 		string_length_full = rpc_roundup(string_length);
 		string_length_packet = pi.captured_len - (offset + 4);
-		if (string_length > string_length_packet) {
+		if (string_length_packet < string_length) {
+			/* truncated string */
+			string_truncated = 1;
 			string_length_copy = string_length_packet;
-			string_fill = 0;
+			fill_truncated = 2;
+			fill_length = 0;
+			fill_length_packet = 0;
+			fill_length_copy = 0;
 		}
 		else {
+			/* full string data */
+			string_truncated = 0;
 			string_length_copy = string_length;
-			string_fill = string_length_full - string_length;
+			fill_length = string_length_full - string_length;
+			fill_length_packet = pi.captured_len - (offset + 4 + string_length);
+			if (fill_length_packet < fill_length) {
+				/* truncated fill bytes */
+				fill_length_copy = fill_length_packet;
+				fill_truncated = 1;
+			}
+			else {
+				/* full fill bytes */
+				fill_length_copy = fill_length;
+				fill_truncated = 0;
+			}
 		}
-		string_buffer = (char*)g_malloc(string_length_copy + 1);
+		string_buffer = (char*)g_malloc(string_length_copy + 
+			(string_data ? 1 : 0));
 		memcpy(string_buffer,pd+offset+4,string_length_copy);
-		string_buffer[string_length_copy] = '\0';
-
-		/* I use here strdup functions. This works only, if the
-		   string does not contain 0 bytes. */
+		if (string_data)
+			string_buffer[string_length_copy] = '\0';
 
 		/* calculate a nice printable string */
 		if (string_length) {
 			if (string_length != string_length_copy) {
-				string_buffer_print = (char*)g_malloc(string_length_copy + 1 + 12);
-				memcpy(string_buffer_print,string_buffer,string_length_copy);
-				memcpy(string_buffer_print+string_length_copy,
-					"<TRUNCATED>", 12);
+				if (string_data) {
+					/* alloc maximum data area */
+					string_buffer_print = (char*)g_malloc(string_length_copy + 12 + 1);
+					/* copy over the data */
+					memcpy(string_buffer_print,string_buffer,string_length_copy);
+					/* append a 0 byte for sure printing */
+					string_buffer_print[string_length_copy] = '\0';
+					/* append <TRUNCATED> */
+					/* This way, we get the TRUNCATED even
+					   in the case of totally wrong packets,
+					   where \0 are inside the string.
+					   TRUNCATED will appear at the
+					   first \0 or at the end (where we 
+					   put the securing \0).
+					*/
+					strcat(string_buffer_print,"<TRUNCATED>");
+				}
+				else {
+					string_buffer_print = g_strdup("<DATA><TRUNCATED>");
+				}
 			}
 			else {
-				string_buffer_print = g_strdup(string_buffer);
+				if (string_data) {
+					string_buffer_print = g_strdup(string_buffer);
+				}
+				else {
+					string_buffer_print = g_strdup("<DATA>");
+				}
 			}
 		}
 		else {
@@ -437,7 +501,9 @@ dissect_rpc_string(const u_char *pd, int offset, frame_data *fd, proto_tree *tre
 		}
 	}
 	else {
-		truncated_length = 1;
+		length_truncated = 1;
+		string_truncated = 2;
+		fill_truncated = 2;
 		string_buffer = g_strdup("");
 		string_buffer_print = g_strdup("<TRUNCATED>");
 	}
@@ -445,34 +511,47 @@ dissect_rpc_string(const u_char *pd, int offset, frame_data *fd, proto_tree *tre
 	if (tree) {
 		string_item = proto_tree_add_text(tree,offset+0, END_OF_FRAME,
 			"%s: %s", proto_registrar_get_name(hfindex), string_buffer_print);
-		proto_tree_add_item_hidden(tree, hfindex, offset+4,
-			string_length, string_buffer);
+		if (string_data) {
+			proto_tree_add_item_hidden(tree, hfindex, offset+4,
+				string_length_copy, string_buffer);
+		}
 		if (string_item) {
 			string_tree = proto_item_add_subtree(string_item, ett_rpc_string);
 		}
 	}
-	if (string_tree) {
-		if (truncated_length) {
+	if (length_truncated) {
+		if (string_tree)
 			proto_tree_add_text(string_tree,
 				offset,pi.captured_len-offset,
 				"length: <TRUNCATED>");
-			offset = pi.captured_len;
-		}
-		else {
+		offset = pi.captured_len;
+	} else {
+		if (string_tree)
 			proto_tree_add_text(string_tree,offset+0,4,
 				"length: %u", string_length);
-			offset += 4;
-		}
-		proto_tree_add_text(string_tree,offset,string_length_copy,
-			"text: %s", string_buffer_print);
+		offset += 4;
+
+		if (string_tree)
+			proto_tree_add_text(string_tree,offset,string_length_copy,
+				"contents: %s", string_buffer_print);
 		offset += string_length_copy;
-		if (string_fill) {
-			proto_tree_add_text(string_tree,offset,string_fill,
-				"fill bytes: opaque data");
-			offset += string_fill;
+		if (fill_length) {
+			if (string_tree) {
+				if (fill_truncated) {
+					proto_tree_add_text(string_tree,
+					offset,fill_length_copy,
+					"fill bytes: opaque data<TRUNCATED>");
+				}
+				else {
+					proto_tree_add_text(string_tree,
+					offset,fill_length_copy,
+					"fill bytes: opaque data");
+				}
+			}
+			offset += fill_length_copy;
 		}
 	}
-
+	
 	if (string_item) {
 		proto_item_set_len(string_item, offset - old_offset);
 	}
@@ -483,6 +562,24 @@ dissect_rpc_string(const u_char *pd, int offset, frame_data *fd, proto_tree *tre
 }
 
 
+int
+dissect_rpc_string(const u_char *pd, int offset, frame_data *fd, proto_tree *tree, int hfindex)
+{
+	offset = dissect_rpc_opaque_data(pd, offset, fd, tree, hfindex, 1);
+
+	return offset;
+}
+
+
+int
+dissect_rpc_data(const u_char *pd, int offset, frame_data *fd, proto_tree *tree, int hfindex)
+{
+	offset = dissect_rpc_opaque_data(pd, offset, fd, tree, hfindex, 0);
+
+	return offset;
+}
+
+
 void
 dissect_rpc_auth( const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 {
@@ -849,7 +946,7 @@ dissect_rpc( const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 		}
 		if (rpc_tree) {
 			proto_tree_add_item_format(rpc_tree,
-				hf_rpc_procedure, offset+12, 4, prog,
+				hf_rpc_procedure, offset+12, 4, proc,
 				"Procedure: %s (%u)", procname, proc);
 		}
 
@@ -885,6 +982,12 @@ dissect_rpc( const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 			/* duplicate request */
 			if (check_col(fd, COL_INFO)) {
 				col_append_fstr(fd, COL_INFO, " dup XID 0x%x", xid);
+				if (rpc_tree) {
+					proto_tree_add_item_hidden(rpc_tree,
+						hf_rpc_dup, 0,0, xid);
+					proto_tree_add_item_hidden(rpc_tree,
+						hf_rpc_call_dup, 0,0, xid);
+				}
 			}
 		}
 		else {
@@ -965,13 +1068,19 @@ dissect_rpc( const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 			proto_tree_add_item(rpc_tree,
 				hf_rpc_programversion, 0, 0, vers);
 			proto_tree_add_item_format(rpc_tree,
-				hf_rpc_procedure, 0, 0, prog,
+				hf_rpc_procedure, 0, 0, proc,
 				"Procedure: %s (%u)", procname, proc);
 		}
 
 		if (rpc_call->replies>1) {
 			if (check_col(fd, COL_INFO)) {
 				col_append_fstr(fd, COL_INFO, " dup XID 0x%x", xid);
+				if (rpc_tree) {
+					proto_tree_add_item_hidden(rpc_tree,
+						hf_rpc_dup, 0,0, xid);
+					proto_tree_add_item_hidden(rpc_tree,
+						hf_rpc_reply_dup, 0,0, xid);
+				}
 			}
 		}
 
@@ -1076,7 +1185,7 @@ dissect_rpc_prog:
 			proto_tree_add_item(ptree,
 				hf_rpc_programversion, 0, 0, vers);
 			proto_tree_add_item_format(ptree,
-				hf_rpc_procedure, 0, 0, prog,
+				hf_rpc_procedure, 0, 0, proc,
 				"Procedure: %s (%u)", procname, proc);
 		}
 	}
@@ -1175,6 +1284,15 @@ proto_register_rpc(void)
 		{ &hf_rpc_auth_machinename, {
 			"Machine Name", "rpc.auth.machinename", FT_STRING, 
 			BASE_DEC, NULL, 0, "Machine Name" }},
+		{ &hf_rpc_dup, {
+			"Duplicate Transaction", "rpc.dup", FT_UINT32, BASE_DEC,
+			NULL, 0, "Duplicate Transaction" }},
+		{ &hf_rpc_call_dup, {
+			"Duplicate Call", "rpc.call.dup", FT_UINT32, BASE_DEC,
+			NULL, 0, "Duplicate Call" }},
+		{ &hf_rpc_reply_dup, {
+			"Duplicate Reply", "rpc.reply.dup", FT_UINT32, BASE_DEC,
+			NULL, 0, "Duplicate Reply" }}
 	};
 	static gint *ett[] = {
 		&ett_rpc,
