@@ -1,7 +1,7 @@
 /* packet-tcp.c
  * Routines for TCP packet disassembly
  *
- * $Id: packet-tcp.c,v 1.47 1999/11/28 03:35:10 gerald Exp $
+ * $Id: packet-tcp.c,v 1.54 1999/12/09 20:54:32 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -37,7 +37,7 @@
 
 #include <stdio.h>
 #include <glib.h>
-#include "packet.h"
+#include "globals.h"
 #include "resolv.h"
 #include "follow.h"
 #include "util.h"
@@ -51,6 +51,10 @@
 # include "snprintf.h"
 #endif
 
+#ifdef HAVE_DLFCN_H
+#include "plugins.h"
+#endif
+
 #ifndef __PACKET_IP_H__
 #include "packet-ip.h"
 #endif
@@ -85,23 +89,32 @@ static gint ett_tcp_option_sack = -1;
 
 /* TCP Ports */
 
-#define TCP_PORT_FTPDATA  20
-#define TCP_PORT_FTP      21
-#define TCP_PORT_TELNET   23
-#define TCP_PORT_SMTP     25
-#define TCP_PORT_HTTP     80
-#define TCP_PORT_POP      110
-#define TCP_PORT_NNTP     119
-#define TCP_PORT_NTP      123
-#define TCP_PORT_NBSS     139
-#define TCP_PORT_IMAP     143
-#define TCP_PORT_BGP      179
-#define TCP_PORT_PRINTER  515
-#define TCP_ALT_PORT_HTTP 8080
-#define TCP_PORT_PPTP     1723
-#define TCP_PORT_RTSP     554
-#define TCP_PORT_YHOO     5050
-#define TCP_PORT_MAPI     1065
+#define TCP_PORT_FTPDATA		20
+#define TCP_PORT_FTP			21
+#define TCP_PORT_TELNET			23
+#define TCP_PORT_SMTP			25
+#define TCP_PORT_HTTP			80
+#define TCP_PORT_TACACS			49
+#define TCP_PORT_POP			110
+#define TCP_PORT_NNTP			119
+#define TCP_PORT_NTP			123
+#define TCP_PORT_NBSS			139
+#define TCP_PORT_IMAP			143
+#define TCP_PORT_BGP			179
+#define TCP_PORT_LDAP			389
+#define TCP_PORT_SRVLOC			427
+#define TCP_PORT_PRINTER		515
+#define TCP_PORT_NCP			524
+#define TCP_PORT_RTSP			554
+#define TCP_PORT_MAPI			1065
+#define TCP_PORT_TNS			1521
+#define TCP_PORT_PPTP			1723
+#define TCP_PORT_PROXY_HTTP		3128
+#define TCP_PORT_PROXY_ADMIN_HTTP	3132
+#define TCP_PORT_YHOO			5050
+#define TCP_ALT_PORT_HTTP		8080
+#define TCP_PORT_IRC			6667
+	/* good candidate for dynamic port specification */
 
 /* TCP structs and definitions */
 
@@ -476,6 +489,22 @@ dissect_tcp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
      (it could be an ACK-only packet) */
   if (packet_max > offset) {
 
+    /* try to apply the plugins */
+#ifdef HAVE_DLFCN_H
+    plugin *pt_plug = plugin_list;
+
+    if (pt_plug) {
+      while (pt_plug) {
+	if (pt_plug->enabled && !strcmp(pt_plug->protocol, "tcp") &&
+	    tree && dfilter_apply(pt_plug->filter, tree, pd)) {
+	  pt_plug->dissector(pd, offset, fd, tree);
+	  goto reas;
+	}
+	pt_plug = pt_plug->next;
+      }
+    }
+#endif
+
     /* ONC RPC.  We can't base this on anything in the TCP header; we have
        to look at the payload.  If "dissect_rpc()" returns TRUE, it was
        an RPC packet, otherwise it's some other type of packet. */
@@ -512,7 +541,8 @@ dissect_tcp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
       pi.match_port = TCP_PORT_PPTP;
       dissect_pptp(pd, offset, fd, tree);
     } else if (PORT_IS(TCP_PORT_HTTP) || PORT_IS(TCP_ALT_PORT_HTTP)
-            || PORT_IS(631))
+            || PORT_IS(631) || PORT_IS(TCP_PORT_PROXY_HTTP)
+            || PORT_IS(TCP_PORT_PROXY_ADMIN_HTTP))
       dissect_http(pd, offset, fd, tree);
     else if (PORT_IS(TCP_PORT_NBSS)) {
       pi.match_port = TCP_PORT_NBSS;
@@ -522,9 +552,27 @@ dissect_tcp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
     else if (PORT_IS(TCP_PORT_BGP)) {
       pi.match_port = TCP_PORT_BGP;
       dissect_bgp(pd, offset, fd, tree);
+    } else if (PORT_IS(TCP_PORT_TACACS)) {
+      pi.match_port = TCP_PORT_TACACS;
+      dissect_tacplus(pd, offset, fd, tree);
     } else if (PORT_IS(TCP_PORT_MAPI)) {
       pi.match_port = TCP_PORT_MAPI;
       dissect_mapi(pd, offset, fd, tree);
+    } else if (PORT_IS(TCP_PORT_TNS)) {
+      pi.match_port = TCP_PORT_TNS;
+      dissect_tns(pd, offset, fd, tree);
+    } else if (PORT_IS(TCP_PORT_IRC)) {
+      pi.match_port = TCP_PORT_IRC;
+      dissect_irc(pd, offset, fd, tree);
+    } else if (PORT_IS(TCP_PORT_LDAP)) {
+      pi.match_port = TCP_PORT_LDAP;
+      dissect_ldap(pd, offset, fd, tree);
+    } else if (PORT_IS(TCP_PORT_SRVLOC)) {
+      pi.match_port = TCP_PORT_SRVLOC;
+      dissect_srvloc(pd, offset, fd, tree);
+    } else if (PORT_IS(TCP_PORT_NCP)) {
+      pi.match_port = TCP_PORT_NCP;
+      dissect_ncp(pd, offset, fd, tree);
     } else {
         /* check existence of high level protocols */
 
