@@ -3,7 +3,7 @@
  *
  * Richard Sharpe <rsharpe@ns.aus.com>
  *
- * $Id: packet-tftp.c,v 1.5 1999/07/29 05:47:06 gram Exp $
+ * $Id: packet-tftp.c,v 1.7 1999/11/16 11:43:00 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@unicom.net>
@@ -42,6 +42,10 @@
 #include "packet.h"
 
 static int proto_tftp = -1;
+static int hf_tftp_type = -1;
+static int hf_tftp_error_code = -1;
+
+static gint ett_tftp = -1;
 
 #define	RRQ	1
 #define	WRQ	2
@@ -91,9 +95,12 @@ dissect_tftp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 	if (tree) {
 
 	  ti = proto_tree_add_item(tree, proto_tftp, offset, END_OF_FRAME, NULL);
-	  tftp_tree = proto_item_add_subtree(ti, ETT_TFTP);
+	  tftp_tree = proto_item_add_subtree(ti, ett_tftp);
 
-	  switch (i1 = pntohs(pd+offset)) {
+	  i1 = pntohs(pd+offset);
+	  proto_tree_add_item_hidden(tftp_tree, hf_tftp_type, offset, 2, i1);
+	    
+	  switch (i1) {
 	  case RRQ:
 	    proto_tree_add_text(tftp_tree, offset, 2, "Read Request");
 	    offset += 2;
@@ -129,6 +136,7 @@ dissect_tftp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 	    proto_tree_add_text(tftp_tree, offset, 2, "Error Code");
 	    offset += 2;
 	    i1 = pntohs(pd+offset);
+	    proto_tree_add_item_hidden(tftp_tree, hf_tftp_error_code, offset, 2, i1);
 	    proto_tree_add_text(tftp_tree, offset, 2, "Code = %s", tftp_errors[i1 % 8]);
 	    offset += 2;
 	    proto_tree_add_text(tftp_tree, offset, END_OF_FRAME, "Error Message: %s", pd + offset);
@@ -147,11 +155,23 @@ dissect_tftp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 void
 proto_register_tftp(void)
 {
-/*        static hf_register_info hf[] = {
-                { &variable,
-                { "Name",           "tftp.abbreviation", TYPE, VALS_POINTER }},
-        };*/
 
-        proto_tftp = proto_register_protocol("Trivial File Transfer Protocol", "tftp");
- /*       proto_register_field_array(proto_tftp, hf, array_length(hf));*/
+  static hf_register_info hf[] = {
+    { &hf_tftp_type,
+      { "Type",		      "tftp.type",
+	FT_UINT16, BASE_DEC, NULL, 0x0,
+      	"TFTP message type" }},
+
+    { &hf_tftp_error_code,
+      { "Error code",         "tftp.error.code",
+	FT_UINT16, BASE_DEC, NULL, 0x0,
+      	"Error code in case of TFTP error message" }}
+  };
+  static gint *ett[] = {
+    &ett_tftp,
+  };
+
+  proto_tftp = proto_register_protocol("Trivial File Transfer Protocol", "tftp");
+  proto_register_field_array(proto_tftp, hf, array_length(hf));
+  proto_register_subtree_array(ett, array_length(ett));
 }
