@@ -2,7 +2,7 @@
  * Routines for Token-Ring Media Access Control
  * Gilbert Ramirez <gram@verdict.uthscsa.edu>
  *
- * $Id: packet-trmac.c,v 1.15 1999/09/17 04:20:23 gram Exp $
+ * $Id: packet-trmac.c,v 1.17 1999/11/16 11:43:01 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@unicom.net>
@@ -55,6 +55,10 @@ static int hf_trmac_errors_freq = -1;
 static int hf_trmac_errors_token = -1;
 static int hf_trmac_naun = -1;
 
+static gint ett_tr_mac = -1;
+static gint ett_tr_ierr_cnt = -1;
+static gint ett_tr_nerr_cnt = -1;
+
 /* Major Vector */
 static value_string major_vector_vs[] = {
 		{ 0x00, "Response" },
@@ -230,7 +234,7 @@ sv_text(const u_char *pd, int pkt_offset, proto_tree *tree)
 			memcpy(errors, &pd[2], 6);
 			ti = proto_tree_add_item(tree, hf_trmac_errors_iso, pkt_offset+1, sv_length-1,
 				errors[0] + errors[1] + errors[2] + errors[3] + errors[4]);
-			sv_tree = proto_item_add_subtree(ti, ETT_TR_IERR_CNT);
+			sv_tree = proto_item_add_subtree(ti, ett_tr_ierr_cnt);
 
 			proto_tree_add_item(sv_tree, hf_trmac_errors_line, pkt_offset+2, 1, errors[0]);
 			proto_tree_add_item(sv_tree, hf_trmac_errors_internal, pkt_offset+3, 1, errors[1]);
@@ -244,7 +248,7 @@ sv_text(const u_char *pd, int pkt_offset, proto_tree *tree)
 			memcpy(errors, &pd[2], 6);
 			ti = proto_tree_add_item(tree, hf_trmac_errors_noniso, pkt_offset+1, sv_length-1,
 				errors[0] + errors[1] + errors[2] + errors[3] + errors[4]);
-			sv_tree = proto_item_add_subtree(ti, ETT_TR_NERR_CNT);
+			sv_tree = proto_item_add_subtree(ti, ett_tr_nerr_cnt);
 
 			proto_tree_add_item(sv_tree, hf_trmac_errors_lost, pkt_offset+2, 1, errors[0]);
 			proto_tree_add_item(sv_tree, hf_trmac_errors_congestion, pkt_offset+3, 1, errors[1]);
@@ -289,7 +293,7 @@ dissect_trmac(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
 	if (tree) {
 
 		ti = proto_tree_add_item(tree, proto_trmac, offset, mv_length, NULL);
-		mac_tree = proto_item_add_subtree(ti, ETT_TR_MAC);
+		mac_tree = proto_item_add_subtree(ti, ett_tr_mac);
 
 		proto_tree_add_item(mac_tree, hf_trmac_mv, offset+3, 1, mv_val);
 		proto_tree_add_item_format(mac_tree, hf_trmac_length, offset, 2, mv_length,
@@ -320,60 +324,84 @@ proto_register_trmac(void)
 {
         static hf_register_info hf[] = {
                 { &hf_trmac_mv,
-                { "Major Vector",			"trmac.mvec", FT_VALS_UINT8, VALS(major_vector_vs) }},
+                { "Major Vector",			"trmac.mvec", FT_UINT8, BASE_HEX, major_vector_vs, 0x0,
+			"" }},
 
                 { &hf_trmac_length,
-                { "Total Length",			"trmac.length", FT_UINT8, NULL }},
+                { "Total Length",			"trmac.length", FT_UINT8, BASE_DEC, NULL, 0x0,
+			"" }},
 
                 { &hf_trmac_srcclass,
-                { "Source Class",			"trmac.srcclass", FT_VALS_UINT8, VALS(classes_vs) }},
+                { "Source Class",			"trmac.srcclass", FT_UINT8, BASE_HEX, classes_vs, 0x0,
+			"" }},
 
                 { &hf_trmac_dstclass,
-                { "Destination Class",			"trmac.dstclass", FT_VALS_UINT8, VALS(classes_vs) }},
+                { "Destination Class",			"trmac.dstclass", FT_UINT8, BASE_HEX, classes_vs, 0x0,
+			"" }},
 
                 { &hf_trmac_sv,
-                { "Sub-Vector",				"trmac.svec", FT_UINT8, NULL }},
+                { "Sub-Vector",				"trmac.svec", FT_UINT8, BASE_HEX, NULL, 0x0,
+			"" }},
 
 		{ &hf_trmac_errors_iso,
-		{ "Isolating Errors",			"trmac.errors.iso", FT_UINT16, NULL }},
+		{ "Isolating Errors",			"trmac.errors.iso", FT_UINT16, BASE_DEC, NULL, 0x0,
+			"" }},
 
 		{ &hf_trmac_errors_line,
-		{ "Line Errors",			"trmac.errors.line", FT_UINT8, NULL }},
+		{ "Line Errors",			"trmac.errors.line", FT_UINT8, BASE_DEC, NULL, 0x0,
+			"" }},
 
 		{ &hf_trmac_errors_internal,
-		{ "Internal Errors",			"trmac.errors.internal", FT_UINT8, NULL }},
+		{ "Internal Errors",			"trmac.errors.internal", FT_UINT8, BASE_DEC, NULL, 0x0,
+			"" }},
 
 		{ &hf_trmac_errors_burst,
-		{ "Burst Errors",			"trmac.errors.burst", FT_UINT8, NULL }},
+		{ "Burst Errors",			"trmac.errors.burst", FT_UINT8, BASE_DEC, NULL, 0x0,
+			"" }},
 
 		{ &hf_trmac_errors_ac,
-		{ "A/C Errors",				"trmac.errors.ac", FT_UINT8, NULL }},
+		{ "A/C Errors",				"trmac.errors.ac", FT_UINT8, BASE_DEC, NULL, 0x0,
+			"" }},
 
 		{ &hf_trmac_errors_abort,
-		{ "Abort Delimiter Transmitted Errors",	"trmac.errors.abort", FT_UINT8, NULL }},
+		{ "Abort Delimiter Transmitted Errors",	"trmac.errors.abort", FT_UINT8, BASE_DEC, NULL, 0x0,
+			"" }},
 
 		{ &hf_trmac_errors_noniso,
-		{ "Non-Isolating Errors",		"trmac.errors.noniso", FT_UINT16, NULL }},
+		{ "Non-Isolating Errors",		"trmac.errors.noniso", FT_UINT16, BASE_DEC, NULL, 0x0,
+			"" }},
 
 		{ &hf_trmac_errors_lost,
-		{ "Lost Frame Errors",			"trmac.errors.lost", FT_UINT8, NULL }},
+		{ "Lost Frame Errors",			"trmac.errors.lost", FT_UINT8, BASE_DEC, NULL, 0x0,
+			"" }},
 
 		{ &hf_trmac_errors_congestion,
-		{ "Receiver Congestion Errors",		"trmac.errors.congestion", FT_UINT8, NULL }},
+		{ "Receiver Congestion Errors",		"trmac.errors.congestion", FT_UINT8, BASE_DEC, NULL, 0x0,
+			"" }},
 
 		{ &hf_trmac_errors_fc,
-		{ "Frame-Copied Errors",		"trmac.errors.fc", FT_UINT8, NULL }},
+		{ "Frame-Copied Errors",		"trmac.errors.fc", FT_UINT8, BASE_DEC, NULL, 0x0,
+			"" }},
 
 		{ &hf_trmac_errors_freq,
-		{ "Frequency Errors",			"trmac.errors.freq", FT_UINT8, NULL }},
+		{ "Frequency Errors",			"trmac.errors.freq", FT_UINT8, BASE_DEC, NULL, 0x0,
+			"" }},
 
 		{ &hf_trmac_errors_token,
-		{ "Token Errors",			"trmac.errors.token", FT_UINT8, NULL }},
+		{ "Token Errors",			"trmac.errors.token", FT_UINT8, BASE_DEC, NULL, 0x0,
+			"" }},
 
 		{ &hf_trmac_naun,
-		{ "NAUN",				"trmac.naun", FT_ETHER, NULL }}
+		{ "NAUN",				"trmac.naun", FT_ETHER, BASE_DEC, NULL, 0x0,
+			"" }},
         };
+	static gint *ett[] = {
+		&ett_tr_mac,
+		&ett_tr_ierr_cnt,
+		&ett_tr_nerr_cnt,
+	};
 
         proto_trmac = proto_register_protocol("Token-Ring Media Access Control", "trmac");
 	proto_register_field_array(proto_trmac, hf, array_length(hf));
+	proto_register_subtree_array(ett, array_length(ett));
 }
