@@ -1,7 +1,7 @@
 /* packet-udp.c
  * Routines for UDP packet disassembly
  *
- * $Id: packet-udp.c,v 1.38 1999/11/23 17:09:57 gram Exp $
+ * $Id: packet-udp.c,v 1.44 1999/12/12 05:11:44 gram Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -41,9 +41,13 @@
 #include <stdlib.h>
 
 #include <glib.h>
-#include "packet.h"
+#include "globals.h"
 #include "resolv.h"
 
+#ifdef HAVE_DLFCN_H
+#include "plugins.h"
+#endif
+
 static int proto_udp = -1;		
 static int hf_udp_srcport = -1;
 static int hf_udp_dstport = -1;
@@ -64,6 +68,7 @@ typedef struct _e_udphdr {
 
 /* UDP Ports -> should go in packet-udp.h */
 
+#define UDP_PORT_TACACS  49
 #define UDP_PORT_DNS     53
 #define UDP_PORT_BOOTPS  67
 #define UDP_PORT_TFTP    69
@@ -72,10 +77,13 @@ typedef struct _e_udphdr {
 #define UDP_PORT_NBNS	137
 #define UDP_PORT_NBDGM	138
 #define UDP_PORT_SNMP   161
+#define UDP_PORT_SRVLOC 427
 #define UDP_PORT_PIM_RP_DISC 496
 #define UDP_PORT_ISAKMP	500
+#define UDP_PORT_WHO    513
 #define UDP_PORT_RIP    520
 #define UDP_PORT_RIPNG  521
+#define UDP_PORT_NCP    524
 #define UDP_PORT_VINES	573
 #define UDP_PORT_RADIUS 1645
 #define UDP_PORT_RADIUS_NEW 1812
@@ -88,6 +96,7 @@ typedef struct _e_udphdr {
 #define UDP_PORT_RX_LOW 7000
 #define UDP_PORT_RX_HIGH 7009
 #define UDP_PORT_RX_AFS_BACKUPS 7021
+#define UDP_PORT_WCCP	2048
 
 struct hash_struct {
   guint16 proto;
@@ -177,7 +186,7 @@ void init_dissect_udp(void) {
   udp_hash_add(UDP_PORT_SAP, dissect_sap);
   udp_hash_add(UDP_PORT_HSRP, dissect_hsrp);
   udp_hash_add(UDP_PORT_PIM_RP_DISC, dissect_auto_rp);
-
+  udp_hash_add(UDP_PORT_TACACS, dissect_tacacs);
 }
 
 void
@@ -236,12 +245,32 @@ dissect_udp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
   if (dissect_rpc(pd, offset, fd, tree))
     return;
 
+  /* try to apply the plugins */
+#ifdef HAVE_DLFCN_H
+  {
+      plugin *pt_plug = plugin_list;
+
+      if (pt_plug) {
+	  while (pt_plug) {
+	      if (pt_plug->enabled && !strcmp(pt_plug->protocol, "udp") &&
+		  tree && dfilter_apply(pt_plug->filter, tree, pd)) {
+		  pt_plug->dissector(pd, offset, fd, tree);
+		  return;
+	      }
+	      pt_plug = pt_plug->next;
+	  }
+      }
+  }
+#endif
+
   /* XXX - we should do all of this through the table of ports. */
 #define PORT_IS(port)	(uh_sport == port || uh_dport == port)
   if (PORT_IS(UDP_PORT_BOOTPS))
       dissect_bootp(pd, offset, fd, tree);
   else if (PORT_IS(UDP_PORT_DNS))
       dissect_dns(pd, offset, fd, tree);
+  else if (PORT_IS(UDP_PORT_SRVLOC))
+      dissect_srvloc(pd, offset, fd, tree);
   else if (PORT_IS(UDP_PORT_ISAKMP))
       dissect_isakmp(pd, offset, fd, tree);
   else if (PORT_IS(UDP_PORT_RIP)) {
@@ -249,22 +278,24 @@ dissect_udp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
       dissect_rip(pd, offset, fd, tree);
   } else if (PORT_IS(UDP_PORT_RIPNG))
       dissect_ripng(pd, offset, fd, tree);
+  else if (PORT_IS(UDP_PORT_NCP))
+      dissect_ncp(pd, offset, fd, tree);
   else if (PORT_IS(UDP_PORT_NBNS))
       dissect_nbns(pd, offset, fd, tree);
   else if (PORT_IS(UDP_PORT_NBDGM))
       dissect_nbdgm(pd, offset, fd, tree);
   else if (PORT_IS(UDP_PORT_NTP))
       dissect_ntp(pd, offset, fd, tree);
+  else if (PORT_IS(UDP_PORT_WHO))
+      dissect_who(pd, offset, fd, tree);
   else if (PORT_IS(UDP_PORT_IPX)) /* RFC 1234 */
       dissect_ipx(pd, offset, fd, tree);
   else if ((uh_sport >= UDP_PORT_RX_LOW && uh_sport <= UDP_PORT_RX_HIGH) ||
 	(uh_dport >= UDP_PORT_RX_LOW && uh_dport <= UDP_PORT_RX_HIGH) ||
 	PORT_IS(UDP_PORT_RX_AFS_BACKUPS)) 
       dissect_rx(pd, offset, fd, tree); /* transarc AFS's RX protocol */
-#if defined(HAVE_UCD_SNMP_SNMP_H) || defined(HAVE_SNMP_SNMP_H)
   else if (PORT_IS(UDP_PORT_SNMP))
       dissect_snmp(pd, offset, fd, tree);
-#endif
   else if (PORT_IS(UDP_PORT_VINES)) {
       /* FIXME: AFAIK, src and dst port must be the same */
       dissect_vines_frp(pd, offset, fd, tree);
@@ -281,6 +312,8 @@ dissect_udp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) {
 	dissect_icp(pd,offset,fd,tree);
  } else if ( PORT_IS(UDP_PORT_ICQ)) {
         dissect_icq(pd,offset,fd,tree);
+ } else if (PORT_IS(UDP_PORT_WCCP) ) {
+ 	dissect_wccp(pd, offset, fd, tree);
  } else {
       /* OK, find a routine in the table, else use the default */
 
