@@ -1,7 +1,7 @@
 /* packet-vines.c
  * Routines for Banyan VINES protocol packet disassembly
  *
- * $Id: packet-vines.c,v 1.7 1999/07/07 22:51:57 gram Exp $
+ * $Id: packet-vines.c,v 1.10 1999/11/16 11:43:02 guy Exp $
  *
  * Don Lafontaine <lafont02@cn.ca>
  *
@@ -40,6 +40,9 @@
 #include "packet.h"
 #include "packet-vines.h"
 
+static gint ett_vines = -1;
+static gint ett_vines_frp = -1;
+static gint ett_vines_spp = -1;
 
 /* AFAIK Vines FRP (Fragmentation Protocol) is used on all media except Ethernet
  * and TR (and probably FDDI) - Fragmentation on these media types is not possible
@@ -85,7 +88,7 @@ dissect_vines_frp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree
   
   if (tree) {
     ti = proto_tree_add_text(tree, offset, 2, "Vines Fragmentation Protocol");
-    vines_frp_tree = proto_item_add_subtree(ti, ETT_VINES_FRP);
+    vines_frp_tree = proto_item_add_subtree(ti, ett_vines_frp);
     proto_tree_add_text(vines_frp_tree, offset,     1, "Control Flags: 0x%02x = %s fragment", vines_frp_ctrl, frp_flags_str);
     proto_tree_add_text(vines_frp_tree, offset + 1, 1, "Sequence Number: 0x%02x", vines_frp_seqno);
   }
@@ -97,6 +100,24 @@ dissect_vines_frp(const u_char *pd, int offset, frame_data *fd, proto_tree *tree
   dissect_vines(pd, offset, fd, tree);
 }
 
+gchar *
+vines_addr_to_str(const guint8 *addrp)
+{
+  static gchar	str[3][214];
+  static gchar	*cur;
+
+  if (cur == &str[0][0]) {
+    cur = &str[1][0];
+  } else if (cur == &str[1][0]) {
+    cur = &str[2][0];
+  } else {
+    cur = &str[0][0];
+  }
+
+  sprintf(cur, "%08x.%04x", pntohl(&addrp[0]), pntohs(&addrp[4]));
+  return cur;
+}
+
 void
 dissect_vines(const u_char *pd, int offset, frame_data *fd, proto_tree *tree) 
 	{
@@ -114,8 +135,8 @@ dissect_vines(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
   	viph.vip_chksum = pntohs(&pd[offset]);
   	viph.vip_pktlen = pntohs(&pd[offset+2]);
   	viph.vip_dnet = pntohl(&pd[offset+6]);
-  	viph.vip_snet = pntohl(&pd[offset+12]);
   	viph.vip_dsub = pntohs(&pd[offset+10]);
+  	viph.vip_snet = pntohl(&pd[offset+12]);
   	viph.vip_ssub = pntohs(&pd[offset+16]);
 
     	switch (viph.vip_proto) {
@@ -156,10 +177,10 @@ dissect_vines(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
 			col_add_fstr(fd, COL_INFO, "Unknown VIP protocol (%02x)", viph.vip_proto);
 	}
 
-			if (check_col(fd, COL_RES_NET_SRC))
-  	  	col_add_fstr(fd, COL_RES_NET_SRC, "%08x.%04x", viph.vip_snet, viph.vip_ssub);
-			if (check_col(fd, COL_RES_NET_DST))
-    		col_add_fstr(fd, COL_RES_NET_DST, "%08x.%04x", viph.vip_dnet, viph.vip_dsub);
+	SET_ADDRESS(&pi.net_src, AT_VINES, 6, &pd[offset+12]);
+	SET_ADDRESS(&pi.src, AT_VINES, 6, &pd[offset+12]);
+	SET_ADDRESS(&pi.net_dst, AT_VINES, 6, &pd[offset+6]);
+	SET_ADDRESS(&pi.dst, AT_VINES, 6, &pd[offset+6]);
 
  	/* helpers to decode flags */
 	/* FIXME: Not used yet */
@@ -195,7 +216,7 @@ dissect_vines(const u_char *pd, int offset, frame_data *fd, proto_tree *tree)
   	if (tree) 
   		{
     	ti = proto_tree_add_text(tree, offset, (viph.vip_pktlen), "Vines IP");
-    	vip_tree = proto_item_add_subtree(ti, ETT_VINES);
+    	vip_tree = proto_item_add_subtree(ti, ett_vines);
     	proto_tree_add_text(vip_tree, offset,      2, "Packet checksum: 0x%04x", viph.vip_chksum);
     	proto_tree_add_text(vip_tree, offset +  2, 2, "Packet length: 0x%04x (%d)", viph.vip_pktlen, viph.vip_pktlen); 
     	proto_tree_add_text(vip_tree, offset +  4, 1, "Transport control: 0x%02x",
@@ -282,7 +303,7 @@ void dissect_vines_spp(const u_char *pd, int offset, frame_data *fd, proto_tree
   	if (tree) 
   		{
     	ti = proto_tree_add_text(tree, offset, sizeof(viph), "Vines SPP");
-    	vspp_tree = proto_item_add_subtree(ti, ETT_VINES_SPP);
+    	vspp_tree = proto_item_add_subtree(ti, ett_vines_spp);
     	proto_tree_add_text(vspp_tree, offset,      2, "Source port: 0x%04x", viph.vspp_sport);
     	proto_tree_add_text(vspp_tree, offset+2,    2, "Destination port: 0x%04x", viph.vspp_dport); 
     	proto_tree_add_text(vspp_tree, offset+4,    1, "Packet type: 0x%02x", viph.vspp_pkttype);
@@ -292,6 +313,17 @@ void dissect_vines_spp(const u_char *pd, int offset, frame_data *fd, proto_tree
     	proto_tree_add_text(vspp_tree, offset+10,   2, "Sequence number: 0x%04x", viph.vspp_seqno);
     	proto_tree_add_text(vspp_tree, offset+12,   2, "Ack number: 0x%04x", viph.vspp_ack);
     	proto_tree_add_text(vspp_tree, offset+14,   2, "Window: 0x%04x", viph.vspp_win);
-  		}
-
+		}
 	}
+
+void
+proto_register_vines(void)
+{
+	static gint *ett[] = {
+		&ett_vines,
+		&ett_vines_frp,
+		&ett_vines_spp,
+	};
+
+	proto_register_subtree_array(ett, array_length(ett));
+}
