@@ -1,7 +1,7 @@
 /* prefs.h
  * Definitions for preference handling routines
  *
- * $Id: prefs.h,v 1.8 1999/09/09 03:31:50 gram Exp $
+ * $Id: prefs.h,v 1.12 1999/12/29 20:09:47 gram Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -32,13 +32,21 @@
 #define PR_DEST_CMD  0
 #define PR_DEST_FILE 1
 
+#ifndef __GTK_H__
+#include <gtk/gtk.h>
+#endif
+
 typedef struct _e_prefs {
-  gint    pr_format;
-  gint    pr_dest;
-  gchar  *pr_file;
-  gchar  *pr_cmd;
-  GList  *col_list;
-  gint    num_cols;
+  gint     pr_format;
+  gint     pr_dest;
+  gchar   *pr_file;
+  gchar   *pr_cmd;
+  GList   *col_list;
+  gint     num_cols;
+  GdkColor st_client_fg, st_client_bg, st_server_fg, st_server_bg;
+  gboolean	gui_scrollbar_on_right;
+  gboolean	gui_plist_sel_browse;
+  gboolean	gui_ptree_sel_browse;
 } e_prefs;
 
 extern e_prefs prefs;
