@@ -1,7 +1,7 @@
 /* print.h
  * Definitions for printing packet analysis trees.
  *
- * $Id: print.h,v 1.12 1999/09/12 20:23:34 guy Exp $
+ * $Id: print.h,v 1.15 1999/11/22 06:24:41 gram Exp $
  *
  * Gilbert Ramirez <gram@verdict.uthscsa.edu>
  *
@@ -28,13 +28,28 @@
 #ifndef __PRINT_H__
 #define __PRINT_H__
 
+typedef struct {
+  gboolean	to_file;	/* TRUE if we're printing to a file */
+  char		*dest;		/* if printing to file, pathname;
+				   if not, command string */
+  gboolean	print_summary;	/* TRUE if we should just print summary;
+				   FALSE if we should print protocol tree. */
+  gboolean	print_hex;	/* TRUE if we should also print hex data;
+				   FALSE if we should print only if not dissected. */
+  gboolean	expand_all;	/* TRUE if we should expand all levels;
+				   FALSE if we should expand as displayed. */
+} print_args_t;
+
 /* Functions in print.h */
 
 FILE *open_print_dest(int to_file, const char *dest);
 void close_print_dest(int to_file, FILE *fh);
 void print_preamble(FILE *fh);
 void print_finale(FILE *fh);
-void proto_tree_print(gboolean print_one_packet, gboolean print_all_levels,
+void print_file(FILE* fh, const char* filename);
+void proto_tree_print(gboolean print_one_packet, print_args_t *print_args,
     GNode *protocol_tree, const u_char *pd, frame_data *fd, FILE *fh);
+void print_hex_data(FILE *fh, register const u_char *cp,
+    register u_int length, char_enc encoding);
 
 #endif /* print.h */
