@@ -4,7 +4,7 @@
  * Creates random packet traces. Useful for debugging sniffers by testing
  * assumptions about the veracity of the data found in the packet.
  *
- * $Id: randpkt.c,v 1.3 1999/09/17 04:38:14 gram Exp $
+ * $Id: randpkt.c,v 1.5 1999/10/07 07:55:12 guy Exp $
  *
  * Copyright (C) 1999 by Gilbert Ramirez <gram@xiexie.org>
  * 
@@ -49,10 +49,13 @@
 /* Types of produceable packets */
 enum {
 	PKT_ARP,
+	PKT_DNS,
 	PKT_ETHERNET,
 	PKT_FDDI,
+	PKT_ICMP,
 	PKT_IP,
 	PKT_LLC,
+	PKT_NBNS,
 	PKT_TCP,
 	PKT_TR,
 	PKT_UDP
@@ -75,6 +78,38 @@ guint8 pkt_arp[] = {
 	0x08, 0x06
 };
 
+/* Ethernet+IP+UP, indicating DNS */
+guint8 pkt_dns[] = {
+	0xff, 0xff, 0xff, 0xff,
+	0xff, 0xff, 0x01, 0x01,
+	0x01, 0x01, 0x01, 0x01,
+	0x08, 0x00,
+
+	0x45, 0x00, 0x00, 0x3c,
+	0xc5, 0x9e, 0x40, 0x00,
+	0xff, 0x11, 0xd7, 0xe0,
+	0xd0, 0x15, 0x02, 0xb8,
+	0x0a, 0x01, 0x01, 0x63,
+
+	0x05, 0xe8, 0x00, 0x35,
+	0x00, 0x00, 0x2a, 0xb9,
+	0x30
+};
+
+/* Ethernet+IP, indicating ICMP */
+guint8 pkt_icmp[] = {
+	0xff, 0xff, 0xff, 0xff,
+	0xff, 0xff, 0x01, 0x01,
+	0x01, 0x01, 0x01, 0x01,
+	0x08, 0x00,
+
+	0x45, 0x00, 0x00, 0x54,
+	0x8f, 0xb3, 0x40, 0x00,
+	0xfd, 0x01, 0x8a, 0x99,
+	0xcc, 0xfc, 0x66, 0x0b,
+	0xce, 0x41, 0x62, 0x12
+};
+
 /* Ethernet, indicating IP */
 guint8 pkt_ip[] = {
 	0xff, 0xff, 0xff, 0xff,
@@ -91,6 +126,24 @@ guint8 pkt_llc[] = {
 	0xc4, 0x67
 };
 
+/* Ethernet+IP+UP, indicating NBNS */
+guint8 pkt_nbns[] = {
+	0xff, 0xff, 0xff, 0xff,
+	0xff, 0xff, 0x01, 0x01,
+	0x01, 0x01, 0x01, 0x01,
+	0x08, 0x00,
+
+	0x45, 0x00, 0x00, 0x3c,
+	0xc5, 0x9e, 0x40, 0x00,
+	0xff, 0x11, 0xd7, 0xe0,
+	0xd0, 0x15, 0x02, 0xb8,
+	0x0a, 0x01, 0x01, 0x63,
+
+	0x00, 0x89, 0x00, 0x89,
+	0x00, 0x00, 0x2a, 0xb9,
+	0x30
+};
+
 /* TR+LLC+IP, indicating TCP */
 guint8 pkt_tcp[] = {
 	0x10, 0x40, 0x68, 0x00,
@@ -127,18 +180,27 @@ pkt_example examples[] = {
 	{ "arp", "Address Resolution Protocol",
 		PKT_ARP,	pkt_arp,	WTAP_ENCAP_ETHERNET,	array_length(pkt_arp) },
 
+	{ "dns", "Domain Name Service",
+		PKT_DNS,	pkt_dns,	WTAP_ENCAP_ETHERNET,	array_length(pkt_dns) },
+
 	{ "eth", "Ethernet",
 		PKT_ETHERNET,	NULL,		WTAP_ENCAP_ETHERNET,	0 },
 
 	{ "fddi", "Fiber Distributed Data Interface",
 		PKT_FDDI,	NULL,		WTAP_ENCAP_FDDI,	0 },
 
+	{ "icmp", "Internet Control Message Protocol",
+		PKT_ICMP,	pkt_icmp,	WTAP_ENCAP_ETHERNET,	array_length(pkt_icmp) },
+
 	{ "ip", "Internet Protocol",
 		PKT_IP,		pkt_ip,		WTAP_ENCAP_ETHERNET,	array_length(pkt_ip) },
 
 	{ "llc", "Logical Link Control",
 		PKT_LLC,	pkt_llc,	WTAP_ENCAP_TR,		array_length(pkt_llc) },
 
+	{ "nbns", "NetBIOS-over-TCP Name Service",
+		PKT_NBNS,	pkt_nbns,	WTAP_ENCAP_ETHERNET,	array_length(pkt_nbns) },
+
 	{ "tcp", "Transmission Control Protocol",
 		PKT_TCP,	pkt_tcp,	WTAP_ENCAP_TR,		array_length(pkt_tcp) },
 
