@@ -1,7 +1,7 @@
 /* resolv.c
  * Routines for network object lookup
  *
- * $Id: resolv.c,v 1.11 1999/09/18 16:01:52 deniel Exp $
+ * $Id: resolv.c,v 1.21 1999/11/22 06:03:46 gram Exp $
  *
  * Laurent Deniel <deniel@worldnet.fr>
  *
@@ -39,7 +39,6 @@
 #endif
 #endif
 
-
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -56,6 +55,8 @@
 #include <netdb.h>
 #endif
 
+#include <arpa/inet.h>
+
 #include <signal.h>
 
 #ifdef HAVE_SYS_SOCKET_H
@@ -66,16 +67,20 @@
 # include <setjmp.h>
 #endif
 
+#ifdef NEED_INET_V6DEFS_H
+# include "inet_v6defs.h"
+#endif
+
 #include "packet.h"
 #include "packet-ipv6.h"
+#include "packet-ipx.h"
+#include "globals.h"
 #include "resolv.h"
 
-#ifndef MAXNAMELEN
-#define MAXNAMELEN  	64	/* max name length (hostname and port name) */
-#endif
 #define MAXMANUFLEN	9	/* max vendor name length with ending '\0' */
 #define HASHETHSIZE	1024
 #define HASHHOSTSIZE	1024
+#define HASHIPXNETSIZE	256
 #define HASHMANUFSIZE   256
 #define HASHPORTSIZE	256
 
@@ -87,6 +92,10 @@ typedef struct hashname {
   struct hashname 	*next;
 } hashname_t;
 
+/* hash table used for IPX network lookup */
+
+typedef struct hashname hashipxnet_t;
+
 /* hash tables used for ethernet and manufacturer lookup */
 
 typedef struct hashmanuf {
@@ -98,6 +107,7 @@ typedef struct hashmanuf {
 typedef struct hashether {
   u_char 		addr[6];
   char 			name[MAXNAMELEN];
+  gboolean		is_name_from_file;
   struct hashether     	*next;
 } hashether_t;
 
@@ -109,13 +119,23 @@ typedef struct _ether
   char 			name[MAXNAMELEN];
 } ether_t;
 
+/* internal ipxnet type */
+
+typedef struct _ipxnet
+{
+  u_int			addr;
+  char 			name[MAXNAMELEN];
+} ipxnet_t;
+
 static hashname_t 	*host_table[HASHHOSTSIZE];
 static hashname_t 	*udp_port_table[HASHPORTSIZE];
 static hashname_t 	*tcp_port_table[HASHPORTSIZE];
 static hashether_t 	*eth_table[HASHETHSIZE];
 static hashmanuf_t 	*manuf_table[HASHMANUFSIZE];
+static hashipxnet_t 	*ipxnet_table[HASHIPXNETSIZE];
 
 static int 		eth_resolution_initialized = 0;
+static int 		ipxnet_resolution_initialized = 0;
 
 /*
  *  Global variables (can be changed in GUI sections)
@@ -125,6 +145,8 @@ int g_resolving_actif = 1; 		/* routines are active by default */
 
 gchar *g_ethers_path  = EPATH_ETHERS;
 gchar *g_pethers_path = NULL; 		/* "$HOME"/EPATH_PERSONAL_ETHERS    */
+gchar *g_ipxnets_path  = EPATH_IPXNETS;
+gchar *g_pipxnets_path = NULL;		/* "$HOME"/EPATH_PERSONAL_IPXNETS   */
 gchar *g_manuf_path   = EPATH_MANUF;	/* may only be changed before the   */
 					/* first resolving call        	    */
 
@@ -369,7 +391,8 @@ static int parse_ether_line(char *line, ether_t *eth, int six_bytes)
   /*
    *  See man ethers(4) for /etc/ethers file format
    *  (not available on all systems).
-   *  We allow both ethernet address separators (':' and '-').
+   *  We allow both ethernet address separators (':' and '-'),
+   *  as well as Ethereal's '.' separator.
    */
 
   gchar *cp;
@@ -383,13 +406,17 @@ static int parse_ether_line(char *line, ether_t *eth, int six_bytes)
 
   if (six_bytes) {
     if (sscanf(cp, "%x:%x:%x:%x:%x:%x", &a0, &a1, &a2, &a3, &a4, &a5) != 6) {
-      if (sscanf(cp, "%x-%x-%x-%x-%x-%x", &a0, &a1, &a2, &a3, &a4, &a5) != 6)
-	return -1;
+      if (sscanf(cp, "%x-%x-%x-%x-%x-%x", &a0, &a1, &a2, &a3, &a4, &a5) != 6) {
+        if (sscanf(cp, "%x.%x.%x.%x.%x.%x", &a0, &a1, &a2, &a3, &a4, &a5) != 6)
+	  return -1;
+      }
     }
   } else {
     if (sscanf(cp, "%x:%x:%x", &a0, &a1, &a2) != 3) {
-      if (sscanf(cp, "%x-%x-%x", &a0, &a1, &a2) != 3)
+      if (sscanf(cp, "%x-%x-%x", &a0, &a1, &a2) != 3) {
+        if (sscanf(cp, "%x.%x.%x", &a0, &a1, &a2) != 3)
 	return -1;
+      }
     }
   }
 
@@ -478,7 +505,7 @@ static ether_t *get_ethbyname(u_char *name)
 
 } /* get_ethbyname */
 
-static ether_t *get_ethbyaddr(u_char *addr)
+static ether_t *get_ethbyaddr(const u_char *addr)
 {
 
   ether_t *eth;
@@ -532,7 +559,7 @@ static void add_manuf_name(u_char *addr, u_char *name)
 
 } /* add_manuf_name */
 
-static hashmanuf_t *manuf_name_lookup(u_char *addr)
+static hashmanuf_t *manuf_name_lookup(const u_char *addr)
 {
 
   hashmanuf_t *tp;
@@ -551,6 +578,36 @@ static hashmanuf_t *manuf_name_lookup(u_char *addr)
 
 } /* manuf_name_lookup */
 
+static void initialize_ethers(void)
+{
+  ether_t *eth;
+
+#ifdef DEBUG_RESOLV
+  signal(SIGSEGV, SIG_IGN);
+#endif
+
+  /* Set g_pethers_path here, but don't actually do anything
+   * with it. It's used in get_ethbyname() and get_ethbyaddr()
+   */
+  if (g_pethers_path == NULL) {
+    g_pethers_path = g_malloc(strlen(getenv("HOME")) + 
+			      strlen(EPATH_PERSONAL_ETHERS) + 2);
+    sprintf(g_pethers_path, "%s/%s", 
+	    (char *)getenv("HOME"), EPATH_PERSONAL_ETHERS);
+  }
+
+  /* manuf hash table initialization */
+
+  set_ethent(g_manuf_path);
+
+  while ((eth = get_ethent(0))) {
+    add_manuf_name(eth->addr, eth->name);
+  }
+
+  end_ethent();
+
+} /* initialize_ethers */
+
 static hashether_t *add_eth_name(u_char *addr, u_char *name)
 {
   hashether_t *tp;
@@ -585,7 +642,7 @@ static hashether_t *add_eth_name(u_char *addr, u_char *name)
 
 } /* add_eth_name */
 
-static u_char *eth_name_lookup(u_char *addr)
+static u_char *eth_name_lookup(const u_char *addr)
 {
   hashmanuf_t *manufp;
   hashether_t *tp;
@@ -629,9 +686,12 @@ static u_char *eth_name_lookup(u_char *addr)
       sprintf(tp->name, "%s_%02x:%02x:%02x", 
 	      manufp->name, addr[3], addr[4], addr[5]);
 
+    tp->is_name_from_file = FALSE;
+
   } else {
     strncpy(tp->name, eth->name, MAXNAMELEN);
     tp->name[MAXNAMELEN-1] = '\0';
+    tp->is_name_from_file = TRUE;
   }
 
   return (tp->name);
@@ -668,32 +728,272 @@ static u_char *eth_addr_lookup(u_char *name)
 
 } /* eth_addr_lookup */
 
-static void initialize_ethers(void)
+
+/* IPXNETS */
+static int parse_ipxnets_line(char *line, ipxnet_t *ipxnet)
+{
+  /*
+   *  We allow three address separators (':', '-', and '.'),
+   *  as well as no separators
+   */
+
+  gchar		*cp;
+  guint32	a, a0, a1, a2, a3;
+  gboolean	found_single_number = FALSE;
+    
+  if ((cp = strchr(line, '#')))
+    *cp = '\0';
+  
+  if ((cp = strtok(line, " \t\n")) == NULL)
+    return -1;
+
+  /* Either fill a0,a1,a2,a3 and found_single_number is FALSE,
+   * fill a and found_single_number is TRUE,
+   * or return -1
+   */
+  if (sscanf(cp, "%x:%x:%x:%x", &a0, &a1, &a2, &a3) != 4) {
+    if (sscanf(cp, "%x-%x-%x-%x", &a0, &a1, &a2, &a3) != 4) {
+      if (sscanf(cp, "%x.%x.%x.%x", &a0, &a1, &a2, &a3) != 4) {
+        if (sscanf(cp, "%x", &a) == 1) {
+	  found_single_number = TRUE;
+  	}
+  	else {
+  	  return -1;
+	}
+      }
+    }
+  }
+
+  if ((cp = strtok(NULL, " \t\n")) == NULL)
+    return -1;
+
+  if (found_single_number) {
+	ipxnet->addr = a;
+  }
+  else {
+	ipxnet->addr = (a0 << 24) | (a1 << 16) | (a2 << 8) | a3;
+  }
+
+  strncpy(ipxnet->name, cp, MAXNAMELEN);
+  ipxnet->name[MAXNAMELEN-1] = '\0';
+
+  return 0;
+
+} /* parse_ipxnets_line */
+
+static FILE *ipxnet_p = NULL;
+
+static void set_ipxnetent(char *path)
+{
+  if (ipxnet_p)
+    rewind(ipxnet_p);
+  else
+    ipxnet_p = fopen(path, "r");
+}
+
+static void end_ipxnetent(void)
+{
+  if (ipxnet_p) {
+    fclose(ipxnet_p);
+    ipxnet_p = NULL;
+  }
+}
+
+static ipxnet_t *get_ipxnetent(void)
+{
+ 
+  static ipxnet_t ipxnet;
+  static int     size = 0;
+  static char   *buf = NULL;
+  
+  if (ipxnet_p == NULL) 
+    return NULL;
+
+  while (fgetline(&buf, &size, ipxnet_p) >= 0) {
+    if (parse_ipxnets_line(buf, &ipxnet) == 0) {
+      return &ipxnet;
+    }
+  }
+    
+  return NULL;
+
+} /* get_ipxnetent */
+
+static ipxnet_t *get_ipxnetbyname(u_char *name)
+{
+  ipxnet_t *ipxnet;
+  
+  set_ipxnetent(g_ipxnets_path);
+
+  while ((ipxnet = get_ipxnetent()) && strncmp(name, ipxnet->name, MAXNAMELEN) != 0)
+    ;
+
+  if (ipxnet == NULL) {
+    end_ipxnetent();
+    
+    set_ipxnetent(g_pipxnets_path);
+
+    while ((ipxnet = get_ipxnetent()) && strncmp(name, ipxnet->name, MAXNAMELEN) != 0)
+      ;
+
+    end_ipxnetent();
+  }
+
+  return ipxnet;
+
+} /* get_ipxnetbyname */
+
+static ipxnet_t *get_ipxnetbyaddr(guint32 addr)
+{
+
+  ipxnet_t *ipxnet;
+  
+  set_ipxnetent(g_ipxnets_path);
+
+  while ((ipxnet = get_ipxnetent()) && (addr != ipxnet->addr) ) ;
+
+  if (ipxnet == NULL) {
+    end_ipxnetent();
+    
+    set_ipxnetent(g_pipxnets_path);
+    
+    while ((ipxnet = get_ipxnetent()) && (addr != ipxnet->addr) )
+      ;
+    
+    end_ipxnetent();
+  }
+
+  return ipxnet;
+
+} /* get_ipxnetbyaddr */
+
+static void initialize_ipxnets(void)
 {
-  ether_t *eth;
 
 #ifdef DEBUG_RESOLV
   signal(SIGSEGV, SIG_IGN);
 #endif
 
-  if (g_pethers_path == NULL) {
-    g_pethers_path = g_malloc(strlen(getenv("HOME")) + 
-			      strlen(EPATH_PERSONAL_ETHERS) + 2);
-    sprintf(g_pethers_path, "%s/%s", 
-	    (char *)getenv("HOME"), EPATH_PERSONAL_ETHERS);
+  /* Set g_pipxnets_path here, but don't actually do anything
+   * with it. It's used in get_ipxnetbyname() and get_ipxnetbyaddr()
+   */
+  if (g_pipxnets_path == NULL) {
+    g_pipxnets_path = g_malloc(strlen(getenv("HOME")) + 
+			      strlen(EPATH_PERSONAL_IPXNETS) + 2);
+    sprintf(g_pipxnets_path, "%s/%s", 
+	    (char *)getenv("HOME"), EPATH_PERSONAL_IPXNETS);
   }
 
-  /* manuf hash table initialization */
+} /* initialize_ipxnets */
 
-  set_ethent(g_manuf_path);
+static hashipxnet_t *add_ipxnet_name(u_int addr, u_char *name)
+{
+  hashipxnet_t *tp;
+  hashipxnet_t **table = ipxnet_table;
 
-  while ((eth = get_ethent(0))) {
-    add_manuf_name(eth->addr, eth->name);
+  /* XXX - check goodness of hash function */
+
+  tp = table[ addr & (HASHIPXNETSIZE - 1)];
+
+  if( tp == NULL ) {
+    tp = table[ addr & (HASHIPXNETSIZE - 1)] = 
+      (hashipxnet_t *)g_malloc(sizeof(hashipxnet_t));
+  } else {  
+    while(1) {
+      if (tp->next == NULL) {
+	tp->next = (hashipxnet_t *)g_malloc(sizeof(hashipxnet_t));
+	tp = tp->next;
+	break;
+      }
+      tp = tp->next;
+    }
+  }
+  
+  tp->addr = addr;
+  strncpy(tp->name, name, MAXNAMELEN);
+  tp->name[MAXNAMELEN-1] = '\0';
+  tp->next = NULL;
+
+  return tp;
+
+} /* add_ipxnet_name */
+
+static u_char *ipxnet_name_lookup(const u_int addr)
+{
+  hashipxnet_t *tp;
+  hashipxnet_t **table = ipxnet_table;
+  ipxnet_t *ipxnet;
+
+  tp = table[ addr & (HASHIPXNETSIZE - 1)];
+
+  if( tp == NULL ) {
+    tp = table[ addr & (HASHIPXNETSIZE - 1)] = 
+      (hashipxnet_t *)g_malloc(sizeof(hashipxnet_t));
+  } else {  
+    while(1) {
+      if (tp->addr == addr) {
+	return tp->name;
+      }
+      if (tp->next == NULL) {
+	tp->next = (hashipxnet_t *)g_malloc(sizeof(hashipxnet_t));
+	tp = tp->next;
+	break;
+      }
+      tp = tp->next;
+    }
+  }
+  
+  /* fill in a new entry */
+
+  tp->addr = addr;
+  tp->next = NULL;
+
+  if ( (ipxnet = get_ipxnetbyaddr(addr)) == NULL) {
+    /* unknown name */
+      sprintf(tp->name, "%X", addr);
+
+  } else {
+    strncpy(tp->name, ipxnet->name, MAXNAMELEN);
+    tp->name[MAXNAMELEN-1] = '\0';
   }
 
-  end_ethent();
+  return (tp->name);
+
+} /* ipxnet_name_lookup */
+
+static u_int ipxnet_addr_lookup(u_char *name, gboolean *success)
+{
+  ipxnet_t *ipxnet;
+  hashipxnet_t *tp;
+  hashipxnet_t **table = ipxnet_table;
+  int i;
+
+  /* to be optimized (hash table from name to addr) */
+  for (i = 0; i < HASHIPXNETSIZE; i++) {
+    tp = table[i];
+    while (tp) {
+      if (strcmp(tp->name, name) == 0)
+	return tp->addr;
+      tp = tp->next;
+    }
+  }
+
+  /* not in hash table : performs a file lookup */
+
+  if ((ipxnet = get_ipxnetbyname(name)) == NULL) {
+	  *success = FALSE;
+	  return 0;
+  }
+
+  /* add new entry in hash table */
+
+  tp = add_ipxnet_name(ipxnet->addr, name);
+
+  *success = TRUE;
+  return tp->addr;
+
+} /* ipxnet_addr_lookup */
 
-} /* initialize_ethers */
 
 /* 
  *  External Functions
@@ -789,7 +1089,7 @@ extern u_char *get_tcp_port(u_int port)
 
 } /* get_tcp_port */
 
-extern u_char *get_ether_name(u_char *addr)
+extern u_char *get_ether_name(const u_char *addr)
 {
   if (!g_resolving_actif)
     return ether_to_str((guint8 *)addr);
@@ -803,6 +1103,67 @@ extern u_char *get_ether_name(u_char *addr)
 
 } /* get_ether_name */
 
+/* Look for an ether name in the hash, and return it if found.
+ * If it's not found, simply return NULL. We DO NOT make a new
+ * hash entry for it with the hex digits turned into a string.
+ */
+u_char *get_ether_name_if_known(const u_char *addr)
+{
+  hashether_t *tp;
+  hashether_t **table = eth_table;
+  int i,j;
+
+  /* Initialize ether structs if we're the first
+   * ether-related function called */
+  if (!g_resolving_actif)
+    return NULL;
+  
+  if (!eth_resolution_initialized) {
+    initialize_ethers();
+    eth_resolution_initialized = 1;
+  }
+
+  j = (addr[2] << 8) | addr[3];
+  i = (addr[4] << 8) | addr[5];
+
+  tp = table[ (i ^ j) & (HASHETHSIZE - 1)];
+
+  if( tp == NULL ) {
+	  /* Hash key not found in table.
+	   * Force a lookup (and a hash entry) for addr, then call
+	   * myself. I plan on not getting into an infinite loop because
+	   * eth_name_lookup() is guaranteed to make a hashtable entry,
+	   * so when I call myself again, I can never get into this
+	   * block of code again. Knock on wood...
+	   */
+	  (void) eth_name_lookup(addr);
+	  return get_ether_name_if_known(addr); /* a well-placed goto would suffice */
+  }
+  else { 
+    while(1) {
+      if (memcmp(tp->addr, addr, sizeof(tp->addr)) == 0) {
+	      if (tp->is_name_from_file) {
+		/* A name was found, and its origin is an ethers file */
+		return tp->name;
+	      }
+	      else {
+		/* A name was found, but it was created, not found in a file */
+		return NULL;
+	      }
+      }
+      if (tp->next == NULL) {
+	  /* Read my reason above for why I'm sure I can't get into an infinite loop */
+	  (void) eth_name_lookup(addr);
+	  return get_ether_name_if_known(addr); /* a well-placed goto would suffice */
+      }
+      tp = tp->next;
+    }
+  }
+  g_assert_not_reached();
+  return NULL;
+}
+
+
 extern u_char *get_ether_addr(u_char *name)
 {
 
@@ -817,6 +1178,42 @@ extern u_char *get_ether_addr(u_char *name)
 
 } /* get_ether_addr */
 
+
+extern u_char *get_ipxnet_name(const guint32 addr)
+{
+
+  if (!g_resolving_actif) {
+	  return ipxnet_to_str_punct(addr, '\0');
+  }
+
+  if (!ipxnet_resolution_initialized) {
+    initialize_ipxnets();
+    ipxnet_resolution_initialized = 1;
+  }
+
+  return ipxnet_name_lookup(addr);
+
+} /* get_ipxnet_name */
+
+extern guint32 get_ipxnet_addr(u_char *name, gboolean *known)
+{
+	guint32 addr;
+	gboolean success;
+
+  /* force resolution (do not check g_resolving_actif) */
+
+  if (!ipxnet_resolution_initialized) {
+    initialize_ipxnets();
+    ipxnet_resolution_initialized = 1;
+  }
+
+  addr =  ipxnet_addr_lookup(name, &success);
+
+  *known = success;
+  return addr;
+
+} /* get_ipxnet_addr */
+
 extern u_char *get_manuf_name(u_char *addr) 
 {
   static gchar  str[3][MAXMANUFLEN];
@@ -846,22 +1243,60 @@ extern u_char *get_manuf_name(u_char *addr)
 
 
 
-/* return IP address of either hostname or IP address in text format.
+/* Translate a string, assumed either to be a dotted-quad IP address or
+ * a host name, to a numeric IP address.  Return TRUE if we succeed and
+ * set "*addrp" to that numeric IP address; return FALSE if we fail.
  * Used more in the dfilter parser rather than in packet dissectors */
-unsigned long get_host_ipaddr(const char *host)
+gboolean get_host_ipaddr(const char *host, guint32 *addrp)
 {
-	struct hostent		*hp = NULL;
-	unsigned long		ipaddr;
+	struct in_addr		ipaddr;
+	struct hostent		*hp;
 
-	hp = gethostbyname(host);
-	if (hp == NULL) {
-		hp = gethostbyaddr(host, strlen(host), AF_INET);
+	/*
+	 * don't change it to inet_pton(AF_INET), they are not 100% compatible.
+	 * inet_pton(AF_INET) does not support hexadecimal notation nor
+	 * less-than-4 octet notation.
+	 */
+	if (!inet_aton(host, &ipaddr)) {
+		/* It's not a valid dotted-quad IP address; is it a valid
+		 * host name? */
+		hp = gethostbyname(host);
 		if (hp == NULL) {
-			return 0;
+			/* No. */
+			return FALSE;
+		} else {
+			/* XXX - is "hp->h_length" the size of a
+			 * "struct in_addr"?  It should be. */
+			memcpy(&ipaddr, hp->h_addr, hp->h_length);
 		}
 	}
 
-	memcpy(&ipaddr, hp->h_addr, hp->h_length);
+	*addrp = ntohl(ipaddr.s_addr);
+	return TRUE;
+}
 
-	return ntohl(ipaddr);
+/*
+ * Translate IPv6 numeric address or FQDN hostname, into binary IPv6 address.
+ * Return TRUE if we succeed and set "*addrp" to that numeric IP address;
+ * return FALSE if we fail.
+ */
+gboolean get_host_ipaddr6(const char *host, struct e_in6_addr *addrp)
+{
+	struct hostent *hp;
+
+	if (inet_pton(AF_INET6, host, addrp) == 1)
+		return TRUE;
+
+	/* try FQDN */
+#ifdef HAVE_GETHOSTBYNAME2
+	hp = gethostbyname2(host, AF_INET6);
+#else
+	hp = NULL;
+#endif
+	if (hp != NULL && hp->h_length == sizeof(struct e_in6_addr)) {
+		memcpy(addrp, hp->h_addr, hp->h_length);
+		return TRUE;
+	}
+
+	return FALSE;
 }
