@@ -1,6 +1,6 @@
 /* file.c
  *
- * $Id: file.c,v 1.29 1999/11/10 19:47:56 gram Exp $
+ * $Id: file.c,v 1.43 1999/12/15 01:34:17 guy Exp $
  *
  * Wiretap Library
  * Copyright (c) 1998 by Gilbert Ramirez <gram@verdict.uthscsa.edu>
@@ -43,6 +43,7 @@
 #include "netmon.h"
 #include "netxray.h"
 #include "toshiba.h"
+#include "i4btrace.h"
 
 /* The open_file_* routines should return:
  *
@@ -86,6 +87,7 @@ static int (*open_routines[])(wtap *, int *) = {
 	 */
 	ascend_open,
 	toshiba_open,
+	i4btrace_open,
 };
 
 int wtap_def_seek_read (FILE *fh, int seek_off, guint8 *pd, int len)
@@ -171,6 +173,147 @@ success:
 	return wth;
 }
 
+/* Table of the file types we know about. */
+const static struct file_type_info {
+	const char *name;
+	const char *short_name;
+	int	(*can_write_encap)(int, int);
+	int	(*dump_open)(wtap_dumper *, int *);
+} dump_open_table[WTAP_NUM_FILE_TYPES] = {
+	/* WTAP_FILE_UNKNOWN */
+	{ NULL, NULL,
+	  NULL, NULL },
+
+	/* WTAP_FILE_WTAP */
+	{ "Wiretap (Ethereal)", NULL,
+	  NULL, NULL },
+
+	/* WTAP_FILE_PCAP */
+	{ "libpcap (tcpdump, Ethereal, etc.)", "libpcap",
+	  libpcap_dump_can_write_encap, libpcap_dump_open },
+
+	/* WTAP_FILE_PCAP_MODIFIED */
+	{ "modified libpcap (tcpdump)", "modlibpcap",
+	  libpcap_dump_can_write_encap, libpcap_dump_open },
+
+	/* WTAP_FILE_PCAP_RH_6_1 */
+	{ "Red Hat Linux 6.1 libpcap (tcpdump)", "rh6_1libpcap",
+	  libpcap_dump_can_write_encap, libpcap_dump_open },
+
+	/* WTAP_FILE_LANALYZER */
+	{ "Novell LANalyzer", NULL,
+	  NULL, NULL },
+
+	/* WTAP_FILE_NGSNIFFER */
+	{ "Network Associates Sniffer (DOS-based)", "ngsniffer",
+	  ngsniffer_dump_can_write_encap, ngsniffer_dump_open },
+
+	/* WTAP_FILE_SNOOP */
+	{ "Sun snoop", "snoop",
+	  snoop_dump_can_write_encap, snoop_dump_open },
+
+	/* WTAP_FILE_IPTRACE_1_0 */
+	{ "AIX iptrace 1.0", NULL,
+	  NULL, NULL },
+
+	/* WTAP_FILE_IPTRACE_2_0 */
+	{ "AIX iptrace 2.0", NULL,
+	  NULL, NULL },
+
+	/* WTAP_FILE_NETMON_1_x */
+	{ "Microsoft Network Monitor 1.x", "netmon1",
+	  netmon_dump_can_write_encap, netmon_dump_open },
+
+	/* WTAP_FILE_NETMON_2_x */
+	{ "Microsoft Network Monitor 2.x", NULL,
+	  NULL, NULL },
+
+	/* WTAP_FILE_NETXRAY_1_0 */
+	{ "Cinco Networks NetXRay", NULL,
+	  NULL, NULL },
+
+	/* WTAP_FILE_NETXRAY_1_1 */
+	{ "Network Associates Sniffer (Windows-based) 1.1", "ngwsniffer_1_1",
+	  netxray_dump_can_write_encap, netxray_dump_open_1_1 },
+
+	/* WTAP_FILE_NETXRAY_2_001 */
+	{ "Network Associates Sniffer (Windows-based) 2.001", NULL,
+	  NULL, NULL },
+
+	/* WTAP_FILE_RADCOM */
+	{ "RADCOM WAN/LAN analyzer", NULL,
+	  NULL, NULL },
+
+	/* WTAP_FILE_ASCEND */
+	{ "Lucent/Ascend access server trace", NULL,
+	  NULL, NULL },
+
+	/* WTAP_FILE_NETTL */
+	{ "HP-UX nettl trace", NULL,
+	  NULL, NULL },
+
+	/* WTAP_FILE_TOSHIBA */
+	{ "Toshiba Compact ISDN Router snoop trace", NULL,
+	  NULL, NULL },
+
+	/* WTAP_FILE_I4BTRACE */
+	{ "I4B ISDN trace", NULL,
+	  NULL, NULL },
+
+};
+
+/* Name that should be somewhat descriptive. */
+const char *wtap_file_type_string(int filetype)
+{
+	if (filetype < 0 || filetype >= WTAP_NUM_FILE_TYPES) {
+		g_error("Unknown capture file type %d", filetype);
+		return NULL;
+	} else
+		return dump_open_table[filetype].name;
+}
+
+/* Name to use in, say, a command-line flag specifying the type. */
+const char *wtap_file_type_short_string(int filetype)
+{
+	if (filetype < 0 || filetype >= WTAP_NUM_FILE_TYPES)
+		return NULL;
+	else
+		return dump_open_table[filetype].short_name;
+}
+
+/* Translate a short name to a capture file type. */
+int wtap_short_string_to_file_type(const char *short_name)
+{
+	int filetype;
+
+	for (filetype = 0; filetype < WTAP_NUM_FILE_TYPES; filetype++) {
+		if (dump_open_table[filetype].short_name != NULL &&
+		    strcmp(short_name, dump_open_table[filetype].short_name) == 0)
+			return filetype;
+	}
+	return -1;	/* no such file type, or we can't write it */
+}
+
+gboolean wtap_dump_can_open(int filetype)
+{
+	if (filetype < 0 || filetype >= WTAP_NUM_FILE_TYPES
+	    || dump_open_table[filetype].dump_open == NULL)
+		return FALSE;
+
+	return TRUE;
+}
+
+gboolean wtap_dump_can_write_encap(int filetype, int encap)
+{
+	if (filetype < 0 || filetype >= WTAP_NUM_FILE_TYPES
+	    || dump_open_table[filetype].can_write_encap == NULL)
+		return FALSE;
+
+	if ((*dump_open_table[filetype].can_write_encap)(filetype, encap) != 0)
+		return FALSE;
+
+	return TRUE;
+}
 
 static wtap_dumper* wtap_dump_open_common(FILE *fh, int filetype,
     int encap, int snaplen, int *err);
@@ -212,6 +355,28 @@ static wtap_dumper* wtap_dump_open_common(FILE *fh, int filetype, int encap,
 {
 	wtap_dumper *wdh;
 
+	if (filetype < 0 || filetype >= WTAP_NUM_FILE_TYPES
+	    || dump_open_table[filetype].dump_open == NULL) {
+		/* Invalid type, or type we don't know how to write. */
+		*err = WTAP_ERR_UNSUPPORTED_FILE_TYPE;
+		/* NOTE: this means the FD handed to "wtap_dump_fdopen()"
+		   will be closed if we can't write that file type. */
+		fclose(fh);
+		return NULL;
+	}
+
+	/* OK, we know how to write that type; can we write the specified
+	   encapsulation type? */
+	*err = (*dump_open_table[filetype].can_write_encap)(filetype, encap);
+	if (*err != 0) {
+		/* NOTE: this means the FD handed to "wtap_dump_fdopen()"
+		   will be closed if we can't write that encapsulation type. */
+		fclose(fh);
+		return NULL;
+	}
+
+	/* OK, we can write the specified encapsulation type.  Allocate
+	   a data structure for the output stream. */
 	wdh = g_malloc(sizeof (wtap_dumper));
 	if (wdh == NULL) {
 		*err = errno;
@@ -224,25 +389,21 @@ static wtap_dumper* wtap_dump_open_common(FILE *fh, int filetype, int encap,
 	wdh->file_type = filetype;
 	wdh->snaplen = snaplen;
 	wdh->encap = encap;
+	wdh->private.opaque = NULL;
+	wdh->subtype_write = NULL;
+	wdh->subtype_close = NULL;
 
-	switch (filetype) {
-
-	case WTAP_FILE_PCAP:
-		if (!libpcap_dump_open(wdh, err))
-			goto fail;
-		break;
-
-	default:
-		/* We currently only support dumping "libpcap" files */
-		*err = WTAP_ERR_UNSUPPORTED_FILE_TYPE;
-		goto fail;
+	/* Now try to open the file for writing. */
+	if (!(*dump_open_table[filetype].dump_open)(wdh, err)) {
+		/* The attempt failed. */
+		g_free(wdh);
+		/* NOTE: this means the FD handed to "wtap_dump_fdopen()"
+		   will be closed if the open fails. */
+		fclose(fh);
+		return NULL;
 	}
-	return wdh;
 
-fail:
-	g_free(wdh);
-	fclose(fh);
-	return NULL;	/* XXX - provide a reason why we failed */
+	return wdh;	/* success! */
 }
 
 FILE* wtap_dump_file(wtap_dumper *wdh)
@@ -250,18 +411,21 @@ FILE* wtap_dump_file(wtap_dumper *wdh)
 	return wdh->fh;
 }
 
-int wtap_dump(wtap_dumper *wdh, const struct wtap_pkthdr *phdr,
+gboolean wtap_dump(wtap_dumper *wdh, const struct wtap_pkthdr *phdr,
     const u_char *pd, int *err)
 {
 	return (wdh->subtype_write)(wdh, phdr, pd, err);
 }
 
-int wtap_dump_close(wtap_dumper *wdh, int *err)
+gboolean wtap_dump_close(wtap_dumper *wdh, int *err)
 {
-	int ret = 1;
+	gboolean ret = TRUE;
 
-	if (!(wdh->subtype_close)(wdh, err))
-		ret = 0;
+	if (wdh->subtype_close != NULL) {
+		/* There's a close routine for this dump stream. */
+		if (!(wdh->subtype_close)(wdh, err))
+			ret = FALSE;
+	}
 	errno = WTAP_ERR_CANT_CLOSE;
 	if (fclose(wdh->fh) == EOF) {
 		if (ret) {
@@ -271,8 +435,10 @@ int wtap_dump_close(wtap_dumper *wdh, int *err)
 			if (err != NULL)
 				*err = errno;
 		}
-		ret = 0;
+		ret = FALSE;
 	}
+	if (wdh->private.opaque != NULL)
+		g_free(wdh->private.opaque);
 	g_free(wdh);
 	return ret;
 }
