@@ -1,6 +1,6 @@
 /* file.h
  *
- * $Id: file.h,v 1.3 1999/09/24 05:49:50 guy Exp $
+ * $Id: file.h,v 1.5 1999/10/31 17:46:07 gram Exp $
  *
  * Wiretap Library
  * Copyright (c) 1998 by Gilbert Ramirez <gram@verdict.uthscsa.edu>
@@ -33,12 +33,8 @@
 #define file_close gzclose
 #define file_tell gztell
 #define file_getc gzgetc
-static inline int file_error(void *fh) {
-    int errnum;
-    gzerror(fh, &errnum);
-    if (errnum<0) return errnum;
-    return 0;
-}
+#define file_gets(buf, len, file) gzgets((file), (buf), (len))
+extern int file_error(void *fh);
 
 #else /* No zLib */
 #define file_open fopen
@@ -47,9 +43,10 @@ static inline int file_error(void *fh) {
 #define file_read fread
 #define file_write fwrite
 #define file_close fclose
-#define file_error ferror
+extern int file_error(FILE *fh);
 #define file_tell ftell
 #define file_getc fgetc
+#define file_gets fgets
 #endif /* HAVE_LIBZ */
 
 #endif /* __FILE_H__ */
