@@ -1,6 +1,6 @@
 /* libpcap.c
  *
- * $Id: libpcap.c,v 1.23 1999/11/06 10:31:47 guy Exp $
+ * $Id: libpcap.c,v 1.28 1999/12/15 02:25:50 guy Exp $
  *
  * Wiretap Library
  * Copyright (c) 1998 by Gilbert Ramirez <gram@verdict.uthscsa.edu>
@@ -53,16 +53,6 @@
 #define	PCAP_MODIFIED_MAGIC		0xa1b2cd34
 #define	PCAP_SWAPPED_MODIFIED_MAGIC	0x34cdb2a1
 
-/* Macros to byte-swap 32-bit and 16-bit quantities. */
-#define	BSWAP32(x) \
-	((((x)&0xFF000000)>>24) | \
-	 (((x)&0x00FF0000)>>8) | \
-	 (((x)&0x0000FF00)<<8) | \
-	 (((x)&0x000000FF)<<24))
-#define	BSWAP16(x) \
-	 ((((x)&0xFF00)>>8) | \
-	  (((x)&0x00FF)<<8))
-
 /* On some systems, the FDDI MAC addresses are bit-swapped. */
 #if !defined(ultrix) && !defined(__alpha) && !defined(__bsdi__)
 #define BIT_SWAPPED_MAC_ADDRS
@@ -98,9 +88,8 @@ struct pcaprec_modified_hdr {
 
 static int libpcap_read(wtap *wth, int *err);
 static void adjust_header(wtap *wth, struct pcaprec_hdr *hdr);
-static int libpcap_dump(wtap_dumper *wdh, const struct wtap_pkthdr *phdr,
+static gboolean libpcap_dump(wtap_dumper *wdh, const struct wtap_pkthdr *phdr,
     const u_char *pd, int *err);
-static int libpcap_dump_close(wtap_dumper *wdh, int *err);
 
 /*
  * XXX - this is a bit of a mess.  OpenBSD, and perhaps NetBSD, and
@@ -346,7 +335,7 @@ int libpcap_open(wtap *wth, int *err)
 			 * if that doesn't work, it probably *is*
 			 * a corrupt file.
 			 */
-			wth->file_type = WTAP_FILE_PCAP_MODIFIED;
+			wth->file_type = WTAP_FILE_PCAP_RH_6_1;
 			wth->capture.pcap->modified = TRUE;
 		}
 
@@ -460,55 +449,76 @@ int wtap_pcap_encap_to_wtap_encap(int encap)
 	return pcap_encap[encap];
 }
 
-/* Returns 1 on success, 0 on failure; sets "*err" to an error code on
-   failure */
-int libpcap_dump_open(wtap_dumper *wdh, int *err)
+static const int wtap_encap[] = {
+	-1,		/* WTAP_ENCAP_UNKNOWN -> unsupported */
+	1,		/* WTAP_ENCAP_ETHERNET -> DLT_EN10MB */
+	6,		/* WTAP_ENCAP_TR -> DLT_IEEE802 */
+	8,		/* WTAP_ENCAP_SLIP -> DLT_SLIP */
+	9,		/* WTAP_ENCAP_PPP -> DLT_PPP */
+	10,		/* WTAP_ENCAP_FDDI -> DLT_FDDI */
+	10,		/* WTAP_ENCAP_FDDI_BITSWAPPED -> DLT_FDDI */
+	12,		/* WTAP_ENCAP_RAW_IP -> DLT_RAW */
+	7,		/* WTAP_ENCAP_ARCNET -> DLT_ARCNET */
+	11,		/* WTAP_ENCAP_ATM_RFC1483 -> DLT_ATM_RFC1483 */
+	19,		/* WTAP_ENCAP_LINUX_ATM_CLIP */
+	-1,		/* WTAP_ENCAP_LAPB -> unsupported*/
+	-1,		/* WTAP_ENCAP_ATM_SNIFFER -> unsupported */
+	0		/* WTAP_ENCAP_NULL -> DLT_NULL */
+};
+#define NUM_WTAP_ENCAPS (sizeof wtap_encap / sizeof wtap_encap[0])
+
+/* Returns 0 if we could write the specified encapsulation type,
+   an error indication otherwise. */
+int libpcap_dump_can_write_encap(int filetype, int encap)
 {
-	static const guint32 pcap_magic = PCAP_MAGIC;
-	struct pcap_hdr file_hdr;
-	static const int wtap_encap[] = {
-		-1,		/* WTAP_ENCAP_UNKNOWN -> unsupported */
-		1,		/* WTAP_ENCAP_ETHERNET -> DLT_EN10MB */
-		6,		/* WTAP_ENCAP_TR -> DLT_IEEE802 */
-		8,		/* WTAP_ENCAP_SLIP -> DLT_SLIP */
-		9,		/* WTAP_ENCAP_PPP -> DLT_PPP */
-		10,		/* WTAP_ENCAP_FDDI -> DLT_FDDI */
-		10,		/* WTAP_ENCAP_FDDI_BITSWAPPED -> DLT_FDDI */
-		12,		/* WTAP_ENCAP_RAW_IP -> DLT_RAW */
-		7,		/* WTAP_ENCAP_ARCNET -> DLT_ARCNET */
-		11,		/* WTAP_ENCAP_ATM_RFC1483 -> DLT_ATM_RFC1483 */
-		19,		/* WTAP_ENCAP_LINUX_ATM_CLIP */
-		-1,		/* WTAP_ENCAP_LAPB -> unsupported*/
-		-1,		/* WTAP_ENCAP_ATM_SNIFFER -> unsupported */
-		0		/* WTAP_ENCAP_NULL -> DLT_NULL */
-	};
-	#define NUM_WTAP_ENCAPS (sizeof wtap_encap / sizeof wtap_encap[0])
-	int nwritten;
-
 	/* Per-packet encapsulations aren't supported. */
-	if (wdh->encap == WTAP_ENCAP_PER_PACKET) {
-		*err = WTAP_ERR_ENCAP_PER_PACKET_UNSUPPORTED;
-		return 0;
-	}
+	if (encap == WTAP_ENCAP_PER_PACKET)
+		return WTAP_ERR_ENCAP_PER_PACKET_UNSUPPORTED;
 
-	if (wdh->encap < 0 || wdh->encap >= NUM_WTAP_ENCAPS
-	    || wtap_encap[wdh->encap] == -1) {
-		*err = WTAP_ERR_UNSUPPORTED_ENCAP;
-		return 0;
-	}
+	if (encap < 0 || encap >= NUM_WTAP_ENCAPS || wtap_encap[encap] == -1)
+		return WTAP_ERR_UNSUPPORTED_ENCAP;
+
+	return 0;
+}
+
+/* Returns TRUE on success, FALSE on failure; sets "*err" to an error code on
+   failure */
+gboolean libpcap_dump_open(wtap_dumper *wdh, int *err)
+{
+	guint32 magic;
+	struct pcap_hdr file_hdr;
+	int nwritten;
 
 	/* This is a libpcap file */
 	wdh->subtype_write = libpcap_dump;
-	wdh->subtype_close = libpcap_dump_close;
+	wdh->subtype_close = NULL;
 
 	/* Write the file header. */
-	nwritten = fwrite(&pcap_magic, 1, sizeof pcap_magic, wdh->fh);
-	if (nwritten != sizeof pcap_magic) {
+	switch (wdh->file_type) {
+
+	case WTAP_FILE_PCAP:
+	case WTAP_FILE_PCAP_RH_6_1:	/* modified, but with the old magic, sigh */
+		magic = PCAP_MAGIC;
+		break;
+
+	case WTAP_FILE_PCAP_MODIFIED:
+		magic = PCAP_MODIFIED_MAGIC;
+		break;
+
+	default:
+		/* We should never get here - our open routine
+		   should only get called for the types above. */
+		*err = WTAP_ERR_UNSUPPORTED_FILE_TYPE;
+		return FALSE;
+	}
+
+	nwritten = fwrite(&magic, 1, sizeof magic, wdh->fh);
+	if (nwritten != sizeof magic) {
 		if (nwritten < 0)
 			*err = errno;
 		else
 			*err = WTAP_ERR_SHORT_WRITE;
-		return 0;
+		return FALSE;
 	}
 
 	/* current "libpcap" format is 2.4 */
@@ -524,31 +534,72 @@ int libpcap_dump_open(wtap_dumper *wdh, int *err)
 			*err = errno;
 		else
 			*err = WTAP_ERR_SHORT_WRITE;
-		return 0;
+		return FALSE;
 	}
 
-	return 1;
+	return TRUE;
 }
 
 /* Write a record for a packet to a dump file.
-   Returns 1 on success, 0 on failure. */
-static int libpcap_dump(wtap_dumper *wdh, const struct wtap_pkthdr *phdr,
+   Returns TRUE on success, FALSE on failure. */
+static gboolean libpcap_dump(wtap_dumper *wdh, const struct wtap_pkthdr *phdr,
     const u_char *pd, int *err)
 {
-	struct pcaprec_hdr rec_hdr;
+	struct pcaprec_modified_hdr rec_hdr;
+	int hdr_size;
 	int nwritten;
 
-	rec_hdr.ts_sec = phdr->ts.tv_sec;
-	rec_hdr.ts_usec = phdr->ts.tv_usec;
-	rec_hdr.incl_len = phdr->caplen;
-	rec_hdr.orig_len = phdr->len;
-	nwritten = fwrite(&rec_hdr, 1, sizeof rec_hdr, wdh->fh);
-	if (nwritten != sizeof rec_hdr) {
+	rec_hdr.hdr.ts_sec = phdr->ts.tv_sec;
+	rec_hdr.hdr.ts_usec = phdr->ts.tv_usec;
+	rec_hdr.hdr.incl_len = phdr->caplen;
+	rec_hdr.hdr.orig_len = phdr->len;
+	switch (wdh->file_type) {
+
+	case WTAP_FILE_PCAP:
+		hdr_size = sizeof rec_hdr.hdr;
+		break;
+
+	case WTAP_FILE_PCAP_RH_6_1:	/* modified, but with the old magic, sigh */
+	case WTAP_FILE_PCAP_MODIFIED:
+		/* XXX - what should we supply here?
+
+		   Alexey's "libpcap" looks up the interface in the system's
+		   interface list if "ifindex" is non-zero, and prints
+		   the interface name.  It ignores "protocol", and uses
+		   "pkt_type" to tag the packet as "host", "broadcast",
+		   "multicast", "other host", "outgoing", or "none of the
+		   above", but that's it.
+
+		   If the capture we're writing isn't a modified or
+		   RH 6.1 capture, we'd have to do some work to
+		   generate the packet type and interface index - and
+		   we can't generate the interface index unless we
+		   just did the capture ourselves in any case.
+
+		   I'm inclined to continue to punt; systems other than
+		   those with the older patch can read standard "libpcap"
+		   files, and systems with the older patch, e.g. RH 6.1,
+		   will just have to live with this. */
+		rec_hdr.ifindex = 0;
+		rec_hdr.protocol = 0;
+		rec_hdr.pkt_type = 0;
+		hdr_size = sizeof rec_hdr;
+		break;
+
+	default:
+		/* We should never get here - our open routine
+		   should only get called for the types above. */
+		*err = WTAP_ERR_UNSUPPORTED_FILE_TYPE;
+		return FALSE;
+	}
+
+	nwritten = fwrite(&rec_hdr, 1, hdr_size, wdh->fh);
+	if (nwritten != hdr_size) {
 		if (nwritten < 0)
 			*err = errno;
 		else
 			*err = WTAP_ERR_SHORT_WRITE;
-		return 0;
+		return FALSE;
 	}
 	nwritten = fwrite(pd, 1, phdr->caplen, wdh->fh);
 	if (nwritten != phdr->caplen) {
@@ -556,15 +607,7 @@ static int libpcap_dump(wtap_dumper *wdh, const struct wtap_pkthdr *phdr,
 			*err = errno;
 		else
 			*err = WTAP_ERR_SHORT_WRITE;
-		return 0;
+		return FALSE;
 	}
-	return 1;
-}
-
-/* Finish writing to a dump file.
-   Returns 1 on success, 0 on failure. */
-static int libpcap_dump_close(wtap_dumper *wdh, int *err)
-{
-	/* Nothing to do here. */
-	return 1;
+	return TRUE;
 }
