@@ -1,6 +1,6 @@
 /* netmon.h
  *
- * $Id: netmon.h,v 1.2 1999/08/19 05:31:36 guy Exp $
+ * $Id: netmon.h,v 1.5 1999/12/04 09:38:38 guy Exp $
  *
  * Wiretap Library
  * Copyright (c) 1998 by Gilbert Ramirez <gram@verdict.uthscsa.edu>
@@ -22,3 +22,5 @@
  */
 
 int netmon_open(wtap *wth, int *err);
+gboolean netmon_dump_open(wtap_dumper *wdh, int *err);
+int netmon_dump_can_write_encap(int filetype, int encap);
