@@ -1,6 +1,6 @@
 /* netxray.h
  *
- * $Id: netxray.h,v 1.2 1999/08/19 05:31:36 guy Exp $
+ * $Id: netxray.h,v 1.3 1999/12/14 01:12:59 nneul Exp $
  *
  * Wiretap Library
  * Copyright (c) 1998 by Gilbert Ramirez <gram@verdict.uthscsa.edu>
@@ -22,3 +22,5 @@
  */
 
 int netxray_open(wtap *wth, int *err);
+gboolean netxray_dump_open_1_1(wtap_dumper *wdh, int *err);
+int netxray_dump_can_write_encap(int filetype, int encap);
