@@ -1,6 +1,6 @@
 /* radcom.c
  *
- * $Id: radcom.c,v 1.12 1999/09/24 05:49:52 guy Exp $
+ * $Id: radcom.c,v 1.15 1999/11/18 21:48:52 guy Exp $
  *
  * Wiretap Library
  * Copyright (c) 1998 by Gilbert Ramirez <gram@verdict.uthscsa.edu>
@@ -48,7 +48,7 @@ struct unaligned_frame_date {
 	char	usec[4];
 };
 
-static char radcom_magic[8] = {
+static guint8 radcom_magic[8] = {
 	0x42, 0xD2, 0x00, 0x34, 0x12, 0x66, 0x22, 0x88
 };
 
@@ -85,14 +85,13 @@ int radcom_open(wtap *wth, int *err)
 	errno = WTAP_ERR_CANT_READ;
 	bytes_read = file_read(magic, 1, 8, wth->fh);
 	if (bytes_read != 8) {
-		if (file_error(wth->fh)) {
-			*err = errno;
+		*err = file_error(wth->fh);
+		if (*err != 0)
 			return -1;
-		}
 		return 0;
 	}
 
-	if (memcmp(magic, radcom_magic, 8)) {
+	if (memcmp(magic, radcom_magic, 8) != 0) {
 		return 0;
 	}
 
@@ -101,10 +100,9 @@ int radcom_open(wtap *wth, int *err)
 	errno = WTAP_ERR_CANT_READ;
 	bytes_read = file_read(&byte, 1, 1, wth->fh);
 	if (bytes_read != 1) {
-		if (file_error(wth->fh)) {
-			*err = errno;
+		*err = file_error(wth->fh);
+		if (*err != 0)
 			return -1;
-		}
 		return 0;
 	}
 	wth->data_offset += 1;
@@ -112,10 +110,9 @@ int radcom_open(wtap *wth, int *err)
 		errno = WTAP_ERR_CANT_READ;
 		bytes_read = file_read(&byte, 1, 1, wth->fh);
 		if (bytes_read != 1) {
-			if (file_error(wth->fh)) {
-				*err = errno;
+			*err = file_error(wth->fh);
+			if (*err != 0)
 				return -1;
-			}
 			return 0;
 		}
 		wth->data_offset += 1;
@@ -127,10 +124,9 @@ int radcom_open(wtap *wth, int *err)
 	errno = WTAP_ERR_CANT_READ;
 	bytes_read = file_read(&start_date, 1, sizeof(struct frame_date), wth->fh);
 	if (bytes_read != sizeof(struct frame_date)) {
-		if (file_error(wth->fh)) {
-			*err = errno;
+		*err = file_error(wth->fh);
+		if (*err != 0)
 			return -1;
-		}
 		return 0;
 	}
 	wth->data_offset += sizeof(struct frame_date);
@@ -215,12 +211,12 @@ int radcom_open(wtap *wth, int *err)
 	return 1;
 
 read_error:
-	if (file_error(wth->fh)) {
-		*err = errno;
-		free(wth->capture.radcom);
+	*err = file_error(wth->fh);
+	if (*err != 0) {
+		g_free(wth->capture.radcom);
 		return -1;
 	}
-	free(wth->capture.radcom);
+	g_free(wth->capture.radcom);
 	return 0;
 }
 
@@ -239,10 +235,9 @@ static int radcom_read(wtap *wth, int *err)
 	errno = WTAP_ERR_CANT_READ;
 	bytes_read = file_read(&hdr, 1, sizeof hdr, wth->fh);
 	if (bytes_read != sizeof hdr) {
-		if (file_error(wth->fh)) {
-			*err = errno;
+		*err = file_error(wth->fh);
+		if (*err != 0)
 			return -1;
-		}
 		if (bytes_read != 0) {
 			*err = WTAP_ERR_SHORT_READ;
 			return -1;
@@ -281,9 +276,8 @@ static int radcom_read(wtap *wth, int *err)
 			length, wth->fh);
 
 	if (bytes_read != length) {
-		if (file_error(wth->fh))
-			*err = errno;
-		else
+		*err = file_error(wth->fh);
+		if (*err == 0)
 			*err = WTAP_ERR_SHORT_READ;
 		return -1;
 	}
@@ -297,9 +291,8 @@ static int radcom_read(wtap *wth, int *err)
 		errno = WTAP_ERR_CANT_READ;
 		bytes_read = file_read(&fcs, 1, sizeof fcs, wth->fh);
 		if (bytes_read != sizeof fcs) {
-			if (file_error(wth->fh))
-				*err = errno;
-			else
+			*err = file_error(wth->fh);
+			if (*err == 0)
 				*err = WTAP_ERR_SHORT_READ;
 			return -1;
 		}
