@@ -1,6 +1,6 @@
 /* snoop.h
  *
- * $Id: snoop.h,v 1.2 1999/08/19 05:31:35 guy Exp $
+ * $Id: snoop.h,v 1.6 1999/12/04 09:38:38 guy Exp $
  *
  * Wiretap Library
  * Copyright (c) 1998 by Gilbert Ramirez <gram@verdict.uthscsa.edu>
@@ -22,3 +22,5 @@
  */
 
 int snoop_open(wtap *wth, int *err);
+gboolean snoop_dump_open(wtap_dumper *wdh, int *err);
+int snoop_dump_can_write_encap(int filetype, int encap);
