@@ -1,6 +1,6 @@
 /* wtap.c
  *
- * $Id: wtap.c,v 1.31 1999/11/26 17:57:14 gram Exp $
+ * $Id: wtap.c,v 1.35 1999/12/12 22:53:25 guy Exp $
  *
  * Wiretap Library
  * Copyright (c) 1998 by Gilbert Ramirez <gram@verdict.uthscsa.edu>
@@ -57,64 +57,92 @@ int wtap_file_encap(wtap *wth)
 	return wth->file_encap;
 }
 
-const char *wtap_file_type_string(wtap *wth)
+/* Table of the encapsulation types we know about. */
+const static struct encap_type_info {
+	const char *name;
+	const char *short_name;
+} encap_table[WTAP_NUM_ENCAP_TYPES] = {
+	/* WTAP_ENCAP_UNKNOWN */
+	{ "Unknown", NULL },
+
+	/* WTAP_ENCAP_ETHERNET */
+	{ "Ethernet", "ether" },
+
+	/* WTAP_ENCAP_TR */
+	{ "Token Ring", "tr" },
+
+	/* WTAP_ENCAP_SLIP */
+	{ "SLIP", "slip" },
+
+	/* WTAP_ENCAP_PPP */
+	{ "PPP", "ppp" },
+
+	/* WTAP_ENCAP_FDDI */
+	{ "FDDI", "fddi" },
+
+	/* WTAP_ENCAP_FDDI_BITSWAPPED */
+	{ "FDDI with bit-swapped MAC addresses", "fddi-swapped" },
+
+	/* WTAP_ENCAP_RAW_IP */
+	{ "Raw IP", "rawip" },
+
+	/* WTAP_ENCAP_ARCNET */
+	{ "ARCNET", "arcnet" },
+
+	/* WTAP_ENCAP_ATM_RFC1483 */
+	{ "RFC 1483 ATM", "atm-rfc1483" },
+
+	/* WTAP_ENCAP_LINUX_ATM_CLIP */
+	{ "Linux ATM CLIP", "linux-atm-clip" },
+
+	/* WTAP_ENCAP_LAPB */
+	{ "LAPB", "lapb" },
+
+	/* WTAP_ENCAP_ATM_SNIFFER */
+	{ "ATM Sniffer", "atm-sniffer" },
+
+	/* WTAP_ENCAP_NULL */
+	{ "NULL", "null" },
+
+	/* WTAP_ENCAP_ASCEND */
+	{ "Lucent/Ascend access equipment", "ascend" },
+
+	/* WTAP_ENCAP_LAPD */
+	{ "LAPD", "lapd" },
+
+	/* WTAP_ENCAP_V120 */
+	{ "V.120", "v120" },
+};
+
+/* Name that should be somewhat descriptive. */
+const char *wtap_encap_string(int encap)
 {
-	switch (wth->file_type) {
-		case WTAP_FILE_WTAP:
-			return "wiretap";
+	if (encap < 0 || encap >= WTAP_NUM_ENCAP_TYPES)
+		return NULL;
+	else
+		return encap_table[encap].name;
+}
 
-		case WTAP_FILE_PCAP:
-			return "pcap";
+/* Name to use in, say, a command-line flag specifying the type. */
+const char *wtap_encap_short_string(int encap)
+{
+	if (encap < 0 || encap >= WTAP_NUM_ENCAP_TYPES)
+		return NULL;
+	else
+		return encap_table[encap].short_name;
+}
 
-		case WTAP_FILE_PCAP_MODIFIED:
-			return "pcap-modified";
+/* Translate a short name to a capture file type. */
+int wtap_short_string_to_encap(const char *short_name)
+{
+	int encap;
 
-		case WTAP_FILE_LANALYZER:
-			return "Novell LANalyzer";
-
-		case WTAP_FILE_NGSNIFFER:
-			return "Network Associates Sniffer (DOS-based)";
-
-		case WTAP_FILE_SNOOP:
-			return "snoop";
-
-		case WTAP_FILE_IPTRACE_1_0:
-			return "AIX iptrace 1.0";
-
-		case WTAP_FILE_IPTRACE_2_0:
-			return "AIX iptrace 2.0";
-
-		case WTAP_FILE_NETMON_1_x:
-			return "Microsoft Network Monitor 1.x";
-
-		case WTAP_FILE_NETMON_2_x:
-			return "Microsoft Network Monitor 2.x";
-
-		case WTAP_FILE_NETXRAY_1_0:
-			return "Cinco Networks NetXRay";
-
-		case WTAP_FILE_NETXRAY_1_1:
-			return "Network Associates Sniffer (Windows-based) 1.1";
-
-		case WTAP_FILE_NETXRAY_2_001:
-			return "Network Associates Sniffer (Windows-based) 2.001";
-
-		case WTAP_FILE_RADCOM:
-			return "RADCOM WAN/LAN analyzer";
-
-		case WTAP_FILE_ASCEND:
-			return "Lucent/Ascend access server trace";
-
-		case WTAP_FILE_NETTL:
-			return "HP-UX nettl trace";
-
-		case WTAP_FILE_TOSHIBA:
-			return "Toshiba Compact ISDN Router snoop trace";
-
-		default:
-			g_error("Unknown capture file type %d", wth->file_type);
-			return NULL;
+	for (encap = 0; encap < WTAP_NUM_ENCAP_TYPES; encap++) {
+		if (encap_table[encap].short_name != NULL &&
+		    strcmp(short_name, encap_table[encap].short_name) == 0)
+			return encap;
 	}
+	return -1;	/* no such encapsulation type */
 }
 
 static const char *wtap_errlist[] = {
