@@ -1,6 +1,6 @@
 /* wtap.h
  *
- * $Id: wtap.h,v 1.50 1999/11/26 17:57:14 gram Exp $
+ * $Id: wtap.h,v 1.62 1999/12/15 02:25:50 guy Exp $
  *
  * Wiretap Library
  * Copyright (c) 1998 by Gilbert Ramirez <gram@verdict.uthscsa.edu>
@@ -92,9 +92,10 @@
 #define WTAP_ENCAP_NULL				13
 #define WTAP_ENCAP_ASCEND			14
 #define WTAP_ENCAP_LAPD				15
+#define WTAP_ENCAP_V120				16
 
 /* last WTAP_ENCAP_ value + 1 */
-#define WTAP_NUM_ENCAP_TYPES			16
+#define WTAP_NUM_ENCAP_TYPES			17
 
 /* File types that can be read by wiretap.
    We may eventually support writing some or all of these file types,
@@ -103,20 +104,25 @@
 #define WTAP_FILE_WTAP				1
 #define WTAP_FILE_PCAP				2
 #define WTAP_FILE_PCAP_MODIFIED			3
-#define WTAP_FILE_LANALYZER			4
-#define WTAP_FILE_NGSNIFFER			5
-#define WTAP_FILE_SNOOP				6
-#define WTAP_FILE_IPTRACE_1_0			7
-#define WTAP_FILE_IPTRACE_2_0			8
-#define WTAP_FILE_NETMON_1_x			9
-#define WTAP_FILE_NETMON_2_x			10
-#define WTAP_FILE_NETXRAY_1_0			11
-#define WTAP_FILE_NETXRAY_1_1			12
-#define WTAP_FILE_NETXRAY_2_001			13
-#define WTAP_FILE_RADCOM			14
-#define WTAP_FILE_ASCEND			15
-#define WTAP_FILE_NETTL				16
-#define WTAP_FILE_TOSHIBA			17
+#define WTAP_FILE_PCAP_RH_6_1			4
+#define WTAP_FILE_LANALYZER			5
+#define WTAP_FILE_NGSNIFFER			6
+#define WTAP_FILE_SNOOP				7
+#define WTAP_FILE_IPTRACE_1_0			8
+#define WTAP_FILE_IPTRACE_2_0			9
+#define WTAP_FILE_NETMON_1_x			10
+#define WTAP_FILE_NETMON_2_x			11
+#define WTAP_FILE_NETXRAY_1_0			12
+#define WTAP_FILE_NETXRAY_1_1			13
+#define WTAP_FILE_NETXRAY_2_001			14
+#define WTAP_FILE_RADCOM			15
+#define WTAP_FILE_ASCEND			16
+#define WTAP_FILE_NETTL				17
+#define WTAP_FILE_TOSHIBA			18
+#define WTAP_FILE_I4BTRACE			19
+
+/* last WTAP_FILE_ value + 1 */
+#define WTAP_NUM_FILE_TYPES			20
 
 /*
  * Maximum packet size we'll support.
@@ -153,6 +159,12 @@ typedef struct {
 	time_t	start;
 } radcom_t;
 
+typedef struct {
+	gboolean byte_swapped;
+	time_t	start;
+	int bchannel_prot[2];	/* For the V.120 heuristic */
+} i4btrace_t;
+
 typedef struct {
 	time_t	start;
 } nettl_t;
@@ -172,6 +184,8 @@ typedef struct {
 	time_t	start_secs;
 	guint32	start_usecs;
 	guint8	version_major;
+	guint32 *frame_table;
+	int	current_frame;
 	int	end_offset;
 } netmon_t;
 
@@ -320,6 +334,7 @@ typedef struct wtap {
 		lanalyzer_t		*lanalyzer;
 		ngsniffer_t		*ngsniffer;
 		radcom_t		*radcom;
+		i4btrace_t		*i4btrace;
 		nettl_t			*nettl;
 		netmon_t		*netmon;
 		netxray_t		*netxray;
@@ -335,15 +350,43 @@ typedef struct wtap {
 
 struct wtap_dumper;
 
-typedef int (*subtype_write_func)(struct wtap_dumper*,
+typedef gboolean (*subtype_write_func)(struct wtap_dumper*,
 		const struct wtap_pkthdr*, const u_char*, int*);
-typedef int (*subtype_close_func)(struct wtap_dumper*, int*);
+typedef gboolean (*subtype_close_func)(struct wtap_dumper*, int*);
+
+typedef struct {
+	gboolean first_frame;
+	time_t start;
+} ngsniffer_dump_t;
+
+typedef struct {
+	gboolean first_frame;
+	struct timeval start;
+	guint32	nframes;
+} netxray_dump_t;
+
+typedef struct {
+	gboolean got_first_record_time;
+	struct timeval first_record_time;
+	guint32	frame_table_offset;
+	guint32	*frame_table;
+	int	frame_table_index;
+	int	frame_table_size;
+} netmon_dump_t;
+
 typedef struct wtap_dumper {
 	FILE*			fh;
 	int			file_type;
 	int			snaplen;
 	int			encap;
 
+	union {
+		void			*opaque;
+		ngsniffer_dump_t	*ngsniffer;
+		netmon_dump_t		*netmon;
+		netxray_dump_t		*netxray;
+	} private;
+
 	subtype_write_func	subtype_write;
 	subtype_close_func	subtype_close;
 } wtap_dumper;
@@ -364,20 +407,30 @@ int wtap_fd(wtap *wth);
 int wtap_snapshot_length(wtap *wth); /* per file */
 int wtap_file_type(wtap *wth);
 int wtap_file_encap(wtap *wth);
-const char *wtap_file_type_string(wtap *wth);
+
+const char *wtap_file_type_string(int filetype);
+const char *wtap_file_type_short_string(int filetype);
+int wtap_short_string_to_file_type(const char *short_name);
+
+const char *wtap_encap_string(int encap);
+const char *wtap_encap_short_string(int encap);
+int wtap_short_string_to_encap(const char *short_name);
+
 const char *wtap_strerror(int err);
 void wtap_close(wtap *wth);
 int wtap_seek_read (int file_type, FILE *fh, int seek_off, guint8 *pd, int len);
 int wtap_def_seek_read (FILE *fh, int seek_off, guint8 *pd, int len);
 
+gboolean wtap_dump_can_open(int filetype);
+gboolean wtap_dump_can_write_encap(int filetype, int encap);
 wtap_dumper* wtap_dump_open(const char *filename, int filetype, int encap,
 	int snaplen, int *err);
 wtap_dumper* wtap_dump_fdopen(int fd, int filetype, int encap, int snaplen,
 	int *err);
-int wtap_dump(wtap_dumper *, const struct wtap_pkthdr *, const u_char *,
+gboolean wtap_dump(wtap_dumper *, const struct wtap_pkthdr *, const u_char *,
 	int *err);
 FILE* wtap_dump_file(wtap_dumper *);
-int wtap_dump_close(wtap_dumper *, int *);
+gboolean wtap_dump_close(wtap_dumper *, int *);
 
 /* XXX - needed until "wiretap" can do live packet captures */
 int wtap_pcap_encap_to_wtap_encap(int encap);
@@ -423,6 +476,31 @@ int wtap_pcap_encap_to_wtap_encap(int encap);
 #define	WTAP_ERR_ZLIB_MAX			-100
 #define	WTAP_ERR_ZLIB_MIN			-300
 
+/* Macros to byte-swap 32-bit and 16-bit quantities. */
+#define	BSWAP32(x) \
+	((((x)&0xFF000000)>>24) | \
+	 (((x)&0x00FF0000)>>8) | \
+	 (((x)&0x0000FF00)<<8) | \
+	 (((x)&0x000000FF)<<24))
+#define	BSWAP16(x) \
+	 ((((x)&0xFF00)>>8) | \
+	  (((x)&0x00FF)<<8))
+
+/* Turn host-byte-order values into little-endian values. */
+#ifdef WORDS_BIGENDIAN
+#define htoles(s) ((guint16)                       \
+                    ((guint16)((s) & 0x00FF)<<8|  \
+                     (guint16)((s) & 0xFF00)>>8))
+
+#define htolel(l) ((guint32)((l) & 0x000000FF)<<24|  \
+                   (guint32)((l) & 0x0000FF00)<<8|  \
+                   (guint32)((l) & 0x00FF0000)>>8|   \
+                   (guint32)((l) & 0xFF000000)>>24)
+#else
+#define htoles(s)	(s)
+#define htolel(l)	(l)
+#endif
+
 /* Pointer versions of ntohs and ntohl.  Given a pointer to a member of a
  * byte array, returns the value of the two or four bytes at the pointer.
  * The pletoh[sl] versions return the little-endian representation.
