@@ -2,7 +2,7 @@
  * Routines for use by various SDLC-derived protocols, such as HDLC
  * and its derivatives LAPB, IEEE 802.2 LLC, etc..
  *
- * $Id: xdlc.c,v 1.7 1999/08/27 18:02:41 guy Exp $
+ * $Id: xdlc.c,v 1.10 1999/11/16 11:43:07 guy Exp $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@unicom.net>
@@ -66,31 +66,6 @@
 #define XDLC_REJ		0x08	/* Reject */
 #define XDLC_SREJ		0x0C	/* Selective reject */
 
-/*
- * U-format modifiers.
- */
-#define XDLC_U_MODIFIER_MASK	0xEC
-#define XDLC_UI		0x00	/* Unnumbered Information */
-#define XDLC_UP		0x20	/* Unnumbered Poll */
-#define XDLC_DISC	0x40	/* Disconnect (command) */
-#define XDLC_RD		0x40	/* Request Disconnect (response) */
-#define XDLC_UA		0x60	/* Unnumbered Acknowledge */
-#define XDLC_SNRM	0x80	/* Set Normal Response Mode */
-#define XDLC_TEST	0xE0	/* Test */
-#define XDLC_SIM	0x04	/* Set Initialization Mode (command) */
-#define XDLC_RIM	0x04	/* Request Initialization Mode (response) */
-#define XDLC_FRMR	0x84	/* Frame reject */
-#define XDLC_CFGR	0xC4	/* Configure */
-#define XDLC_SARM	0x0C	/* Set Asynchronous Response Mode (command) */
-#define XDLC_DM		0x0C	/* Disconnected mode (response) */
-#define XDLC_SABM	0x2C	/* Set Asynchronous Balanced Mode */
-#define XDLC_SARME	0x4C	/* Set Asynchronous Response Mode Extended */
-#define XDLC_SABME	0x6C	/* Set Asynchronous Balanced Mode Extended */
-#define XDLC_RESET	0x8C	/* Reset */
-#define XDLC_XID	0xAC	/* Exchange identification */
-#define XDLC_SNRME	0xCC	/* Set Normal Response Mode Extended */
-#define XDLC_BCN	0xEC	/* Beacon */
-
 static const value_string stype_vals[] = {
     { XDLC_RR,   "Receiver ready" },
     { XDLC_RNR,  "Receiver not ready" },
@@ -192,10 +167,15 @@ get_xdlc_control(const u_char *pd, int offset, int is_response, int is_extended)
     switch (pd[offset] & 0x03) {
 
     case XDLC_S:
+    default:
         /*
-	 * Supervisory frame - no higher-layer payload.
+	 * Supervisory or Information frame.
 	 */
-	return FALSE;
+	if (is_extended)
+		control = pletohs(&pd[offset]);
+	else
+		control = pd[offset];
+	break;
 
     case XDLC_U:
 	/*
@@ -209,23 +189,14 @@ get_xdlc_control(const u_char *pd, int offset, int is_response, int is_extended)
 	 * need for it to be 2 bytes in extended operation.
 	 */
 	control = pd[offset];
-
-	/*
-	 * This frame has payload only if it's a UI frame.
-	 */
-	return (control & XDLC_U_MODIFIER_MASK) == XDLC_UI;
-
-    default:
-	/*
-	 * Information frame - has higher-layer payload.
-	 */
-	return TRUE;
+	break;
     }
+    return control;
 }
 
 int
 dissect_xdlc_control(const u_char *pd, int offset, frame_data *fd,
-  proto_tree *xdlc_tree, int hf_xdlc_control, 
+  proto_tree *xdlc_tree, int hf_xdlc_control, gint ett_xdlc_control,
   int is_response, int is_extended)
 {
     guint16 control;
@@ -282,7 +253,7 @@ dissect_xdlc_control(const u_char *pd, int offset, frame_data *fd,
 			offset, 2,
 			frame_type,
 			"Control field: %s (0x%04X)", info, control);
-		control_tree = proto_item_add_subtree(tc, ETT_XDLC_CONTROL);
+		control_tree = proto_item_add_subtree(tc, ett_xdlc_control);
 		proto_tree_add_text(control_tree, offset, 2,
 		    decode_numeric_bitfield(control, XDLC_N_R_EXT_MASK, 2*8,
 			"N(R) = %u"));
@@ -303,7 +274,7 @@ dissect_xdlc_control(const u_char *pd, int offset, frame_data *fd,
 			offset, 1,
 			frame_type,
 			"Control field: %s (0x%02X)", info, control);
-		control_tree = proto_item_add_subtree(tc, ETT_XDLC_CONTROL);
+		control_tree = proto_item_add_subtree(tc, ett_xdlc_control);
 		proto_tree_add_text(control_tree, offset, 1,
 		    decode_numeric_bitfield(control, XDLC_N_R_MASK, 1*8,
 			"N(R) = %u"));
@@ -321,11 +292,7 @@ dissect_xdlc_control(const u_char *pd, int offset, frame_data *fd,
 			"Supervisory frame", NULL));
 	    }
 	}
-
-	/*
-	 * Supervisory frames have no higher-layer payload to be analyzed.
-	 */
-	return FALSE;
+	break;
 
     case XDLC_U:
 	/*
@@ -360,7 +327,7 @@ dissect_xdlc_control(const u_char *pd, int offset, frame_data *fd,
 			offset, 1,
 			frame_type,
 			"Control field: %s (0x%02X)", info, control);
-	    control_tree = proto_item_add_subtree(tc, ETT_XDLC_CONTROL);
+	    control_tree = proto_item_add_subtree(tc, ett_xdlc_control);
 	    if (control & XDLC_P_F) {
 		proto_tree_add_text(control_tree, offset, 2,
 		    decode_boolean_bitfield(control, XDLC_P_F, 1*8,
@@ -375,17 +342,16 @@ dissect_xdlc_control(const u_char *pd, int offset, frame_data *fd,
 		decode_boolean_bitfield(control, 0x03, 1*8,
 		    "Unnumbered frame", NULL));
 	}
-
-	/*
-	 * This frame has payload only if it's a UI frame.
-	 */
-	return (control & XDLC_U_MODIFIER_MASK) == XDLC_UI;
+	break;
 
     default:
 	/*
 	 * Information frame.
 	 */
-	control = pd[offset];
+	if (is_extended)
+		control = pletohs(&pd[offset]);
+	else
+		control = pd[offset];
 	if (is_extended) {
 	    sprintf(info, "I%s, N(R) = %u, N(S) = %u",
 			((control & XDLC_P_F_EXT) ? " P" : ""),
@@ -406,7 +372,7 @@ dissect_xdlc_control(const u_char *pd, int offset, frame_data *fd,
 			(is_extended) ? "Control field: %s (0x%04X)"
 			              : "Control field: %s (0x%02X)",
 			info, control);
-	    control_tree = proto_item_add_subtree(tc, ETT_XDLC_CONTROL);
+	    control_tree = proto_item_add_subtree(tc, ett_xdlc_control);
 	    if (is_extended) {
 		proto_tree_add_text(control_tree, offset, 2,
 		    decode_numeric_bitfield(control, XDLC_N_R_EXT_MASK, 2*8,
@@ -441,10 +407,6 @@ dissect_xdlc_control(const u_char *pd, int offset, frame_data *fd,
 			NULL, "Information frame"));
 	    }
 	}
-
-	/*
-	 * Information frames have higher-layer payload to be analyzed.
-	 */
-	return TRUE;
     }
+    return control;
 }
